import { Duck } from '../..';
/**
 * @class PhysicsList
 * @classdesc Creates a PhysicsList that manages Physics objects
 * @description The PhysicsList Class. Keeps track of all Physics objects
 * @since 2.0.0
 */
export default class PhysicsList {
    list: Duck.Types.PhysicsProcessMember[];
    /**
     * @memberof PhysicsList
     * @description Adds a physicsObject to the list
     * @param {Duck.Types.PhysicsProcessMember} physicsObject A physicsObject
     * @since 2.0.0
     */
    add(physicsObject: Duck.Types.PhysicsProcessMember): void;
    /**
     * @memberof PhysicsList
     * @description Removes a physicsObject from the list
     * @param {Duck.Types.PhysicsProcessMember} physicsObject A physicsObject
     * @since 2.0.0
     */
    remove(physicsObject: Duck.Types.PhysicsProcessMember): void;
    /**
     * @memberof PhysicsList
     * @template t
     * @description Loops through each physicsObject and runs a callback with it passed
     * @param {(physicsObject: Duck.Types.PhysicsProcessMember, index: number) => t} cb Callback
     * @since 2.0.0
     */
    each<t = void>(cb: (physicsBody: Duck.Types.PhysicsProcessMember, index: number) => t): void;
    enabledFilter(filter?: boolean): Duck.Types.PhysicsProcessMember[];
    /**
     * @memberof PhysicsList
     * @description Removes an item from the list based on passed index
     * @param {number} index
     * @param {number} [amount]
     * @since 2.0.0
     */
    splice(index: number, amount?: number): Duck.Types.PhysicsProcessMember[];
    /**
     * @memberof PhysicsList
     * @description Gets the length of the list
     * @since 2.0.0
     */
    get length(): number;
    /**
     * @memberof PhysicsList
     * @description Overwrites the list items with passed parameter
     * @param {Duck.Types.PhysicsProcessMember[]} listItems Renderable objects to overwrite with
     * @since 2.0.0
     */
    set listItems(listItems: Duck.Types.PhysicsProcessMember[]);
}
