/*!
 * 
 * DuckEngine 2.1.0
 *
 * https://github.com/ksplatdev/DuckEngine
 *
 * MIT License
 *
 * Copyright (c) 2021 Bleart Emini
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
/******/ // The require scope
/******/ var __webpack_require__ = {};
/******/ 
/************************************************************************/
/******/ /* webpack/runtime/define property getters */
/******/ (() => {
/******/ 	// define getter functions for harmony exports
/******/ 	__webpack_require__.d = (exports, definition) => {
/******/ 		for(var key in definition) {
/******/ 			if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 				Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 			}
/******/ 		}
/******/ 	};
/******/ })();
/******/ 
/******/ /* webpack/runtime/hasOwnProperty shorthand */
/******/ (() => {
/******/ 	__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ })();
/******/ 
/************************************************************************/
var __webpack_exports__ = {};

// EXPORTS
__webpack_require__.d(__webpack_exports__, {
  "_": () => (/* binding */ Duck),
  "Z": () => (/* binding */ lib)
});

;// CONCATENATED MODULE: ./lib/core/debug/debug.js
const __debugStack_duckengine = [];
class Error {
    constructor(message) {
        console.error('DuckEngine Error : ' + message);
        __debugStack_duckengine.push(message);
    }
}
class Warn {
    constructor(message) {
        console.warn('DuckEngine Warning : ' + message);
        __debugStack_duckengine.push(message);
    }
}
class Log {
    constructor(message) {
        console.log('DuckEngine : ' + message);
        __debugStack_duckengine.push(message);
    }
}
const Debug = {
    stack: () => {
        return __debugStack_duckengine;
    },
    Error: Error,
    Warn: Warn,
    Log: Log,
};
/* harmony default export */ const debug = (Debug);

;// CONCATENATED MODULE: ./lib/core/math/clamp.js
/**
 * @function
 * @description Clamps a value based on min and max
 * @param {number} x Number
 * @param {number} min Minimum
 * @param {number} max Maximum
 * @since 1.0.0-beta
 */
function clamp(x, min, max) {
    if (x < min)
        x = min;
    else if (x > max)
        x = max;
    return x;
}

;// CONCATENATED MODULE: ./lib/utils/degToRadians.js
function degToRadians(degrees) {
    return (degrees * Math.PI) / 180;
}

;// CONCATENATED MODULE: ./lib/core/math/vector2.js


/**
 * @class Vector2
 * @classdesc Creates a Vector2
 * @description The Vector2 Class. Represents a point
 * @since 2.0.0
 */
class Vector2 {
    /**
     * @constructor Vector2
     * @description Creates a Vector2 instance
     * @param {number} [x=0] X position, optional -> defaults: 0
     * @param {number} [y=0] Y position, optional -> defaults: 0
     * @since 2.0.0
     */
    constructor(x = 0, y = 0) {
        this.x = x;
        this.y = y;
    }
    /**
     * @memberof Vector2
     * @description Sets the values of the Vector2
     * @param {number} x X position to setValue of
     * @param {number} y Y position to setValue of
     * @returns {Vector2}
     * @since 2.0.0
     */
    setValues(x, y) {
        this.x = x;
        this.y = y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Sets the values of the Vector2 based on another Vector2
     * @param {number} vector Vector2 to use to set the position
     * @returns {Vector2}
     * @since 2.0.0
     */
    setValuesVec(vector) {
        this.x = vector.x;
        this.y = vector.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Adds a Vector2
     * @param {Vector2} vector Vector2 to be added to
     * @returns {Vector2}
     * @since 2.0.0
     */
    add(vector) {
        this.x += vector.x;
        this.y += vector.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Adds a number to the x and y properties
     * @param {number} number Number to add to x and y properties
     * @returns {Vector2}
     * @since 2.0.0
     */
    addNumber(number) {
        this.x += number;
        this.y += number;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Subtracts a Vector2
     * @param {Vector2} vector Vector2 to be subtracted to
     * @returns {Vector2}
     * @since 2.0.0
     */
    subtract(vector) {
        this.x -= vector.x;
        this.y -= vector.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Subtracts a number from the x and y properties
     * @param {number} number Number to subtract from x and y properties
     * @returns {Vector2}
     * @since 2.0.0
     */
    subtractNumber(number) {
        this.x -= number;
        this.y -= number;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Multiplies a Vector2
     * @param {Vector2} vector Vector2 to be multiplied to
     * @returns {Vector2}
     * @since 2.0.0
     */
    multiply(vector) {
        this.x *= vector.x;
        this.y *= vector.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Multiplies a number to the x and y properties
     * @param {number} number Number to multiply to x and y properties
     * @returns {Vector2}
     * @since 2.0.0
     */
    multiplyNumber(number) {
        this.x *= number;
        this.y *= number;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Divides a Vector2
     * @param {Vector2} vector Vector2 to be divided to
     * @returns {Vector2}
     * @since 2.0.0
     */
    divide(vector) {
        this.x /= vector.x;
        this.y /= vector.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Divides a number to the x and y properties
     * @param {number} number Number to divide to x and y properties
     * @returns {Vector2}
     * @since 2.0.0
     */
    divideNumber(number) {
        this.x /= number;
        this.y /= number;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Rounds the Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    round() {
        this.x = Math.round(this.x);
        this.y = Math.round(this.y);
        return this;
    }
    /**
     * @memberof Vector2
     * @description Gets the angle between two Vector2s
     * @param {Vector2} vector A Vector2 to get the angle between from
     * @returns {number}
     * @since 2.0.0
     */
    angleBetween(vector) {
        return Math.atan2(this.x * vector.y - this.y * vector.x, this.x * vector.x + this.y * vector.y);
    }
    /**
     * @memberof Vector2
     * @description Gets the angle to two Vector2s
     * @param {Vector2} vector A Vector2 to get the angle to from
     * @returns {number}
     * @since 2.0.0
     */
    angleTo(vector) {
        return Math.atan2(vector.y - this.y, vector.x - this.x);
    }
    /**
     * @memberof Vector2
     * @description Clones the current Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    clone() {
        return new Vector2(this.x, this.y);
    }
    /**
     * @memberof Vector2
     * @description Gets the distance from another Vector2
     * @param {Vector2} vector A Vector2 to get the distance from
     * @returns {number}
     * @since 2.0.0
     */
    distance(vector) {
        return Math.sqrt((vector.x - this.x) * (vector.x - this.x) +
            (vector.y - this.y) * (vector.y - this.y));
    }
    /**
     * @memberof Vector2
     * @description Gets the distance squared from another Vector2
     * @param {Vector2} vector A Vector2 to get the distance from
     * @returns {number}
     * @since 2.0.0
     */
    distanceSqr(vector) {
        return ((vector.x - this.x) * (vector.x - this.x) +
            (vector.y - this.y) * (vector.y - this.y));
    }
    /**
     * @memberof Vector2
     * @description Gets the dot product with another Vector2
     * @param {Vector2} vector A Vector2 to get the dot product from
     * @returns {number}
     * @since 2.0.0
     */
    dot(vector) {
        return this.x * vector.x + this.y * vector.y;
    }
    /**
     * @memberof Vector2
     * @description Gets the cross dot product with another Vector2
     * @param {Vector2} vector A Vector2 to get the cross dot product from
     * @returns {number}
     * @since 2.0.0
     */
    crossProduct(vector) {
        return this.x * vector.y - this.y * vector.x;
    }
    /**
     * @memberof Vector2
     * @description Checks if another Vector2 is equal on both axises
     * @param {Vector2} vector A Vector2 to compare with
     * @returns {boolean}
     * @since 2.0.0
     */
    equals(vector) {
        return this.x === vector.x && this.y === vector.y;
    }
    /**
     * @memberof Vector2
     * @description Gets the perpendicular values of the Vector2
     * @param {Vector2} [resultVector] The new Vector2 to save the value to, optional -> defaults: new Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    perpendicular(resultVector) {
        resultVector = resultVector || new Vector2();
        return resultVector.setValues(-this.y, this.x);
    }
    /**
     * @memberof Vector2
     * @description Gradually interpolates the Vector2 towards another Vector2 by an amount
     * @param {Vector2} current The Current Vector
     * @param {Vector2} target The Target Vector2
     * @param {number} maxDistanceDelta The amount to increase by
     * @returns {Vector2}
     * @since 2.0.0
     */
    moveTowards(current, target, maxDistanceDelta) {
        const toVector_x = target.x - current.x;
        const toVector_y = target.y - current.y;
        const sqDist = toVector_x * toVector_x + toVector_y * toVector_y;
        if (sqDist === 0 ||
            (maxDistanceDelta >= 0 &&
                sqDist <= maxDistanceDelta * maxDistanceDelta)) {
            return target;
        }
        const dist = Math.sqrt(sqDist);
        const newX = current.x + (toVector_x / dist) * maxDistanceDelta;
        const newY = current.y + (toVector_y / dist) * maxDistanceDelta;
        return new Vector2(newX, newY);
    }
    /**
     * @memberof Vector2
     * @description Normalizes the Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    normalize() {
        const mag = this.magnitude();
        if (mag > 0) {
            this.divideNumber(mag);
        }
        return this;
    }
    /**
     * @memberof Vector2
     * @description Gets the normal value of the Vector2 and another Vector2
     * @param
     * @param {Vector2} [resultVector] The new Vector2 to save the value to, optional -> defaults: new Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    getNormal(vector, resultVector) {
        resultVector = resultVector || new Vector2();
        return resultVector
            .setValues(vector.y - this.y, this.x - vector.x)
            .normalize();
    }
    /**
     * @memberof Vector2
     * @description Determines if Vector2.x and Vector2.y are both equal to 0
     * @returns {boolean}
     * @since 2.0.0
     */
    isZero() {
        return this.x === 0 && this.y === 0;
    }
    /**
     * @memberof Vector
     * @description Scales the Vector2 by a scalar Vector2
     * @param {Vector2} scalar A Vector2 that is used to scale the current Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    scale(scalar) {
        this.x *= scalar.x;
        this.y *= scalar.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Sets Vector2.x and Vector2.y to their negative values
     * @returns {Vector2}
     * @since 2.0.0
     */
    negate() {
        this.x = -this.x;
        this.y = -this.y;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Returns the magnitude or length of the Vector2
     * @returns {number}
     * @since 2.0.0
     */
    magnitude() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    }
    /**
     * @memberof Vector2
     * @description Returns the magnitude/lenth squared of the Vector2
     * @returns {number}
     * @since 2.0.0
     */
    magnitudeSqr() {
        return this.x * this.x + this.y * this.y;
    }
    /**
     * @memberof Vector2
     * @description Scales the Vector2 by the magnitude or length
     * @param magnitude The magnitude or length of the Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    scaleToMagnitude(magnitude) {
        const k = magnitude / this.magnitude();
        this.x *= k;
        this.y *= k;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Returns the string version of the Vector2
     * @example console.log(new Vector2(0, 0).toString()) // Vector2(0, 0)
     * @returns {string}
     * @since 2.0.0
     */
    toString() {
        return `Vector2(${this.x}, ${this.y})`;
    }
    /**
     * @memberof Vector2
     * @description Sets the values to be precise by using Number.toPrecision
     * @param {number} precision The precision
     * @returns {Vector2}
     * @since 2.0.0
     */
    toPrecision(precision) {
        this.x = Number(this.x.toPrecision(precision));
        this.y = Number(this.y.toPrecision(precision));
        return this;
    }
    /**
     * @memberof Vector2
     * @description Adds to the Vector2 by an amount
     * @param {number} dx Delta x, the amount to increase the x value by
     * @param {number} dy Delta y, the amount to increase the y value by
     * @returns {Vector2}
     * @since 2.0.0
     */
    translate(dx, dy) {
        this.x += dx;
        this.y += dy;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Adds to the Vector2.x by an amount
     * @param {number} dx Delta x, the amount to increase the x value by
     * @returns {Vector2}
     * @since 2.0.0
     */
    translateX(dx) {
        this.x += dx;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Adds to the Vector2.y by an amount
     * @param {number} dy Delta y, the amount to increase the y value by
     * @returns {Vector2}
     * @since 2.0.0
     */
    translateY(dy) {
        this.x += dy;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Gets the dot product using three different Vector2s
     * @param {Vector2} a A Vector2
     * @param {Vector2} b A Vector2
     * @param {Vector2} c A Vector2
     * @param {Vector2} [resultVector=Vector2] A Vector2 that the result is saved in, optional -> defaults: new Vector2
     * @returns {Vector2}
     * @since 2.0.0
     */
    tripleProduct(a, b, c, resultVector) {
        resultVector = resultVector || new Vector2();
        const ac = a.dot(c);
        const bc = b.dot(c);
        return resultVector.setValues(b.x * ac - a.x * bc, b.y * ac - a.y * bc);
    }
    /**
     * @memberof Vector2
     * @description Clamps the values to a min and max
     * @param {number} min The min value
     * @param {number} max The max value
     * @returns {Vector2}
     * @since 2.0.0
     */
    clamp(min, max) {
        this.x = clamp(this.x, min, max);
        this.y = clamp(this.y, min, max);
        return this;
    }
    /**
     * @memberof Vector2
     * @description Clamps the values to a min
     * @param {number} min The min value
     * @returns {Vector2}
     * @since 2.0.0
     */
    clampMin(min) {
        if (this.x < min) {
            this.x = min;
        }
        if (this.y < min) {
            this.y = min;
        }
        return this;
    }
    /**
     * @memberof Vector2
     * @description Clamps the values to a max
     * @param {number} max The max value
     * @returns {Vector2}
     * @since 2.0.0
     */
    clampMax(max) {
        if (this.x > max) {
            this.x = max;
        }
        if (this.y > max) {
            this.y = max;
        }
        return this;
    }
    /**
     * @memberof Vector2
     * @description Rotates the Vector2 based on degrees
     * @param {number} degrees The angle in degrees
     * @param {Vector2} [center] The center Vector relative to the Vector, optional -> defaults: Vector2.ZERO
     * @returns {Vector2}
     * @since 2.0.0
     */
    rotate(degrees, center = Vector2.ZERO) {
        const radians = degToRadians(degrees);
        const cx = center.x || 0;
        const cy = center.y || 0;
        const c = Math.cos(radians), s = Math.sin(radians);
        const x = this.x - cx;
        const y = this.y - cy;
        this.x = x * c - y * s + cx;
        this.y = x * s + y * c + cy;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Reflects the Vector2, returns the opposite value on a number line
     *
     * @example new Vector2(100, 50).reflect() // Vector2(-100, -50)
     *
     * @returns {Vector2}
     * @since 2.0.0
     */
    reflect() {
        this.x *= -1;
        this.y *= -1;
        return this;
    }
    /**
     * @memberof Vector2
     * @description Gets the absolute value of the vector
     * @returns {Vector2}
     * @since 2.0.0
     */
    abs() {
        this.x = Math.abs(this.x);
        this.y = Math.abs(this.y);
        return this;
    }
    // STATIC
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with 0 set as x and y
     * @returns {Vector2}
     * @since 2.0.0
     */
    static get ZERO() {
        return new Vector2(0, 0);
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with 0 set as x and -1 set as y
     * @returns {Vector2}
     * @since 2.0.0
     */
    static get UP() {
        return new Vector2(0, -1);
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with 0 set as x and 1 set as y
     * @returns {Vector2}
     * @since 2.0.0
     */
    static get DOWN() {
        return new Vector2(0, 1);
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with -1 set as x and 0 set as y
     * @returns {Vector2}
     * @since 2.0.0
     */
    static get LEFT() {
        return new Vector2(-1, 0);
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with 1 set as x and 0 set as y
     * @returns {Vector2}
     * @since 2.0.0
     */
    static get RIGHT() {
        return new Vector2(1, 0);
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with passed parameters, if no parameters are passed, a Vector2.ZERO is returned
     * @param {number} [x] X position, optional -> defaults: 0
     * @param {number} [y] Y position, optional -> defaults: 0
     * @returns {Vector2}
     * @since 2.0.0
     */
    static CREATE(x, y) {
        if (!x && !y)
            return this.ZERO; // none
        if (x && !y)
            return new Vector2(x); // only x
        if (!x && y)
            return new Vector2(0, y); // only y
        return new Vector2(x, y); // both
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with passed vector2Like object
     * @param {Duck.Types.Math.Vector2Like} vector2Like An object with x and y properties
     * @returns {Vector2}
     * @since 2.0.0
     */
    static fromVector2Like(vector2Like) {
        return new Vector2(vector2Like.x, vector2Like.y);
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2Like object with passed Vector2
     * @param {Vector2} vector2 A Vector2 to convert to Vector2Like object
     * @returns {Vector2}
     * @since 2.0.0
     */
    static toVector2Like(vector2) {
        return {
            x: vector2.x,
            y: vector2.y,
        };
    }
    /**
     * @memberof Vector2
     * @static
     * @description Returns a Vector2 with values from a passed Vector2
     * @param {Vector2} vector Vector2 to create a Vector2 from
     * @returns {Vector2}
     * @since 2.0.0
     */
    static fromVec(vector) {
        return new Vector2(vector.x, vector.y);
    }
}

;// CONCATENATED MODULE: ./lib/core/physics/models/collider.js
// circle collision physics from https://github.com/pothonprogramming/pothonprogramming.github.io/blob/master/content/circle-collision-response/circle-collision-response.html
// most of rect physics also from PothOnProgramming
/**
 * @class Collider
 * @classdesc Creates a DuckEngine Collider
 * @description The Collider Class. Collision Handler and Resolver
 * @since 1.0.0-beta
 */
class Collider {
    /**
     * @constructor Collider
     * @description Creates a Collider instance
     * @param {Hitbox} hitbox Hitbox to append the collider to
     * @param {GameObject<Duck.Types.Texture.Type>[] | Group<GameObject<Duck.Types.Texture.Type>>} collidesWith What the PhysicsBody collides with
     * @param {Game} game Game instance
     * @since 1.0.0-beta
     */
    constructor(hitbox, collidesWith, game) {
        this.hitbox = hitbox;
        this.collidesWith = collidesWith;
        this.game = game;
    }
    /**
     * @memberof Collider
     * @description Updates the collider and checks for collisions with the updated version of the object, and collides with.
     *
     * DO NOT CALL MANUALLY! CALLED IN PHYSICS SERVER!
     *
     * @param {Hitbox} hitbox The updated hitbox that the collider is attached
     * @param {GameObject<Duck.Types.Texture.Type>[] | Group<GameObject<Duck.Types.Texture.Type>>} updatedCollidesWith Updated version of what the object collides with
     * @since 2.0.0
     */
    _update(hitbox, updatedCollidesWith) {
        this.hitbox = hitbox;
        this.collidesWith = updatedCollidesWith;
        if (Array.isArray(this.collidesWith)) {
            this.collidesWith.forEach((otherObject) => {
                if (otherObject.hitbox) {
                    this.collideHitboxes(otherObject.hitbox);
                }
            });
        }
        else {
            this.collidesWith.each((otherObject) => {
                if (otherObject.hitbox) {
                    this.collideHitboxes(otherObject.hitbox);
                }
            });
        }
    }
    collideHitboxes(hitbox2) {
        const rectCX = hitbox2.position.x + hitbox2.w * 0.5;
        const rectCY = hitbox2.position.y + hitbox2.h * 0.5;
        const thisCX = this.hitbox.position.x + this.hitbox.w * 0.5;
        const thisCY = this.hitbox.position.y + this.hitbox.h * 0.5;
        const dx = rectCX - thisCX; // x difference between centers
        const dy = rectCY - thisCY; // y difference between centers
        const aw = (hitbox2.w + this.hitbox.w) * 0.5; // average width
        const ah = (hitbox2.h + this.hitbox.h) * 0.5; // average height
        /* If either distance is greater than the average dimension there is no collision. */
        if (Math.abs(dx) > aw || Math.abs(dy) > ah) {
            return 'none';
        }
        /* To determine which region of this rectangle the rect's center
          point is in, we have to account for the scale of the this rectangle.
          To do that, we divide dx and dy by it's width and height respectively. */
        if (Math.abs(dx / this.hitbox.w) > Math.abs(dy / this.hitbox.h)) {
            if (dx < 0) {
                // left
                this.hitbox.position.x = hitbox2.position.x + hitbox2.w;
                return 'left';
            }
            else {
                // right
                this.hitbox.position.x = hitbox2.position.x - this.hitbox.w;
                return 'right';
            }
        }
        else {
            if (dy < 0) {
                // top
                this.hitbox.position.y = hitbox2.position.y + hitbox2.h;
                return 'top';
            }
            else {
                // bottom
                this.hitbox.position.y = hitbox2.position.y - this.hitbox.h;
                return 'bottom';
            }
        }
    }
}

;// CONCATENATED MODULE: ./lib/utils/uniqueID.js
// https://gist.github.com/gordonbrander/2230317#gistcomment-1618310
function uniqueID() {
    function chr4() {
        return Math.random().toString(16).slice(-4);
    }
    return (chr4() +
        chr4() +
        '-' +
        chr4() +
        '-' +
        chr4() +
        '-' +
        chr4() +
        '-' +
        chr4() +
        chr4() +
        chr4());
}

;// CONCATENATED MODULE: ./lib/core/physics/utils/hitboxFaceIntersect.js
function hitboxFaceIntersect(hitbox, hitbox2) {
    const rectCX = hitbox2.position.x + hitbox2.w * 0.5;
    const rectCY = hitbox2.position.y + hitbox2.h * 0.5;
    const thisCX = hitbox.position.x + hitbox.w * 0.5;
    const thisCY = hitbox.position.y + hitbox.h * 0.5;
    const dx = rectCX - thisCX; // x difference between centers
    const dy = rectCY - thisCY; // y difference between centers
    const aw = (hitbox2.w + hitbox.w) * 0.5; // average width
    const ah = (hitbox2.h + hitbox.h) * 0.5; // average height
    /* If either distance is greater than the average dimension there is no collision. */
    if (Math.abs(dx) > aw || Math.abs(dy) > ah) {
        return 'none';
    }
    /* To determine which region of this rectangle the rect's center
      point is in, we have to account for the scale of the this rectangle.
      To do that, we divide dx and dy by it's width and height respectively. */
    if (Math.abs(dx / hitbox.w) > Math.abs(dy / hitbox.h)) {
        if (dx < 0) {
            // left
            hitbox.position.x = hitbox2.position.x + hitbox2.w;
            return 'left';
        }
        else {
            // right
            hitbox.position.x = hitbox2.position.x - hitbox.w;
            return 'right';
        }
    }
    else {
        if (dy < 0) {
            // top
            hitbox.position.y = hitbox2.position.y + hitbox2.h;
            return 'top';
        }
        else {
            // bottom
            hitbox.position.y = hitbox2.position.y - hitbox.h;
            return 'bottom';
        }
    }
}

;// CONCATENATED MODULE: ./lib/core/physics/utils/rectToRectIntersect.js
/**
 * @function
 * @description Returns a boolean based on if a rectangle is intersecting with a rectangle
 * @param {Rect |Sprite | RoundRect | { position: { x:number; y:number }; w:number; h:number }} rect
 * @param {Rect |Sprite| RoundRect | { position: { x:number; y:number }; w:number; h:number }} rect2
 * @returns {boolean}
 * @since 1.1.0-beta
 */
function rectToRectIntersect(rect, rect2) {
    const rectCX = rect2.position.x + rect2.w * 0.5;
    const rectCY = rect2.position.y + rect2.h * 0.5;
    const thisCX = rect.position.x + rect.w * 0.5;
    const thisCY = rect.position.y + rect.h * 0.5;
    const dx = rectCX - thisCX; // x difference between centers
    const dy = rectCY - thisCY; // y difference between centers
    const aw = (rect2.w + rect.w) * 0.5; // average width
    const ah = (rect2.h + rect.h) * 0.5; // average height
    /* If either distance is greater than the average dimension there is no collision. */
    if (Math.abs(dx) > aw || Math.abs(dy) > ah)
        return false;
    else
        return true;
}

;// CONCATENATED MODULE: ./lib/core/physics/models/hitbox.js






/**
 * @class Hitbox
 * @classdesc Creates a DuckEngine Hitbox
 * @description The Hitbox Class. A AABB Hitbox used for Colliders
 * @since 2.0.0
 */
class Hitbox {
    /**
     * @constructor Hitbox
     * @param {number} id The PhysicsBody ID
     * @param {Vector2} position The position of the Hitbox
     * @param {number} w The width of the Hitbox
     * @param {number} h The height of the Hitbox
     * @param {Vector2} offset The offset position of the Hitbox
     * @param {PhysicsBody<Duck.Types.Texture.Type>} physicsObject The PhysicsBody that the Hitbox is attached to
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param {string|undefined} [debugColor=undefined] The debugColor of the Hitbox
     */
    constructor(position, w, h, offset, physicsObject, game, scene, debugColor) {
        this.id = uniqueID();
        this.position = position;
        this.offset = offset;
        this.w = w;
        this.h = h;
        this.game = game;
        this.scene = scene;
        this.physicsObject = physicsObject;
        this.debugColor = debugColor;
        this.visible = debugColor ? true : false;
        this.zIndex = Duck.Layers.Rendering.zIndex.graphicDebug;
        this.culled = debugColor ? true : false;
        this.collisionState = 'none';
    }
    /**
     * @memberof Hitbox
     * @description Draws the hitbox if a debugColor is passed in the constructor.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     * @since 2.0.0
     */
    _draw() {
        if (this.game.renderer.ctx) {
            if (this.debugColor) {
                this.game.renderer.drawRect(this.position.x, this.position.y, this.w, this.h, this.debugColor);
            }
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined. HTMLCanvasElement is undefined.');
        }
    }
    /**
     * @memberof Hitbox
     * @description Sets the Hitboxes position to the PhysicsBodies position plus the passed offset if one was set
     *
     * DO NOT CALL MANUALLY, CALLED IN SCENE.__tick
     *
     * @since 2.0.0
     */
    _update(physicsObject) {
        this.physicsObject = physicsObject;
        // resolve
        this.position = this.physicsObject.position.add(this.offset);
    }
    /**
     * @memberof Hitbox
     * @description Sets the debugColor and visibility of the Hitbox as a debug aid
     * @param {string} debugColor Color
     * @param {boolean} [visible=true] What to set the visible property as, optional -> defaults: true
     * @since 2.0.0
     */
    setDebugColor(debugColor, visible = true) {
        this.debugColor = debugColor;
        this.visible = visible;
    }
    /**
     * @memberof Hitbox
     * @description Sets the width and height of the Hitbox
     * @param {Vector2} scale The new scale of the Hitbox
     * @since 2.0.0
     */
    scale(scale) {
        this.w = scale.x;
        this.h = scale.y;
    }
    /**
     * @memberof Hitbox
     * @description Sets position of the Hitbox
     * @param {Vector2} newPosition The new position of the Hitbox
     * @param {Vector2} [offset] The new offset position of the Hitbox, optional -> defaults: Hitbox.offset
     * @since 2.0.0
     */
    setPosition(newPosition, offset = this.offset) {
        this.offset = offset;
        this.position = newPosition.add(this.offset);
    }
    /**
     * @memberof Hitbox
     * @description Auto scales, positions, and offsets the Hitbox based on the shape of the PhysicsBody
     * @param {Vector2} [offset] The new offset position of the Hitbox, optional
     * @since 2.0.0
     */
    auto(offset) {
        if (this.physicsObject.shape === 'circle') {
            // top left corner
            const topLeft = new Vector2(this.physicsObject.position.x - this.physicsObject.r, this.physicsObject.position.y - this.physicsObject.r);
            if (offset) {
                topLeft.add(offset);
            }
            this.position = topLeft;
            this.scale(new Vector2(this.physicsObject.r * 2, this.physicsObject.r * 2));
        }
        else {
            this.position = this.physicsObject.position;
            if (offset) {
                this.position.add(offset);
            }
            this.scale(new Vector2(this.physicsObject.w, this.physicsObject.h));
        }
    }
    /**
     * @memberof Hitbox
     * @description Checks if the Hitbox is intersecting with another Hitbox
     * @param {Hitbox} Hitbox Hitbox to use to test the intersection
     * @returns {boolean}
     * @since 2.0.0
     */
    intersectsWith(hitbox) {
        return rectToRectIntersect({
            position: {
                x: this.position.x,
                y: this.position.y,
            },
            w: this.w,
            h: this.h,
        }, {
            position: {
                x: hitbox.position.x,
                y: hitbox.position.y,
            },
            w: hitbox.w,
            h: hitbox.h,
        });
    }
    /**
     * @memberof Hitbox
     * @description Checks if the Hitbox is intersecting with another Hitbox and returns the face that is colliding
     * @param {Hitbox} hitbox Hitbox to use to test the intersection
     * @returns {Duck.Types.Collider.CollisionResponseType}
     * @since 2.0.0
     */
    intersectsFaceWith(hitbox) {
        this.collisionState = hitboxFaceIntersect(this, hitbox);
        return this.collisionState;
    }
    /**
     * @memberof Hitbox
     * @description Checks if the Hitbox is intersecting with other Hitboxes and returns the face that is colliding
     * @param {Group<Hitbox> | Hitbox[]} hitboxes Hitboxes to use to test the intersection
     * @returns {Duck.Types.Collider.CollisionResponseType[]}
     * @since 2.0.0
     */
    groupIntersectsFaceWith(hitboxes) {
        const collisionStates = [];
        if (Array.isArray(hitboxes)) {
            for (let i = 0; i < hitboxes.length; i++) {
                const hitbox = hitboxes[i];
                collisionStates.push(hitboxFaceIntersect(this, hitbox));
            }
        }
        else {
            for (let i = 0; i < hitboxes.group.length; i++) {
                const hitbox = hitboxes.group[i];
                collisionStates.push(hitboxFaceIntersect(this, hitbox));
            }
        }
        return collisionStates;
    }
    /**
     * @memberof Hitbox
     * @description Gets the top most coordinate of the Hitbox
     * @returns {number}
     * @since 2.0.0
     */
    getTop() {
        return this.position.y;
    }
    /**
     * @memberof Hitbox
     * @description Gets the bottom most coordinate of the Hitbox
     * @returns {number}
     * @since 2.0.0
     */
    getBottom() {
        return this.position.y + this.h;
    }
    /**
     * @memberof Hitbox
     * @description Gets the left most coordinate of the Hitbox
     * @returns {number}
     * @since 2.0.0
     */
    getLeft() {
        return this.position.x;
    }
    /**
     * @memberof Hitbox
     * @description Gets the right most coordinate of the Hitbox
     * @returns {number}
     * @since 2.0.0
     */
    getRight() {
        return this.position.x + this.w;
    }
    /**
     * @memberof Hitbox
     * @description Gets the center coordinates of the Hitbox
     * @returns {Vector2}
     * @since 2.0.0
     */
    getCenter() {
        return new Vector2(this.position.x + this.w / 2, this.position.y + this.h / 2);
    }
    /**
     * @memberof Hitbox
     * @description Gets the centerY coordinate of the Hitbox
     * @returns {number}
     * @since 2.0.0
     */
    getCenterY() {
        return this.position.y + this.h / 2;
    }
    /**
     * @memberof Hitbox
     * @description Gets the centerX coordinate of the Hitbox
     * @returns {number}
     * @since 2.0.0
     */
    getCenterX() {
        return this.position.x + this.w / 2;
    }
}

;// CONCATENATED MODULE: ./lib/core/physics/physicsBody.js





/**
 * @class PhysicsBody
 * @classdesc Creates a DuckEngine PhysicsBody
 * @description The PhysicsBody Class. The GameObject class extends this class
 * @since 2.0.0
 */
class PhysicsBody {
    /**
     * @constructor PhysicsBody
     * @description Creates a PhysicsBody instance. Extended by GameObject
     * @param {Duck.Types.Collider.ShapeString} shape Shape of PhysicsBody
     * @param {number} id ID from GameObject ID
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width
     * @param {number} h Height
     * @param {number} r Radius
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(shape, id, x, y, w, h, r, game, scene) {
        this.shape = shape;
        this.id = id;
        this.position = new Vector2(x, y);
        this.w = w;
        this.h = h;
        this.r = r;
        this.options = {
            type: 'KinematicBody',
        };
        this.game = game;
        this.scene = scene;
        this.velocity = Vector2.ZERO;
        this.collider = undefined;
        this.collidesWith = [];
        this.enabled = true;
        this.isAttached = false;
        this.attachedChildren = [];
        this.attachOffset = Vector2.ZERO;
        this.bounds = {
            x: -Infinity,
            y: -Infinity,
            w: Infinity,
            h: Infinity,
        };
        // methods
        this.physics = {
            addCollider: (collidesWith) => {
                var _a;
                if (!this.hitbox) {
                    new debug.Error('Cannot add collider to PhysicsObject. No hitbox exists. Create a hitbox first using PhysicsObject.physics.addHitbox');
                    return undefined;
                }
                if (!((_a = this.game.config.physics) === null || _a === void 0 ? void 0 : _a.enabled)) {
                    new debug.Error('Cannot add collider to PhysicsObject. Game Config.physics.enabled must be truthy!');
                }
                this.collidesWith = collidesWith;
                this.collider = new Collider(this.hitbox, collidesWith, this.game);
                return this.collider;
            },
            addHitbox: (w, h, offset = Vector2.ZERO, debugColor) => {
                var _a;
                if (!((_a = this.game.config.physics) === null || _a === void 0 ? void 0 : _a.enabled)) {
                    new debug.Error('Cannot add hitbox to PhysicsObject. Game Config.physics.enabled must be truthy!');
                }
                this.hitbox = new Hitbox(this.position, w || 0, h || 0, offset, this, this.game, this.scene, debugColor);
                if (!w && !h) {
                    this.hitbox.auto(offset);
                }
                this.scene.displayList.add(this.hitbox);
                return this.hitbox;
            },
            setBounds: (x, y, w, h) => {
                this.bounds.x = x;
                this.bounds.y = y;
                this.bounds.w = w;
                this.bounds.h = h;
            },
        };
    }
    setEnabled(enabled) {
        this.enabled = enabled;
        return this.enabled;
    }
    /**
     * @memberof PhysicsBody
     * @description Updates the PhysicsBody's position by the velocity. Sets velocity to 0 on every tick.
     * Clamps position to bounds if exists. Rounds pixels if roundPixels game config is set to true.
     * Updates hitbox.collisionState if hitbox exists.
     *
     * DO NOT CALL MANUALLY, CALLED IN SCENE.__tick
     *
     * @since 2.0.0
     */
    _update() {
        var _a;
        this.position.x += this.velocity.x * this.game.smoothDeltaTime;
        this.position.y += this.velocity.y * this.game.smoothDeltaTime;
        // clamp to bounds
        this.position.x = clamp(this.position.x, this.bounds.x, this.bounds.w);
        this.position.y = clamp(this.position.y, this.bounds.y, this.bounds.h);
        // set to none
        this.velocity.x = 0;
        this.velocity.y = 0;
        // roundPixels
        if (this.game.config.roundPixels) {
            this.position.round();
        }
        // apply gravity
        if ((_a = this.game.config.physics) === null || _a === void 0 ? void 0 : _a.gravity) {
            if (this.options.type === 'KinematicBody' ||
                this.options.type === 'RigidBody') {
                this.applyGravity(Vector2.fromVector2Like(this.game.config.physics.gravity));
            }
        }
        // update attached children position
        this.attachedChildren.forEach((object) => {
            const pos = this.position.clone();
            pos.subtract(object.attachOffset);
            object.position = pos;
            if (object.hitbox) {
                object.hitbox.position = object.position
                    .clone()
                    .add(object.hitbox.offset);
            }
        });
    }
    /**
     * @memberof PhysicsBody
     * @description Sets the PhysicsBody type, PhysicsBody type determines what can be applied and what effects the body
     * @param {Duck.Types.PhysicsBody.Type} type The type of the PhysicsBody, 'KinematicBody' | 'RigidBody' | 'StaticBody'
     * @since 2.0.0
     */
    setType(type) {
        this.options.type = type;
        return this.options;
    }
    /**
     * @memberof PhysicsBody
     * @description Attaches self to another PhysicsBody, makes self unmovable and follows the PhysicsBody accordingly
     * @param {PhysicsBody<Duck.Types.Texture.Type>} object PhysicsBody to attach to
     * @param {Vector2} [diffOffset=Vector2] A different offset, optional -> defaults: Difference in positions from PhysicsBody to self
     * @since 2.0.0
     */
    attachTo(object, diffOffset) {
        const offset = diffOffset ||
            Vector2.fromVec(object.position).subtract(this.position);
        this.isAttached = true;
        this.attachOffset = offset;
        object.attachedChildren.push(this);
    }
    /**
     * @memberof PhysicsBody
     * @description Attaches a child PhysicsBody to self, makes child unmovable and follows the self accordingly
     * @param {PhysicsBody<Duck.Types.Texture.Type>} object PhysicsBody to attach
     * @param {Vector2} [diffOffset=Vector2] A different offset, optional -> defaults: Difference in positions from self to PhysicsBody
     * @since 2.0.0
     */
    attachChild(object, diffOffset) {
        const offset = diffOffset ||
            Vector2.fromVec(this.position).subtract(object.position);
        object.isAttached = true;
        object.attachOffset = offset;
        this.attachedChildren.push(object);
    }
    /**
     * @memberof PhysicsBody
     * @description Detaches self from another PhysicsBody
     * @param {PhysicsBody<Duck.Types.Texture.Type>} object PhysicsBody to detach from
     * @since 2.0.0
     */
    detachFrom(object) {
        const f = object.attachedChildren.find((o) => o.id === this.id);
        if (f) {
            this.isAttached = false;
            this.attachOffset = Vector2.ZERO;
            object.attachedChildren.splice(object.attachedChildren.findIndex((o) => o.id === this.id), 1);
        }
        else {
            new debug.Error('Cannot detachFrom from object, PhysicsBody is not attached to anything.');
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Detaches PhysicsBody from self
     * @param {PhysicsBody<Duck.Types.Texture.Type>} object PhysicsBody to detach
     * @since 2.0.0
     */
    detachChild(object) {
        const f = this.attachedChildren.find((o) => o.id === object.id);
        if (f) {
            object.isAttached = false;
            object.attachOffset = Vector2.ZERO;
            this.attachedChildren.splice(this.attachedChildren.findIndex((o) => o.id === object.id), 1);
        }
        else {
            new debug.Error('Cannot detachChild from PhysicsBody, object is not attached to anything.');
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Sets the velocity based on an axis, PhysicsBody.options.type must be KinematicBody
     * @param {'x'|'y'} axis The axis to set the velocity of
     * @param {number} pxPerSecond The value to set the velocity axis as, in pixels per second
     * @since 2.0.0
     */
    setVelocity(axis, pxPerSecond) {
        if (this.options.type !== 'KinematicBody') {
            new debug.Error(`Cannot set velocity as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot set velocity as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        if (axis === 'x') {
            this.velocity.x = pxPerSecond;
        }
        if (axis === 'y') {
            this.velocity.y = pxPerSecond;
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Sets the velocity.x, PhysicsBody.options.type must be KinematicBody
     * @param {number} pxPerSecond The value to set the velocity axis as, in pixels per second
     * @since 2.0.0
     */
    setVelocityX(pxPerSecond) {
        if (this.options.type !== 'KinematicBody') {
            new debug.Error(`Cannot set velocity X as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot set velocity X as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        this.velocity.x = pxPerSecond;
    }
    /**
     * @memberof PhysicsBody
     * @description Sets the velocity.y, PhysicsBody.options.type must be KinematicBody
     * @param {number} pxPerSecond The value to set the velocity.y as, in pixels per second
     * @since 2.0.0
     */
    setVelocityY(pxPerSecond) {
        if (this.options.type !== 'KinematicBody') {
            new debug.Error(`Cannot set velocity Y as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot set velocity Y as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        this.velocity.y = pxPerSecond;
    }
    /**
     * @memberof PhysicsBody
     * @description Accelerates the velocity by an amount, PhysicsBody.options.type must be KinematicBody
     * @param {Vector2} target The target velocity
     * @param {number} amount The value to increase the velocity by
     * @since 2.0.0
     */
    accelerateVelocity(target, amount) {
        if (this.options.type !== 'KinematicBody') {
            new debug.Error(`Cannot accelerate velocity as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot accelerate velocity as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        this.velocity.moveTowards(this.velocity, target, amount);
    }
    /**
     * @memberof PhysicsBody
     * @description Applies friction to the velocity by an amount, PhysicsBody.options.type must be KinematicBody or RigidBody
     * @param {number} frictionAmount The value to decrease the velocity by
     * @since 2.0.0
     */
    applyFriction(frictionAmount) {
        if (this.options.type !== 'KinematicBody' &&
            this.options.type !== 'RigidBody') {
            new debug.Error(`Cannot apply friction as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody or RigidBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot apply friction as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        this.velocity.subtract(frictionAmount).clampMin(0);
    }
    /**
     * @memberof PhysicsBody
     * @description Applies gravity to the velocity by a Vector2, PhysicsBody.options.type must be KinematicBody or RigidBody
     * @param {Vector2} gravity The Vector2 to add to the velocity by
     * @since 2.0.0
     */
    applyGravity(gravity) {
        if (this.options.type !== 'KinematicBody' &&
            this.options.type !== 'RigidBody') {
            new debug.Error(`Cannot apply gravity as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody or RigidBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot apply gravity as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        if (gravity.x !== 0) {
            this.velocity.x += gravity.x;
        }
        if (gravity.y !== 0) {
            this.velocity.y += gravity.y;
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Applies gravity to the velocity by a Vector2, PhysicsBody.options.type must be KinematicBody or RigidBody
     * @param {Duck.Types.Math.BoundsLike} [bounds=PhysicsBody.bounds] The bounds of the PhysicsBody, optional -> defaults: PhysicsBody.bounds, if none
     * are set, it is infinite
     * @param {number} [restitution=1] How much energy is lost when bouncing, a number between 0-1 to loose energy,
     * 1-any to increase energy, 1 = none, must be a positive number
     * @since 2.0.0
     */
    bounceVelocityBounds(bounds = this.bounds, restitution = 1) {
        if (this.options.type !== 'KinematicBody' &&
            this.options.type !== 'RigidBody') {
            new debug.Error(`Cannot bounce velocity as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody or RigidBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot bounce velocity as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        if (this.position.x > bounds.w || this.position.x < bounds.x) {
            this.velocity.x = this.velocity.x * -restitution;
        }
        if (this.position.y > bounds.h || this.position.y < bounds.y) {
            this.velocity.y = this.velocity.y * -restitution;
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Reflects the velocity, sets the velocity as the opposite value of the velocity, PhysicsBody.options.type must be KinematicBody or RigidBody
     *
     * @example myPhysicsBody.setVelocity('x', 3);
     * myPhysicsBody.reflect(); // velocity: 0, -3
     *
     * @since 2.0.0
     */
    reflectVelocity() {
        if (this.options.type !== 'KinematicBody' &&
            this.options.type !== 'RigidBody') {
            new debug.Error(`Cannot reflect velocity as PhysicsBody.options.type is ${this.options.type} instead of KinematicBody or RigidBody.`);
            return;
        }
        if (this.isAttached) {
            new debug.Error('Cannot reflect velocity as PhysicsBody is attached to another PhysicsBody.');
            return;
        }
        this.velocity.reflect();
    }
    /**
     * @memberof PhysicsBody
     * @description Auto scales the PhysicsBody.hitbox to fit the shape
     * @param {Vector2} [offset] Position offset, optional -> defaults: undefined
     * @since 2.0.0
     */
    autoFitHitbox(offset) {
        var _a;
        (_a = this.hitbox) === null || _a === void 0 ? void 0 : _a.auto(offset);
    }
    /**
     * @memberof PhysicsBody
     * @description Scales the PhysicsBody.hitbox
     * @param {Vector2} scale Scale Vector2, x is width, y is height
     * @since 2.0.0
     */
    scaleHitbox(scale) {
        var _a;
        (_a = this.hitbox) === null || _a === void 0 ? void 0 : _a.scale(scale);
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the top most coordinate of the PhysicsBody
     * @returns {number}
     * @since 2.0.0
     */
    getTop() {
        return this.position.y;
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the bottom most coordinate of the PhysicsBody
     * @returns {number}
     * @since 2.0.0
     */
    getBottom() {
        return this.position.y + this.h;
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the left most coordinate of the PhysicsBody
     * @returns {number}
     * @since 2.0.0
     */
    getLeft() {
        return this.position.x;
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the right most coordinate of the PhysicsBody
     * @returns {number}
     * @since 2.0.0
     */
    getRight() {
        return this.position.x + this.w;
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the center coordinates of the PhysicsBody
     * @returns {Vector2}
     * @since 2.0.0
     */
    getCenter() {
        if (this.shape === 'circle') {
            return new Vector2(this.position.x, this.position.y);
        }
        else {
            return new Vector2(this.position.x + this.w / 2, this.position.y + this.h / 2);
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the centerY coordinate of the PhysicsBody
     * @returns {number}
     * @since 2.0.0
     */
    getCenterY() {
        if (this.shape === 'circle') {
            return this.position.y + this.r;
        }
        else {
            return this.position.y + this.h / 2;
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Gets the centerX coordinate of the PhysicsBody
     * @returns {number}
     * @since 2.0.0
     */
    getCenterX() {
        if (this.shape === 'circle') {
            return this.position.x + this.r;
        }
        else {
            return this.position.x + this.w / 2;
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Checks and returns the Collision Type if two hitboxes are colliding
     * @param {PhysicsBody<Duck.Types.Texture.Type>} obj PhysicsBody to check their hitbox with
     * @returns {false | Duck.Types.Collider.CollisionResponseType | undefined}
     * @since 2.0.0
     */
    isColliding(obj) {
        var _a, _b;
        if (obj.hitbox) {
            return ((_a = this.hitbox) === null || _a === void 0 ? void 0 : _a.intersectsFaceWith(obj.hitbox)) !== 'none'
                ? (_b = this.hitbox) === null || _b === void 0 ? void 0 : _b.intersectsFaceWith(obj.hitbox)
                : false;
        }
        else {
            return false;
        }
    }
    /**
     * @memberof PhysicsBody
     * @description Checks and returns the Collision Type if multiple hitboxes are colliding
     * @param {Group<PhysicsBody<Duck.Types.Texture.Type>> | PhysicsBody<Duck.Types.Texture.Type>[]} objects PhysicsBodies to check their hitbox with
     * @returns {false | Duck.Types.Collider.CollisionResponseType}
     * @since 2.0.0
     */
    isCollidingGroup(objects) {
        var _a;
        const hitboxes = [];
        if (Array.isArray(objects)) {
            objects.forEach((obj) => {
                if (obj.hitbox) {
                    hitboxes.push(obj.hitbox);
                }
            });
        }
        else {
            objects.each((obj) => {
                if (obj.hitbox) {
                    hitboxes.push(obj.hitbox);
                }
            });
        }
        const states = (_a = this.hitbox) === null || _a === void 0 ? void 0 : _a.groupIntersectsFaceWith(hitboxes);
        return (states === null || states === void 0 ? void 0 : states.includes('top'))
            ? 'top'
            :  false || (states === null || states === void 0 ? void 0 : states.includes('bottom'))
                ? 'bottom'
                :  false || (states === null || states === void 0 ? void 0 : states.includes('left'))
                    ? 'left'
                    :  false || (states === null || states === void 0 ? void 0 : states.includes('right'))
                        ? 'right'
                        : false;
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/gameObject.js
/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable @typescript-eslint/no-empty-function */



/**
 * @class GameObject
 * @classdesc Creates a DuckEngine GameObject
 * @description The GameObject Class. All GameObjects extend this class
 * @since 1.0.0-beta
 */
class GameObject extends PhysicsBody {
    // methods
    /**
     * @constructor
     * @description Creates a GameObject instance.
     * @param {Duck.Types.Collider.ShapeString} shape Shape of the gameobject
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width
     * @param {number} h Height
     * @param {number} r Radius
     * @param {string} fillColor Fill color or Texture instance
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(shape, x, y, w, h, r, texture, game, scene) {
        const id = uniqueID();
        super(shape, id, x, y, w, h, r, game, scene);
        this.texture = texture;
        this.visible = true;
        this.zIndex = Duck.Layers.Rendering.zIndex.gameobject;
        this.culled = true;
        // fix blend mode due to StaticLight setting blend mode to lighten
        if (this.game.renderer.ctx) {
            this.game.renderer.setBlendMode('source-over');
        }
    }
    /**
     * @memberof GameObject
     * @description Draws the gameobject.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     * @since 1.0.0-beta
     */
    _draw() { }
    /**
     * @memberof GameObject
     * @description Sets the visible property and calls the game.renderer.pipeline.pool method to immediately update the visibility
     *
     * **Note: this calls Game.renderer.pipeline.pool to immediately update the visibility**
     *
     * @param {boolean} visible What to set the visible property to
     * @since 2.1.0
     */
    setVisible(visible) {
        this.visible = visible;
        this.game.renderer.pipeline.pool();
    }
    /**
     * @memberof GameObject
     * @description Sets the scale of the GameObject
     * @param {Duck.Types.Misc.Scale|number} scale Scale of the gameobject, can be a number to change the radius
     * @since 1.0.0-beta
     */
    setScale(scale) {
        if (typeof scale !== 'number') {
            if (scale.width) {
                this.w = scale.width;
            }
            if (scale.height) {
                this.h = scale.height;
            }
        }
        else {
            this.r = scale;
        }
    }
    /**
     * @memberof GameObject
     * @description Sets the fill color of the GameObject
     * @param {string} fillColor Fill color
     * @since 1.0.0-beta
     */
    setFillColor(fillColor) {
        this.texture.texture = fillColor;
    }
}

;// CONCATENATED MODULE: ./lib/core/texture/textureBase.js


class TextureBase {
    /**
     * @constructor Texture
     * @description Creates a texture
     * @param {Duck.Types.Texture.Type} type Texture type
     * @param {HTMLImageElement:string} texture Texture source
     * @param {number} w Width of texture
     * @param {number} h Height of texture
     * @since 2.1.0
     */
    constructor(type, dataType, texture, w, h) {
        this.id = uniqueID();
        this.type = type;
        this.dataType = dataType;
        this.texture = texture;
        this.scale = new Vector2(w, h);
        if (this.type === 'image') {
            this.texture.width = this.scale.x;
            this.texture.height = this.scale.y;
        }
    }
    /**
     * @memberof Texture
     * @description Sets the Texture Scale
     * @param {Duck.Types.Misc.Scale} scale New scale of the texture
     * @since 2.1.0
     */
    setScale(scale) {
        this.scale = new Vector2(scale.width, scale.height);
        if (this.type === 'image') {
            this.texture.width = this.scale.x;
            this.texture.height = this.scale.y;
        }
        return this.scale;
    }
    /**
     * @memberof Texture
     * @description Sets the Texture Image Path if the type is image
     * @param {string} imagePath New imagePath of the texture
     * @since 2.1.0
     */
    setImagePath(imagePath) {
        if (this.type === 'image') {
            this.texture.src = imagePath;
        }
    }
    /**
     * @memberof Texture
     * @description Sets the Texture color if the type is color
     * @param {string} color New color of the texture
     * @since 2.1.0
     */
    setFillColor(color) {
        if (this.type === 'color') {
            this.texture = color;
        }
    }
    /**
     * @memberof TextureBase
     * @description Creates a new TextureBase instance from a color
     * @param {string} color Color
     * @param {number} w Width
     * @param {number} h Height
     * @static
     * @returns {TextureBase<'color'>}
     * @since 2.1.0
     */
    static fromColor(color, w, h) {
        return new TextureBase('color', 'base', color, w, h);
    }
    /**
     * @memberof TextureBase
     * @description Creates a new TextureBase instance from an image path
     * @param {string} imgpath Image path
     * @param {number} w Width
     * @param {number} h Height
     * @static
     * @returns {TextureBase<'image'>}
     * @since 2.1.0
     */
    static fromTexture(imgpath, w, h) {
        const img = new Image(w, h);
        img.src = imgpath;
        return new TextureBase('image', 'base', img, w, h);
    }
    /**
     * @memberof TextureBase
     * @description Creates a new TextureBase instance from a color or an image path
     * @param {string} fillColorOrIMGPath Color or Image path
     * @param {number} w Width
     * @param {number} h Height
     * @static
     * @returns {TextureBase<'either'>}
     * @since 2.1.0
     */
    static fromEither(fillColorOrIMGPath, w, h) {
        return new TextureBase('either', 'base', fillColorOrIMGPath, w, h);
    }
}

;// CONCATENATED MODULE: ./lib/core/texture/texture.js

/**
 * @class Texture
 * @classdesc Creates a DuckEngine Texture
 * @description The Texture Class. Stores images and colors in a class
 * @template type Texture type, 'image' | 'color' | 'either'
 * @extends TextureBase
 * @since 2.0.0
 */
class Texture extends TextureBase {
    /**
     * @constructor Texture
     * @description Creates a texture
     * @param {Duck.Types.Texture.Type} type Texture type
     * @param {HTMLImageElement:string} texture Texture source
     * @param {number} w Width of texture
     * @param {number} h Height of texture
     * @since 2.0.0
     */
    constructor(type, texture, w, h) {
        if (type === 'image') {
            super('image', 'base', texture, w, h);
        }
        else if (type === 'color') {
            super('color', 'base', texture, w, h);
        }
        else {
            super('either', 'base', texture, w, h);
        }
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/circle.js



/**
 * @class Circle
 * @classdesc Creates a DuckEngine Circle
 * @description The Circle Class. Represents a circle
 * @since 1.0.0-beta
 */
class Circle extends GameObject {
    /**
     * @constructor Circle
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} r Radius of Circle
     * @param {string} fillColor Fill Color of Circle
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(x, y, r, fillColor, game, scene) {
        super('circle', x, y, 0, 0, r, Texture.fromColor(fillColor, r, r), game, scene);
    }
    /**
     * @description Draws the circle.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            this.game.renderer.drawCircle(this.position.x, this.position.y, this.r, this.texture.texture);
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined. Canvas is undefined.');
        }
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/rect.js



/**
 * @class Rect
 * @classdesc Creates a DuckEngine Rect
 * @description The Rect Class. Represents a rectangle
 * @since 1.0.0-beta
 */
class Rect extends GameObject {
    /**
     * @constructor Rect
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of Rect
     * @param {number} h Height of Rect
     * @param {string} fillColor Fill Color of Rect
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(x, y, w, h, fillColor, game, scene) {
        super('rect', x, y, w, h, 0, Texture.fromColor(fillColor, w, h), game, scene);
    }
    /**
     * @description Draws the rect.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            this.game.renderer.drawRect(this.position.x, this.position.y, this.w, this.h, this.texture.texture);
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined. HTMLCanvasElement is undefined.');
        }
    }
}

;// CONCATENATED MODULE: ./lib/core/physics/utils/circleToRectIntersect.js
/**
 * @function
 * @description Returns a boolean based on if a circle is intersecting with a rectangle
 * @param {Circle| { position: { x:number; y:number }; w:number; h:number; r:number; }} circle Circle
 * @param {Rect | { position: { x:number; y:number }; w:number; h:number } | Sprite} rect Rect
 * @returns {boolean}
 * @since 1.1.0-beta
 */
function circleRectCollision(circle, rect) {
    let dx = Math.abs(circle.position.x - (rect.position.x + rect.w / 2));
    let dy = Math.abs(circle.position.y - (rect.position.y + rect.h / 2));
    if (dx > circle.r + rect.w / 2) {
        return false;
    }
    if (dy > circle.r + rect.h / 2) {
        return false;
    }
    if (dx <= rect.w) {
        return true;
    }
    if (dy <= rect.h) {
        return true;
    }
    dx = dx - rect.w;
    dy = dy - rect.h;
    return dx * dx + dy * dy <= circle.r * circle.r;
}

;// CONCATENATED MODULE: ./lib/core/math/randomInt.js
/**
 * @function
 * @description Returns a random integer between two values
 * @param {number} min Minimum
 * @param {number} max Maximum
 */
function randomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1) + min);
}

;// CONCATENATED MODULE: ./lib/core/math/lerp.js
/**
 * @function
 * @description Linearly interpolates between two values based on an amount
 * @param {number} start Start Value
 * @param {number} end Target Value
 * @param {number} amount amount
 * @since 2.0.0
 */
function lerp(start, end, amount) {
    return (1 - amount) * start + amount * end;
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/roundrect.js



/**
 * @class RoundRect
 * @classdesc Creates a DuckEngine GameObject
 * @description The RoundRect Class. Represents a smooth cornered rect
 * @since 1.0.0-beta
 */
class RoundRect extends GameObject {
    /**
     * @constructor RoundRect
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of RoundRect
     * @param {number} h Height of RoundRect
     * @param {number} r Radius of RoundRect
     * @param {string} fillColor Fill Color of RoundRect
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(x, y, w, h, r, fillColor, game, scene) {
        super('roundrect', x, y, w, h, r, Texture.fromColor(fillColor, w, h), game, scene);
    }
    /**
     * @description Draws the roundrect.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            this.game.renderer.drawRoundRect(this.position.x, this.position.y, this.w, this.h, this.r, this.texture.texture);
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined as Canvas is undefined.');
        }
    }
}

;// CONCATENATED MODULE: ./lib/base/timer.js
/* eslint-disable @typescript-eslint/no-explicit-any */
class Timer {
    constructor(ms, cb, args, repeat) {
        this.seconds = ms / 1000;
        this.cb = cb;
        this.repeat = repeat - 1;
        this.counter = 0;
        this.repeatCounter = 0;
        this.done = false;
        this.args = args;
    }
    count(delta) {
        if (this.done) {
            return;
        }
        if (this.counter < this.seconds) {
            // normal count
            this.counter += delta;
            return;
        }
        if (this.repeatCounter < this.repeat) {
            // repeat
            this.repeatCounter++;
            this.counter = 0;
            this.cb(...this.args, this.counter, this.repeatCounter, this.done);
            return;
        }
        if (this.counter > this.seconds) {
            // done
            this.done = true;
            this.cb(...this.args, this.counter, this.repeatCounter, this.done);
            return;
        }
    }
    stop() {
        this.done = true;
    }
    reset() {
        this.done = false;
        this.counter = 0;
        this.repeatCounter = 0;
    }
    pause() {
        this.done = true;
    }
    resume() {
        this.done = true;
    }
}

;// CONCATENATED MODULE: ./lib/core/animation/animationFrame.js
/**
 * @class AnimationFrame
 * @classdesc Creates a DuckEngine AnimationFrame
 * @description The AnimationFrame Class. Holds information about what col and row to use for the Animation
 * @since 2.0.0
 */
class AnimationFrame {
    /**
     * @constructor AnimationFrame
     * @description Creates an AnimationFrame instance
     * @param {number} col Column of the Frame
     * @param {number} row Row of the Frame
     * @param {Sprite} sprite Sprite to modify the currentCol and currentRow of
     * @since 2.0.0
     */
    constructor(col, row, sprite) {
        this.col = col;
        this.row = row;
        this.sprite = sprite;
    }
    /**
     * @memberof AnimationFrame
     * @description Sets the Sprite.currentCol to AnimationFrame.col and Sprite.currentRow to AnimationFrame.row
     * @since 2.0.0
     */
    set() {
        this.sprite.currentCol = this.col;
        this.sprite.currentRow = this.row;
    }
}

;// CONCATENATED MODULE: ./lib/core/animation/animation.js


/**
 * @class Animation
 * @classdesc Creates a DuckEngine Animation
 * @description The Animation Class. An animation of frames that are used for Sprites
 * @since 2.0.0
 */
class animation_Animation {
    /**
     * @constructor Animation
     * @description Creates an Animation instance
     * @param {Duck.Types.Animation.Config} config Animation configuration
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param {Sprite} sprite Sprite that the Animation is attached to
     * @since 2.0.0
     */
    constructor(config, game, scene, sprite) {
        this.key = config.key;
        this.config = config;
        this.game = game;
        this.scene = scene;
        this.sprite = sprite;
        this.countBy = this.config.useDelta
            ? this.game.deltaTime
            : 1000 / this.config.frameRate;
        this.repeat = this.config.repeat || 1;
        this.repeatCounter = 0;
        this.frames = this.createFrames();
        this.reversedFrames = this.frames.slice().reverse();
        this.animationNormalTimer = new Timer(1000 / this.config.frameRate, () => {
            this.normalStep(this);
        }, [], this.frames.length * this.repeat);
        this.animationReverseTimer = new Timer(1000 / this.config.frameRate, () => {
            this.reverseStep(this);
        }, [], this.reversedFrames.length * this.repeat);
        if (this.config.delay) {
            this.delayTimer = new Timer(this.config.delay, () => undefined, [], 1);
        }
        this.currentIndex = 0;
        this.currentFrame = this.frames[this.currentIndex];
    }
    createFrames() {
        const res = [];
        for (let i = 0; i < this.config.frames.length; i++) {
            const frameBase = this.config.frames[i];
            res.push(new AnimationFrame(frameBase.col, frameBase.row, this.sprite));
        }
        return res;
    }
    normalStep(self) {
        if (self.currentIndex < self.frames.length - 1) {
            self.currentIndex += 1;
            self.currentFrame = self.frames[self.currentIndex];
            self.currentFrame.set();
        }
        else {
            self.repeatCounter += 1;
            if (self.config.yoyo) {
                self.stop();
                self.playReverse();
            }
            else if (self.repeatCounter < self.repeat) {
                self.currentIndex = 0;
                self.currentFrame = self.frames[self.currentIndex];
                self.currentFrame.set();
            }
        }
    }
    reverseStep(self) {
        if (self.currentIndex < self.reversedFrames.length - 1) {
            self.currentIndex += 1;
            self.currentFrame = self.reversedFrames[self.currentIndex];
            self.currentFrame.set();
        }
        else {
            self.repeatCounter += 1;
            if (self.config.yoyo) {
                self.stopReverse();
                self.play();
            }
            else if (self.repeatCounter < self.repeat) {
                self.currentIndex = 0;
                self.currentFrame = self.frames[self.currentIndex];
                self.currentFrame.set();
            }
        }
    }
    /**
     * @memberof Animation
     * @description Sets the repeat amount of the internal normal timer
     * @param {number} repeat Repeat amount
     * @since 2.0.0
     */
    setRepeat(repeat) {
        this.repeat = repeat;
        this.animationNormalTimer.repeat = this.repeat;
    }
    /**
     * @memberof Animation
     * @description Sets the repeat amount of the internal reverse timer
     * @param {number} repeat Repeat amount
     * @since 2.0.0
     */
    setRepeatReverse(repeat) {
        this.repeat = repeat;
        this.animationReverseTimer.repeat = this.repeat;
    }
    /**
     * @memberof Animation
     * @description Plays the Animation using the internal normal timer
     * @since 2.0.0
     */
    play() {
        this.countBy = this.config.useDelta
            ? this.game.deltaTime
            : 1000 / this.config.frameRate;
        if (this.delayTimer) {
            this.delayTimer.count(this.countBy);
            if (this.delayTimer.done) {
                this.animationNormalTimer.count(this.countBy);
            }
        }
        else {
            this.animationNormalTimer.count(this.countBy);
        }
    }
    /**
     * @memberof Animation
     * @description Plays the Animation using the internal reverse timer
     * @since 2.0.0
     */
    playReverse() {
        this.countBy = this.config.useDelta
            ? this.game.deltaTime
            : 1000 / this.config.frameRate;
        if (this.delayTimer) {
            this.delayTimer.count(this.countBy);
            if (this.delayTimer.done) {
                this.animationReverseTimer.count(this.countBy);
            }
        }
        else {
            this.animationReverseTimer.count(this.countBy);
        }
    }
    /**
     * @memberof Animation
     * @description Pauses the Animation using the internal normal timer
     * @since 2.0.0
     */
    pause() {
        this.animationNormalTimer.pause();
    }
    /**
     * @memberof Animation
     * @description Pauses the Animation using the internal reverse timer
     * @since 2.0.0
     */
    pauseReverse() {
        this.animationReverseTimer.pause();
    }
    /**
     * @memberof Animation
     * @description REsumes the Animation using the internal normal timer
     * @since 2.0.0
     */
    resume() {
        this.animationNormalTimer.resume();
    }
    /**
     * @memberof Animation
     * @description Resumes the Animation using the internal reverse timer
     * @since 2.0.0
     */
    resumeReverse() {
        this.animationReverseTimer.resume();
    }
    /**
     * @memberof Animation
     * @description Stops/Cancels the Animation using the internal normal timer
     * @since 2.0.0
     */
    stop() {
        this.animationNormalTimer.stop();
    }
    /**
     * @memberof Animation
     * @description Stops/Cancels the Animation using the internal reverse timer
     * @since 2.0.0
     */
    stopReverse() {
        this.animationReverseTimer.stop();
    }
    /**
     * @memberof Animation
     * @description Restarts the Animation using the internal normal timer
     * @since 2.0.0
     */
    restart() {
        this.animationNormalTimer.reset();
    }
    /**
     * @memberof Animation
     * @description Restarts the Animation using the internal reverse timer
     * @since 2.0.0
     */
    restartReverse() {
        this.animationReverseTimer.reset();
    }
}

;// CONCATENATED MODULE: ./lib/core/animation/animationState.js
/**
 * @class AnimationState
 * @classdesc Creates a DuckEngine AnimationState
 * @description The AnimationState Class. Holds information that can be used and added by StateMachine
 * @since 2.0.0
 */
class AnimationState {
    /**
     * @constructor AnimationState
     * @param {string} key The name/key of the AnimationState, used to identify between states, the same as the Animation
     * @param {Animation} animation The Animation to play by the AnimationState
     * @param {Vector2} vector The Vector2 position of the AnimationState on an imaginary 2D Plane, used to travel in between states by StateMachine
     * @param {boolean} autoAdvance Determines whether the AnimationState auto advances to another AnimationState on completion
     * @since 2.0.0
     */
    constructor(key, animation, vector, autoAdvance) {
        this.key = key;
        this.animation = animation;
        this.autoAdvance = autoAdvance;
        this.vector = vector;
        this.connections = [];
    }
}

;// CONCATENATED MODULE: ./lib/core/animation/stateMachine.js

class StateMachine {
    /**
     * @memberof StateMachine
     * @description Creates a StateMachine instance
     * @param {Duck.Types.StateMachine.Config} config StateMachine configuration
     * @param {Animation[]} animations All Animations needed by the StateMachine
     * @since 2.0.0
     */
    constructor(config, animations) {
        this.config = config;
        this.animations = animations;
        this.connections = [];
        // connect from base connections config
        this.config.connections.forEach((base) => {
            if (base.connType === 'one') {
                this.connectTo(base.from, base.to);
            }
            else {
                this.connectLoop(base.from, base.to);
            }
        });
        this.currentState = this.connections.find((conn) => conn.find((state) => state.key === this.config.defaultState));
    }
    /**
     * @memberof StateMachine
     * @description Creates an AnimationState for both passed and values and makes a connection between from -> to
     * @param {Duck.Types.StateMachine.ConnectionBaseValue} from AnimationState base config to connect from
     * @param {Duck.Types.StateMachine.ConnectionBaseValue} to AnimationState base config to connect to
     * @since 2.0.0
     */
    connectTo(from, to) {
        const fromAnim = this.animations.find((anim) => anim.config.key === from.key);
        const toAnim = this.animations.find((anim) => anim.config.key === to.key);
        if (fromAnim && toAnim) {
            const fromState = new AnimationState(fromAnim.key, fromAnim, from.vector, from.autoAdvance || false);
            const toState = new AnimationState(toAnim.key, toAnim, to.vector, to.autoAdvance || false);
            this.connections.push([fromState, toState]);
        }
    }
    /**
     * @memberof StateMachine
     * @description Creates an AnimationState for both passed and values and makes a connection between from -> to and to -> from
     * @param {Duck.Types.StateMachine.ConnectionBaseValue} from AnimationState base config to connect from
     * @param {Duck.Types.StateMachine.ConnectionBaseValue} to AnimationState base config to connect to
     * @since 2.0.0
     */
    connectLoop(from, to) {
        const fromAnim = this.animations.find((anim) => anim.config.key === from.key);
        const toAnim = this.animations.find((anim) => anim.config.key === to.key);
        if (fromAnim && toAnim) {
            const fromState = new AnimationState(fromAnim.key, fromAnim, from.vector, from.autoAdvance || false);
            const toState = new AnimationState(toAnim.key, toAnim, to.vector, to.autoAdvance || false);
            fromState.connections.push(toState);
            toState.connections.push(fromState);
            this.connections.push([fromState, toState], [toState, fromState]);
        }
    }
    /**
     * @memberof StateMachine
     * @description Travels along the imaginary 2D plane to find a connection based on other connections' vectors
     * @param {Vector2} vector Vector to match to another Connection that will set the currentState as
     * @since 2.0.0
     */
    travel(vector) {
        const state = this.connections.find((connArray) => connArray.find((state) => state.vector.equals(vector)));
        if (this.canTravel(state.key)) {
            this.currentState = state;
        }
    }
    /**
     * @memberof StateMachine
     * @description Checks if the currentState's connections include another AnimationState connection with a passed key
     * @param {string} dest The destination or key to find
     * @returns {boolean}
     * @since 2.0.0
     */
    canTravel(dest) {
        return this.currentState.connections.find((state) => state.key === dest)
            ? true
            : false;
    }
}

;// CONCATENATED MODULE: ./lib/core/animation/animationManager.js


class AnimationManager {
    /**
     * @constructor AnimationManager
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param {Sprite} sprite Sprite that the AnimationManager is attached to
     * @param {Duck.Types.Animation.Config} defaultAnimation Configuration to make a default animation of the Sprite
     * @since 2.0.0
     */
    constructor(game, scene, sprite, defaultAnimation) {
        this.game = game;
        this.scene = scene;
        this.sprite = sprite;
        this.animations = [];
        this.currentAnimation = this.add(defaultAnimation);
    }
    /**
     * @memberof AnimationManager
     * @description Creates a StateMachine
     * @param {Duck.Types.StateMachine.Config} config StateMachine config
     * @param {Animation[]} animations Animations to add to the StateMachine
     * @returns {StateMachine}
     * @since 2.0.0
     */
    createStateMachine(config, animations) {
        return new StateMachine(config, animations);
    }
    /**
     * @memberof AnimationManager
     * @description Sets the current animation
     * @param {string} key Key of the Animation to set as the currentAnimation
     * @since 2.0.0
     */
    setCurrentAnimation(key) {
        this.currentAnimation = this.get(key);
    }
    /**
     * @memberof AnimationManager
     * @description Adds and creates a new Animation
     * @param {string} key Key of the Animation to set as the currentAnimation
     * @returns {Animation}
     * @since 2.0.0
     */
    add(config) {
        const anim = new animation_Animation(config, this.game, this.scene, this.sprite);
        this.animations.push(anim);
        return anim;
    }
    /**
     * @memberof AnimationManager
     * @description Removes an Animation from the array of animations (AnimationManager#animations)
     * @param {string} key Key of the Animation to remove
     * @since 2.0.0
     */
    remove(key) {
        const anim = this.get(key);
        if (anim) {
            this.animations.splice(this.getIndex(key));
        }
    }
    /**
     * @memberof AnimationManager
     * @description Finds and returns an Animation from the array of animations (AnimationManager#animations)
     * @param {string} key Key of the Animation find
     * @returns {Animation | undefined}
     * @since 2.0.0
     */
    get(key) {
        return this.animations.find((anim) => anim.key === key);
    }
    /**
     * @memberof AnimationManager
     * @description Finds and returns the index of an Animation from the array of animations (AnimationManager#animations)
     * @param {string} key Key of the Animation find the index of
     * @returns {number}
     * @since 2.0.0
     */
    getIndex(key) {
        return this.animations.findIndex((anim) => anim.key === key);
    }
    /**
     * @memberof AnimationManager
     * @description Clears all animations
     * @since 2.0.0
     */
    clear() {
        this.animations = [];
    }
    /**
     * @memberof AnimationManager
     * @description Sets the current animation repeat
     * @param {number} repeat
     * @since 2.0.0
     */
    setRepeat(repeat) {
        this.currentAnimation.setRepeat(repeat);
    }
    /**
     * @memberof AnimationManager
     * @description Sets the current animation reversed repeat
     * @param {number} repeat
     * @since 2.0.0
     */
    setRepeatReverse(repeat) {
        this.currentAnimation.setRepeatReverse(repeat);
    }
    /**
     * @memberof AnimationManager
     * @description Plays the current animation normally
     * @since 2.0.0
     */
    play() {
        this.currentAnimation.play();
    }
    /**
     * @memberof AnimationManager
     * @description Plays the current animation reversely
     * @since 2.0.0
     */
    playReverse() {
        this.currentAnimation.playReverse();
    }
    /**
     * @memberof AnimationManager
     * @description Pauses the current animation normally
     * @since 2.0.0
     */
    pause() {
        this.currentAnimation.pause();
    }
    /**
     * @memberof AnimationManager
     * @description Pauses the current animation reversely
     * @since 2.0.0
     */
    pauseReverse() {
        this.currentAnimation.pause();
    }
    /**
     * @memberof AnimationManager
     * @description Resumes the current animation normally
     * @since 2.0.0
     */
    resume() {
        this.currentAnimation.resume();
    }
    /**
     * @memberof AnimationManager
     * @description Resumes the current animation reversely
     * @since 2.0.0
     */
    resumeReverse() {
        this.currentAnimation.resumeReverse();
    }
    /**
     * @memberof AnimationManager
     * @description Stops the current animation normally
     * @since 2.0.0
     */
    stop() {
        this.currentAnimation.stop();
    }
    /**
     * @memberof AnimationManager
     * @description Stops the current animation reversely
     * @since 2.0.0
     */
    stopReverse() {
        this.currentAnimation.stopReverse();
    }
    /**
     * @memberof AnimationManager
     * @description Restarts the current animation normally
     * @since 2.0.0
     */
    restart() {
        this.currentAnimation.restart();
    }
    /**
     * @memberof AnimationManager
     * @description Restarts the current animation reversely
     * @since 2.0.0
     */
    restartReverse() {
        this.currentAnimation.restartReverse();
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/sprite.js



/**
 * @class Sprite
 * @classdesc Creates a DuckEngine Sprite
 * @description The Sprite Class. Represents a gameobject image
 * @since 1.0.0-beta
 */
class Sprite extends GameObject {
    /**
     * @constructor Sprite
     * @description Creates a Sprite instance
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of Sprite
     * @param {number} h Height of Sprite
     * @param {string} textureKey Key of a preloaded texture sheet
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param {number} [currentRow] The default row to use for the spritesheet texture
     * @param {number} [currentCol] The default column to use for the spritesheet texture
     * @since 1.0.0-beta
     */
    constructor(x, y, w, h, textureKey, game, scene, currentRow, currentCol) {
        super('sprite', x, y, w, h, 0, scene.loader.textureStack.find((t) => t.key === textureKey).value, game, scene);
        this.currentRow = currentRow;
        this.currentCol = currentCol;
        const defaultAnimConfig = {
            key: '__default',
            frameRate: 0,
            frames: [
                {
                    col: this.currentCol,
                    row: this.currentRow,
                },
            ],
        };
        this.anims = new AnimationManager(this.game, this.scene, this, defaultAnimConfig);
        this.defaultAnim = this.anims.get('__default');
    }
    /**
     * @description Draws the sprite.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            if (this.texture.dataType === 'sheet') {
                this.game.renderer.drawSprite(this.position.x, this.position.y, this.w, this.h, this.texture, this.currentRow, this.currentCol);
            }
            else {
                this.game.renderer.drawSprite(this.position.x, this.position.y, this.w, this.h, this.texture);
            }
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined. HTMLCanvasElement is undefined.');
        }
    }
    /**
     * @memberof Sprite
     * @description Sets the scale of the Sprite
     * @param {Duck.Types.Misc.Scale} scale
     * @since 1.0.0-beta
     */
    setScale(scale) {
        if (scale.width) {
            this.w = scale.width;
            this.texture.setScale({
                width: this.w,
            });
        }
        if (scale.height) {
            this.h = scale.height;
            this.texture.setScale({
                height: this.h,
            });
        }
    }
    /**
     * @memberof Sprite
     * @description Sets the image path of the sprite
     * @param {string} imgpath Image Path
     * @since 1.0.0-beta
     */
    setImagePath(imgpath) {
        this.texture.texture.src = imgpath;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    setFillColor(color) {
        new debug.Warn('Cannot fill color of a sprite. Changed the image path instead. Use setImagePath instead.');
    }
}

;// CONCATENATED MODULE: ./lib/helper/color/getValuesHSL.js
function getValuesHSL(hsl) {
    let res = hsl
        .substring(4, hsl.length - 1)
        .replace(/ /g, '')
        .replace(/%/g, '')
        .split(',');
    return res;
}

;// CONCATENATED MODULE: ./lib/helper/color/isHex.js
function isHex(str) {
    return /^#[0-9A-F]{6}|#[0-9A-F]{3}$/i.test(str);
}

;// CONCATENATED MODULE: ./lib/helper/color/hexToRGBA.js


function hexToRGBA(hex, alpha) {
    if (isHex(hex)) {
        hex = hex.replace('#', '');
        if (hex.length === 3) {
            hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
        }
        const [r, g, b] = hex.match(/\w\w/g).map((x) => parseInt(x, 16));
        return `rgba(${r},${g},${b},${alpha})`;
    }
    else {
        new debug.Error('Cannot convert hex to RGBA. Invalid hex.');
        return 'rgba(0, 0, 0, 1)';
    }
}

;// CONCATENATED MODULE: ./lib/helper/color/hslToRGB.js
function hslToRGB(h, s, l) {
    let r, g, b;
    h = parseInt(h);
    s = parseInt(s);
    l = parseInt(l);
    if (s === 0) {
        r = g = b = l; // achromatic
    }
    else {
        const hue2rgb = function hue2rgb(p, q, t) {
            if (t < 0)
                t += 1;
            if (t > 1)
                t -= 1;
            if (t < 1 / 6)
                return p + (q - p) * 6 * t;
            if (t < 1 / 2)
                return q;
            if (t < 2 / 3)
                return p + (q - p) * (2 / 3 - t) * 6;
            return p;
        };
        const q = l < 0.5 ? l * (1 + s) : l + s - l * s;
        const p = 2 * l - q;
        r = hue2rgb(p, q, h + 1 / 3);
        g = hue2rgb(p, q, h);
        b = hue2rgb(p, q, h - 1 / 3);
    }
    return `rgb(${Math.round(r * 255)}, ${Math.round(g * 255)}, ${Math.round(b * 255)})`;
}

;// CONCATENATED MODULE: ./lib/helper/color/isHSL.js
function isHSL(str) {
    return /hsl[(]\s*0*(?:[12]?\d{1,2}|3(?:[0-5]\d|60))\s*(?:\s*,\s*0*(?:\d\d?(?:\.\d+)?\s*%|\.\d+\s*%|100(?:\.0*)?\s*%)){2}\s*[)]$/.test(str);
}

;// CONCATENATED MODULE: ./lib/helper/color/isRGB.js
function isRGB(str) {
    return /^rgb[(](?:\s*0*(?:\d\d?(?:\.\d+)?(?:\s*%)?|\.\d+\s*%|100(?:\.0*)?\s*%|(?:1\d\d|2[0-4]\d|25[0-5])(?:\.\d+)?)\s*(?:,(?![)])|(?=[)]))){3}[)]$/.test(str);
}

;// CONCATENATED MODULE: ./lib/helper/color/rgbToRGBA.js
function rgbToRGBA(color, alpha) {
    let new_col = '';
    new_col = color.replace(/rgb/i, 'rgba');
    new_col = new_col.replace(/\)/i, `,${alpha})`);
    return new_col;
}

;// CONCATENATED MODULE: ./lib/helper/color/convertColorToRGBA.js







function convertColorToRGBA(color, alpha) {
    let res = '';
    if (isHex(color)) {
        res = hexToRGBA(color, alpha);
    }
    if (isRGB(color)) {
        res = rgbToRGBA(color, alpha);
    }
    if (isHSL(color)) {
        const values = getValuesHSL(color);
        const a = hslToRGB(values[0], values[1], values[2]);
        res = rgbToRGBA(a, alpha);
    }
    return res;
}

;// CONCATENATED MODULE: ./lib/core/lights/staticLight.js




/**
 * @class StaticLight
 * @classdesc Creates a DuckEngine StaticLight
 * @description The StaticLight Class. A static light
 * @extends GameObject<'color'>
 * @since 1.0.0-beta
 */
class StaticLight extends GameObject {
    /**
     * @constructor StaticLight
     * @description Creates an instance of a StaticLight
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} r Radius
     * @param {string} fillColor Color
     * @param {Duck.Types.Helper.AlphaRange} alpha Alpha
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(x, y, r, fillColor, alpha, game, scene) {
        super('circle', x, y, 0, 0, r, Texture.fromColor(fillColor, r, r), game, scene);
        this.r = r;
        this.alpha = alpha;
        this.game = game;
        this.visible = true;
        this.zIndex = 2;
        this.color = fillColor;
        // convert all colors to RGBA
        this.color = convertColorToRGBA(this.color, this.alpha);
        this.texture.texture = this.color;
    }
    /**
     * @description Draws the StaticLight.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            this.game.renderer.setBlendMode('lighten');
            this.game.renderer.drawCircle(this.position.x, this.position.y, this.r, this.texture.texture);
            this.game.renderer.setBlendMode('source-over');
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined. Canvas is undefined.');
        }
    }
    /**
     * @memberof StaticLight
     * @description Sets the fillColor with alpha
     * @param  {string} color Fill Color
     * @param  {Duck.Types.Helper.AlphaRange} alpha Alpha
     * @since 2.0.0
     */
    setFillColorAlpha(color, alpha) {
        this.color = convertColorToRGBA(color, alpha);
    }
}

;// CONCATENATED MODULE: ./lib/core/events/main/displayEvents.js
const DisplayEvents = {
    SET_SCALE: 'DISPLAY_SET_SCALE',
    FULLSCREEN: 'DISPLAY_FULLSCREEN',
    UNFULLSCREEN: 'DISPLAY_UNFULLSCREEN',
    FOCUS: 'DISPLAY_FOCUS',
    BLUR: 'DISPLAY_BLUR',
};
/* harmony default export */ const displayEvents = (DisplayEvents);

;// CONCATENATED MODULE: ./lib/core/events/main/gameEvents.js
const GameEvents = {
    FOCUS: 'GAME_FOCUS',
    BLUR: 'GAME_BLUR',
    START: 'GAME_START',
    STOP: 'GAME_STOP',
    LOAD_BEGIN: 'GAME_LOAD_BEGIN',
    LOAD_FINISH: 'GAME_LOAD_FINISH',
    SCENE_ADD: 'GAME_SCENE_ADD',
    SCENE_REMOVE: 'GAME_SCENE_REMOVE',
    LOAD_SCENE: 'GAME_LOAD_SCENE',
    SWITCH_SCENE: 'GAME_SWITCH_SCENE',
    SHOW_SCENE: 'GAME_SHOW_SCENE',
    DRAW_SPLASH: 'GAME_DRAW_SPLASH',
    CLEAR_FRAME: 'GAME_CLEAR_FRAME',
    SET_SCALE: 'GAME_SET_SCALE',
    SET_BACKGROUND: 'GAME_SET_BACKGROUND',
    SYNC_CACHE: 'GAME_SYNC_CACHE',
    LOCK_POINTER: 'GAME_LOCK_POINTER',
    UNLOCK_POINTER: 'GAME_UNLOCK_POINTER',
    CONTEXT_LOST: 'GAME_CONTEXT_LOST',
    CONTEXT_RESTORED: 'GAME_CONTEXT_RESTORED',
};
/* harmony default export */ const gameEvents = (GameEvents);

;// CONCATENATED MODULE: ./lib/core/events/main/rendererEvents.js
const RendererEvents = {
    PIPELINE_POOL: 'RENDERER_PIPELINE_POOL',
    CLEAR_FRAME: 'CLEAR_FRAME',
};
/* harmony default export */ const rendererEvents = (RendererEvents);

;// CONCATENATED MODULE: ./lib/core/events/other/buttonEvents.js
const ButtonEvents = {
    CLICK: 'BUTTON_CLICK',
    HOVER: 'BUTTON_HOVER',
    NOTHOVER: 'BUTTON_NOTHOVER',
};
/* harmony default export */ const buttonEvents = (ButtonEvents);

;// CONCATENATED MODULE: ./lib/core/events/other/cutsceneEvents.js
const CutsceneEvents = {
    START: 'CUTSCENE_START',
    END: 'CUTSCENE_END',
    NEXT: 'CUTSCENE_NEXT',
};
/* harmony default export */ const cutsceneEvents = (CutsceneEvents);

;// CONCATENATED MODULE: ./lib/core/events/other/groupEvents.js
const GroupEvents = {
    ADD: 'GROUP_ADD',
    REMOVE: 'GROUP_REMOVE',
};
/* harmony default export */ const groupEvents = (GroupEvents);

;// CONCATENATED MODULE: ./lib/core/events/events.js






const EVENTS = {
    BUTTON: buttonEvents,
    GROUP: groupEvents,
    CUTSCENE: cutsceneEvents,
    GAME: gameEvents,
    RENDERER: rendererEvents,
    DISPLAY: displayEvents,
};
/* harmony default export */ const events = (EVENTS);

;// CONCATENATED MODULE: ./lib/core/gameobjects/ui/ui.js

class UI extends GameObject {
    constructor(shape, x, y, w, h, r, texture, game, scene) {
        super(shape, x, y, w, h, r, texture, game, scene);
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/ui/button.js






/**
 * @class Button
 * @classdesc Creates a DuckEngine Button
 * @description The Button Class. Acts like a Button
 * @extends UI
 * @since 1.0.0
 */
class Button extends UI {
    /**
     * @constructor
     * @description Creates a Button instance
     * @param {Duck.Types.UI.Button.Shape} shape Shape of the button, 'rect', 'roundrect', or 'sprite'
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width
     * @param {number} h Height
     * @param {number} r Radius
     * @param {string} fillColorOrIMGPath Fill Color or Image path of the button
     * @param {Text} text Text instance to render on top of the button
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0
     */
    constructor(shape, x, y, w, h, r, fillColorOrIMGPath, text, game, scene) {
        super(shape, x, y, w, h, r, Texture.fromEither(fillColorOrIMGPath, w, h), game, scene);
        this.shape = shape;
        this.text = text;
        this.hovering = false;
        this.zIndex = Duck.Layers.Rendering.zIndex.button;
        if (this.game.canvas) {
            this.game.canvas.addEventListener('click', (e) => {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const rect = this.game.canvas.getBoundingClientRect();
                const mousePos = {
                    x: e.clientX - rect.left,
                    y: e.clientY - rect.top,
                };
                const buttonPos = { x: this.position.x, y: this.position.y };
                if (this.scene.currentCamera) {
                    const coords = this.scene.currentCamera.worldToScreen(this.position.x, this.position.y, this);
                    buttonPos.x = coords.position.x;
                    buttonPos.y = coords.position.y;
                }
                if (rectToRectIntersect({
                    position: buttonPos,
                    w: this.w,
                    h: this.h,
                }, {
                    position: { x: mousePos.x, y: mousePos.y },
                    w: 1,
                    h: 1,
                }) &&
                    this.scene.visible) {
                    this.game.eventEmitter.emit(events.BUTTON.CLICK, {
                        x: mousePos.x,
                        y: mousePos.y,
                        type: events.BUTTON.CLICK,
                    });
                }
            });
            // hover
            this.game.canvas.addEventListener('mousemove', (e) => {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const rect = this.game.canvas.getBoundingClientRect();
                const mousePos = {
                    x: e.clientX - rect.left,
                    y: e.clientY - rect.top,
                };
                const buttonPos = { x: this.position.x, y: this.position.y };
                if (this.scene.currentCamera) {
                    const coords = this.scene.currentCamera.worldToScreen(this.position.x, this.position.y, this);
                    buttonPos.x = coords.position.x;
                    buttonPos.y = coords.position.y;
                }
                if (rectToRectIntersect({
                    position: buttonPos,
                    w: this.w,
                    h: this.h,
                }, {
                    position: { x: mousePos.x, y: mousePos.y },
                    w: 1,
                    h: 1,
                }) &&
                    this.scene.visible) {
                    this.game.eventEmitter.emit(events.BUTTON.HOVER, {
                        x: mousePos.x,
                        y: mousePos.y,
                        type: events.BUTTON.HOVER,
                    });
                    this.hovering = true;
                }
                else if (this.scene.visible) {
                    this.game.eventEmitter.emit(events.BUTTON.NOTHOVER, {
                        x: mousePos.x,
                        y: mousePos.y,
                        type: events.BUTTON.NOTHOVER,
                    });
                    this.hovering = false;
                }
            });
        }
        else {
            new debug.Error('Cannot add event listeners to canvas for mouseover and click for button. Canvas element is undefined or null.');
        }
    }
    /**
     * @description Draws the button.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            switch (this.shape) {
                case 'rect':
                    this.game.renderer.drawRect(this.position.x, this.position.y, this.w, this.h, this.texture.texture);
                    break;
                case 'roundrect':
                    this.game.renderer.drawRoundRect(this.position.x, this.position.y, this.w, this.h, this.r, this.texture.texture);
                    break;
                case 'sprite':
                    this.game.renderer.drawSprite(this.position.x, this.position.y, this.w, this.h, this.texture);
                    break;
                default:
                    new debug.Error('Cannot draw button. Shape must be a rect, roundrect, or sprite.');
                    break;
            }
        }
    }
    /**
     * @memberof Button
     * @description Adds an event listener to the button
     * @param {Duck.Types.UI.Button.ListenerType} type Listener Type, type of event
     * @param {Duck.Types.UI.Button.ListenerFunc} func Callback function, called on event
     * @since 1.0.0
     */
    on(type, func) {
        this.game.eventEmitter.on(`BUTTON_${type}`, func);
    }
    /**
     * @memberof Button
     * @description Removes an event listener from the button
     * @param {Duck.Types.UI.Button.ListenerType} type Listener Type, type of event
     * @since 1.0.0
     */
    off(type) {
        this.game.eventEmitter.off(`BUTTON_${type}`);
    }
}

;// CONCATENATED MODULE: ./lib/utils/extractNumbers.js
function extractNumbers(string) {
    var _a;
    return Number((_a = string.match(/\d/g)) === null || _a === void 0 ? void 0 : _a.join(''));
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/ui/text.js




/**
 * @class Text
 * @classdesc Creates a DuckEngine Text
 * @description The Button Class. Renders Text to the canvas
 * @extends UI
 * @since 1.0.0-beta
 */
class Text extends UI {
    /**
     * @constructor Text
     * @description Creates a Text instance.
     * @param {string} text Text string
     * @param {Duck.Types.UI.Text.Config} config Text configuration, styles, position and more
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(text, config, game, scene) {
        super('rect', config.x, config.y, 0, 0, 0, Texture.fromColor(text, 0, 0), game, scene);
        this.text = text;
        this.config = config;
        this.game = game;
        // set w and h
        this.w = this.game.renderer.measureText(config.styles.fontCSS, this.text).width;
        this.h = extractNumbers(config.styles.fontCSS);
        this.texture.setScale({
            width: this.w,
            height: this.h,
        });
        this.zIndex = Duck.Layers.Rendering.zIndex.text;
    }
    /**
     * @description Draws the text.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            this.game.renderer.setFont(this.config.styles.fontCSS);
            this.game.renderer.setLineWidth(this.config.styles.strokeWidth || 1);
            this.game.renderer.setStrokeColor(this.config.styles.strokeColor || '#000');
            this.game.renderer.setFillColor(this.config.styles.fillColor || '#000');
            if (this.config.method === 'draw') {
                this.game.renderer.drawText(this.text, this.position.x, this.position.y, this.config.styles.maxWidth);
            }
            if (this.config.method === 'stroke') {
                this.game.renderer.strokeText(this.text, this.position.x, this.position.y, this.config.styles.maxWidth);
            }
            if (this.config.method === 'draw-stroke') {
                this.game.renderer.drawText(this.text, this.position.x, this.position.y, this.config.styles.maxWidth);
                this.game.renderer.strokeText(this.text, this.position.x, this.position.y, this.config.styles.maxWidth);
            }
        }
    }
    /**
     * @memberof Text
     * @description Sets the text string
     * @param {string} text
     * @since 1.0.0-beta
     */
    setText(text) {
        this.text = text;
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/particles/particle.js






/**
 * @class Particle
 * @classdesc Creates a DuckEngine Particle
 * @description The Particle Class. A particle that can be emitted by a ParticleEmitter
 * @extends GameObject
 * @since 1.0.0
 */
class Particle extends GameObject {
    /**
     * @constructor Particle
     * @description Creates a Particle instance
     * @param {Duck.Types.Collider.ShapeString} shape Shape of the particle
     * @param {number} w Width of the particle
     * @param {number} h Height of the particle
     * @param {number} r Radius of the particle
     * @param {string} fillColorOrIMGPath Color to fill the particle with, can be an image path
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(shape, w, h, r, fillColorOrIMGPath, game, scene) {
        super(shape, 0, 0, w, h, r, Texture.fromEither(fillColorOrIMGPath, w, h), game, scene);
        this.originalFillColorOrIMGPath = fillColorOrIMGPath;
        this.w = w;
        this.h = h;
        this.r = r;
        this.floatVelocity = Vector2.ZERO;
        this.zIndex = Duck.Layers.Rendering.zIndex.particle;
        this.visible = false;
        // age
        this.age = 0;
        setInterval(() => {
            this.age++;
        }, 1000);
    }
    /**
     * @memberof Particle
     * @description Draws the particle. Do not call manually, called automatically in scene loop
     * @since 1.0.0-beta
     */
    _draw() {
        if (this.game.renderer.ctx) {
            switch (this.shape) {
                case 'circle':
                    this.game.renderer.drawCircle(this.position.x, this.position.y, this.r, this.texture.texture);
                    break;
                case 'rect':
                    this.game.renderer.drawRect(this.position.x, this.position.y, this.w, this.h, this.texture.texture);
                    break;
                case 'roundrect':
                    this.game.renderer.drawRoundRect(this.position.x, this.position.y, this.w, this.h, this.r, this.texture.texture);
                    break;
                case 'sprite':
                    this.game.renderer.drawSprite(this.position.x, this.position.y, this.w, this.h, this.texture.texture);
                    break;
                default:
                    if (this.game.config.debug) {
                        new debug.Warn('Cannot draw Particle. Particle Shape is not a "circle", "rect", "roundrect", or "sprite".');
                    }
                    break;
            }
        }
        else {
            new debug.Error('Cannot draw particle. CanvasRenderingContext2D is undefined.');
        }
    }
    /**
     * @memberof Particle
     *
     * *Modified from gameobject*
     *
     * @description Updates the gameobject's position by the velocity. Sets velocity to 0 on every tick.
     * DO NOT CALL MANUALLY, CALLED IN SCENE.__tick(deltaTime)
     * @since 2.0.0
     */
    _update() {
        var _a;
        this.position.x += this.velocity.x * this.game.smoothDeltaTime;
        this.position.y += this.velocity.y * this.game.smoothDeltaTime;
        // float
        this.position.x += this.floatVelocity.x * this.game.smoothDeltaTime;
        this.position.y += this.floatVelocity.y * this.game.smoothDeltaTime;
        // clamp to bounds
        this.position.x = clamp(this.position.x, this.bounds.x, this.bounds.w);
        this.position.y = clamp(this.position.y, this.bounds.y, this.bounds.h);
        // set to none
        this.velocity.x = 0;
        this.velocity.y = 0;
        // don't round pixels for particles
        // apply gravity
        if ((_a = this.game.config.physics) === null || _a === void 0 ? void 0 : _a.gravity) {
            if (this.options.type === 'KinematicBody' ||
                this.options.type === 'RigidBody') {
                this.applyGravity(Vector2.fromVector2Like(this.game.config.physics.gravity));
            }
        }
        // update attached children position
        this.attachedChildren.forEach((object) => {
            const pos = this.position.clone();
            pos.subtract(object.attachOffset);
            object.position = pos;
            if (object.hitbox) {
                object.hitbox.position = object.position
                    .clone()
                    .add(object.hitbox.offset);
            }
        });
    }
    /**
     * @memberof Particle
     * @description Sets the particle's image src. Only works if particle's shape was initially 'sprite'
     * @param {string} imagePath Image path
     * @since 1.0.0-beta
     */
    setImagePath(imagePath) {
        this.texture.setImagePath(imagePath);
    }
}

;// CONCATENATED MODULE: ./lib/core/map/map.js
/* eslint-disable @typescript-eslint/no-empty-function */


/**
 * @class Map
 * @classdesc Creates a DuckEngine Map
 * @description The Map Class. Base Map class, extends by TileMap
 * @since 1.2.0
 */
class map_Map {
    /**
     * @constructor Map
     * @description Creates a Map instance.
     * @param {TileLayer[]} tileLayers An array of TileLayers
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(origin, tileLayers, game, scene) {
        this.id = uniqueID();
        this.shape = 'map';
        this.origin = Vector2.fromVector2Like(origin);
        this.tileLayers = tileLayers;
        this.game = game;
        this.scene = scene;
        this.visible = true;
        this.zIndex = 2;
        this.culled = true;
    }
    /**
     * @description Draws the map.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() { }
}

;// CONCATENATED MODULE: ./lib/core/camera/camera.js
// core code from : https://github.com/robashton/camera
// mostly edited
















/**
 * @class Camera
 * @classdesc Creates a DuckEngine Camera
 * @description The Camera Class. Creates and uses a viewport that acts like a camera
 * @since 1.0.0-beta
 */
class Camera {
    /**
     * @constructor Camera
     * @description Creates a Camera instance.
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(game, scene) {
        this.lerpX = 1;
        this.lerpY = 1;
        this.game = game;
        this.scene = scene;
        this.distance = 1000.0;
        this.isMain = false;
        if (scene.mainCamera === this) {
            this.isMain = true;
        }
        this.lookAt = [0, 0];
        this.fieldOfView = Math.PI / 4.0;
        this.viewport = {
            left: 0,
            right: 0,
            top: 0,
            bottom: 0,
            w: 0,
            h: 0,
            scale: [1.0, 1.0],
        };
        this.bounds;
        this.following;
        this.updateViewport();
    }
    /**
     * @memberof Camera
     * @description Begins camera path
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME -> SCENE LOOP AUTOMATICALLY
     *
     * @since 1.0.0-beta
     */
    begin() {
        this.game.renderer.save();
        this.applyScale();
        this.applyTranslation();
        // follow
        if (this.following) {
            if (this.following.shape === 'rect') {
                if (this.bounds) {
                    if (rectToRectIntersect(this.following, this.bounds)) {
                        this.lookAt[0] = lerp(this.lookAt[0], this.following.getCenterX(), this.lerpX);
                        this.lookAt[1] = lerp(this.lookAt[1], this.following.getCenterY(), this.lerpY);
                    }
                }
                else {
                    this.lookAt[0] = lerp(this.lookAt[0], this.following.getCenterX(), this.lerpX);
                    this.lookAt[1] = lerp(this.lookAt[1], this.following.getCenterY(), this.lerpY);
                }
            }
            if (this.following.shape === 'circle') {
                if (this.bounds) {
                    if (circleRectCollision(this.following, this.bounds)) {
                        this.lookAt[0] = lerp(this.lookAt[0], this.following.getCenterX(), this.lerpX);
                        this.lookAt[1] = lerp(this.lookAt[1], this.following.getCenterY(), this.lerpY);
                    }
                }
                else {
                    this.lookAt[0] = lerp(this.lookAt[0], this.following.getCenterX(), this.lerpX);
                    this.lookAt[1] = lerp(this.lookAt[1], this.following.getCenterY(), this.lerpY);
                }
            }
            this.updateViewport();
        }
    }
    /**
     * @memberof Camera
     * @description Ends camera path
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME -> SCENE LOOP AUTOMATICALLY
     *
     * @since 1.0.0-beta
     */
    end() {
        this.game.renderer.restore();
    }
    applyScale() {
        this.game.renderer.scale(this.viewport.scale[0], this.viewport.scale[1]);
    }
    applyTranslation() {
        this.game.renderer.translate(-this.viewport.left, -this.viewport.top);
    }
    updateViewport() {
        if (this.game.renderer.ctx) {
            // dpr scaling
            let cWidth = this.game.canvas.width;
            let cHeight = this.game.canvas.height;
            if (this.game.config.dprScale && window.devicePixelRatio !== 1) {
                cWidth = Number(this.game.canvas.style.width.replace('px', ''));
                cHeight = Number(this.game.canvas.style.height.replace('px', ''));
                // set zoom for dpr scaling
                this.distance = 1000 / window.devicePixelRatio;
            }
            this.aspectRatio = cWidth / cHeight;
            this.viewport.w = this.distance * Math.tan(this.fieldOfView);
            this.viewport.h = this.viewport.w / this.aspectRatio;
            this.viewport.left = this.lookAt[0] - this.viewport.w / 2.0;
            this.viewport.top = this.lookAt[1] - this.viewport.h / 2.0;
            this.viewport.right = this.viewport.left + this.viewport.w;
            this.viewport.bottom = this.viewport.top + this.viewport.h;
            this.viewport.scale[0] = cWidth / this.viewport.w;
            this.viewport.scale[1] = cHeight / this.viewport.h;
        }
        else {
            new debug.Error('Cannot update camera. CanvasRenderingContext2D is undefined.');
        }
    }
    /**
     * @memberof Camera
     * @description Sets the zoom
     * @param {number} z Zoom value
     * @since 1.0.0-beta
     */
    setZoom(z) {
        this.distance = z;
        this.updateViewport();
    }
    /**
     * @memberof Camera
     * @description Sets the zoom smoothly
     * @param {number} intervalMS How often zoom is modified by smoothValue
     * @param {number} smoothValue The number that is added to zoom on an interval
     * @param {number} z Target Zoom value
     * @since 1.0.0-beta
     */
    setZoomSmooth(intervalMS, smoothValue, z) {
        let operation = 'add';
        if (this.distance < z) {
            operation = 'add';
        }
        else {
            operation = 'subtract';
        }
        const int = setInterval(() => {
            if (operation === 'add') {
                if (this.distance < z) {
                    this.distance += smoothValue;
                }
                else {
                    clearInterval(int);
                    if (this.game.config.debug) {
                        new debug.Log('Reached target camera Zoom with setZoomSmooth');
                    }
                }
            }
            else {
                if (this.distance > z) {
                    this.distance -= smoothValue;
                }
                else {
                    clearInterval(int);
                    if (this.game.config.debug) {
                        new debug.Log('Reached target camera Zoom with setZoomSmooth');
                    }
                }
            }
        }, intervalMS);
    }
    /**
     * @memberof Camera
     * @description Moves the camera to a position
     * @param {number} x X position
     * @param {number} y Y position
     * @since 1.0.0-beta
     */
    moveTo(x, y) {
        this.lookAt[0] = x;
        this.lookAt[1] = y;
        this.updateViewport();
    }
    /**
     * @memberof Camera
     * @description Follows a GameObject
     * @param {Duck.TypeClasses.GameObjects.GameObject<Duck.Types.Texture.Type>} gameObject Game object to follow
     * @param {number} [lerpX=1] Lerp on the x axis, optional -> defaults: 1
     * @param {number} [lerpY=1] Lerp on the y axis, optional -> defaults: 1
     * @since 1.0.0-beta
     */
    startFollow(gameObject, lerpX = 1, lerpY = 1) {
        this.following = gameObject;
        this.lerpX = lerpX;
        this.lerpY = lerpY;
    }
    /**
     * @memberof Camera
     * @description Stops following a gameobject
     * @since 1.0.0-beta
     */
    stopFollow() {
        this.following = undefined;
    }
    /**
     * @memberof Camera
     * @description Transforms a coordinate pair from screen coordinates (relative to the canvas) into world coordinates
     * @param {number} x X position
     * @param {number} y Y position
     * @param {Duck.TypeClasses.GameObjects.GameObject<Duck.Types.Texture.Type>} obj GameObject
     * @since 1.0.0-beta
     */
    screenToWorld(x, y, obj) {
        obj.position.x = x / this.viewport.scale[0] + this.viewport.left;
        obj.position.y = y / this.viewport.scale[1] + this.viewport.top;
        return obj;
    }
    /**
     * @memberof Camera
     * @description Transforms a coordinate pair from world coordinates into screen coordinates (relative to the canvas) -
     * useful for interacting with a Button while having a camera
     * @param {number} x X position
     * @param {number} y Y position
     * @param {Duck.TypeClasses.GameObjects.GameObject<Duck.Types.Texture.Type>} obj GameObject
     * @since 1.0.0-beta
     */
    worldToScreen(x, y, obj) {
        obj.position.x = (x - this.viewport.left) * this.viewport.scale[0];
        obj.position.y = (y - this.viewport.top) * this.viewport.scale[1];
        return obj;
    }
    /**
     * @memberof Camera
     * @description Sets the FOV
     * @param {number} f FOV value
     * @since 1.0.0-beta
     */
    setFOV(f) {
        this.fieldOfView = f;
    }
    /**
     * @memberof Camera
     * @description Sets the FOV smoothly
     * @param {number} intervalMS How often FOV is modified by smoothValue
     * @param {number} smoothValue The number that is added to FOV on an interval
     * @param {number} f Target FOV value
     * @since 1.0.0-beta
     */
    setFOVSmooth(intervalMS, smoothValue, f) {
        let operation = 'add';
        if (this.fieldOfView < f) {
            operation = 'add';
        }
        else {
            operation = 'subtract';
        }
        const int = setInterval(() => {
            if (operation === 'add') {
                if (this.fieldOfView < f) {
                    this.fieldOfView += smoothValue;
                }
                else {
                    clearInterval(int);
                    if (this.game.config.debug) {
                        new debug.Log('Reached target camera FOV with setFOVSmooth');
                    }
                }
            }
            else {
                if (this.fieldOfView > f) {
                    this.fieldOfView -= smoothValue;
                }
                else {
                    clearInterval(int);
                    if (this.game.config.debug) {
                        new debug.Log('Reached target camera FOV with setFOVSmooth');
                    }
                }
            }
        }, intervalMS);
    }
    /**
     * @memberof Camera
     * @description Resets the FOV to the default value
     * @since 1.0.0-beta
     */
    resetFOV() {
        this.fieldOfView = Math.PI / 4.0;
    }
    /**
     * @memberof Camera
     * @description Resets the Zoom to the default value
     * @since 1.0.0-beta
     */
    resetZoom() {
        this.distance = 1000.0;
    }
    /**
     * @memberof Camera
     * @description Bounds of the Camera, an area the camera can be in
     * @param {{ position: { x:number; y:number }; w:number; h:number; }} bounds An object with the bounds
     * @since 1.0.0-beta
     */
    setBounds(bounds) {
        this.bounds = bounds;
    }
    /**
     * @memberof Camera
     * @description Shakes the camera
     * @param {number} intervalMS How often (in milliseconds) the passed value is added/subtracted from the Camera viewport position
     * @param {number} timeMS For how long in milliseconds the shake lasts
     * @param {number} v Value to be added/subtracted from the Camera viewport position
     * @since 1.0.0
     */
    shake(intervalMS, timeMS, v) {
        const int = setInterval(() => {
            const r = randomInt(1, 4);
            if (r === 1) {
                this.lookAt[0] += v;
            }
            if (r === 2) {
                this.lookAt[0] -= v;
            }
            if (r === 3) {
                this.lookAt[1] += v;
            }
            if (r === 2) {
                this.lookAt[1] += v;
            }
            this.updateViewport();
        }, intervalMS);
        setTimeout(() => {
            clearInterval(int);
        }, timeMS);
    }
    /**
     * @memberof Camera
     * @description Culls/Renders objects that are passed and does not render other object that are not passed
     * @param {Duck.Types.Renderable[]} renderableObjects Objects that should be culled/rendered
     * @param {Duck.Types.Camera.CullingOptions} [options] Options to modify how objects are culled, optional -> defaults:
     * { preserveVisibility: true, modifyPhysicsEnable: true }
     *
     * Notes:
     *  - Calls CanvasRenderer.pipeline.pool() ignoring the pool interval
     *
     * @since 2.0.0
     */
    cull(renderableObjects, options) {
        const preserveVisibility = options ? options.preserveVisibility : true;
        const modifyPhysicsEnable = options
            ? options.modifyPhysicsEnable
            : true;
        const visibleObjects = this.scene.displayList.visibilityFilter(true);
        const culledObjects = visibleObjects.filter((r) => renderableObjects.find((_r) => _r.id === r.id));
        const nonCulledObjects = visibleObjects.filter((r) => !culledObjects.includes(r));
        for (const culledObject of culledObjects) {
            // skip maps
            if (culledObject instanceof map_Map) {
                continue;
            }
            if (preserveVisibility) {
                culledObject.culled = true;
            }
            else {
                culledObject.visible = true;
            }
            if (modifyPhysicsEnable) {
                if (culledObject instanceof PhysicsBody) {
                    culledObject.enabled = true;
                }
            }
        }
        for (const nonCulledObject of nonCulledObjects) {
            // skip maps
            if (nonCulledObject instanceof map_Map) {
                continue;
            }
            if (preserveVisibility) {
                nonCulledObject.culled = false;
            }
            else {
                nonCulledObject.visible = false;
            }
            if (modifyPhysicsEnable) {
                if (nonCulledObject instanceof PhysicsBody) {
                    nonCulledObject.enabled = false;
                }
            }
        }
        // pool RendererPipeline
        this.game.renderer.pipeline.pool();
    }
    /**
     * @memberof Camera
     * @description A form of Frustum Culling that gets all objects visible to the player by the viewport's width and height and culls those objects
     * and does not render objects outside/not-visible to the player/camera
     * @param {Duck.Types.Camera.CullingOptions} [options] Options to modify how objects are culled, optional -> defaults:
     * { preserveVisibility: true, modifyPhysicsEnable: true }
     *
     * Notes:
     *  - Calls CanvasRenderer.pipeline.pool() ignoring the pool interval
     *
     * @since 2.0.0
     */
    autoCull(options) {
        const preserveVisibility = options ? options.preserveVisibility : true;
        const modifyPhysicsEnable = options
            ? options.modifyPhysicsEnable
            : true;
        const objects = this.scene.displayList.list;
        const culledObjects = objects.filter((r) => {
            if (r instanceof Rect ||
                r instanceof RoundRect ||
                r instanceof Sprite ||
                r instanceof Button ||
                r instanceof Text) {
                if (rectToRectIntersect(r, {
                    position: {
                        x: this.viewport.left,
                        y: this.viewport.top,
                    },
                    w: this.viewport.w,
                    h: this.viewport.h,
                })) {
                    return true;
                }
                else {
                    return false;
                }
            }
            if (r instanceof Circle ||
                r instanceof StaticLight ||
                r instanceof Particle) {
                if (circleRectCollision(r, {
                    position: {
                        x: this.viewport.left,
                        y: this.viewport.top,
                    },
                    w: this.viewport.w,
                    h: this.viewport.h,
                })) {
                    return true;
                }
                else {
                    return false;
                }
            }
            return false;
        });
        const nonCulledObjects = objects.filter((r) => !culledObjects.includes(r));
        for (const culledObject of culledObjects) {
            // skip maps
            if (culledObject instanceof map_Map) {
                continue;
            }
            if (preserveVisibility) {
                culledObject.culled = true;
            }
            else {
                culledObject.visible = true;
            }
            if (modifyPhysicsEnable) {
                if (culledObject instanceof PhysicsBody) {
                    culledObject.enabled = true;
                }
            }
        }
        for (const nonCulledObject of nonCulledObjects) {
            // skip maps
            if (nonCulledObject instanceof map_Map) {
                continue;
            }
            if (preserveVisibility) {
                nonCulledObject.culled = false;
            }
            else {
                nonCulledObject.visible = false;
            }
            if (modifyPhysicsEnable) {
                if (nonCulledObject instanceof PhysicsBody) {
                    nonCulledObject.enabled = false;
                }
            }
        }
        // pool RendererPipeline
        this.game.renderer.pipeline.pool();
    }
    /**
     * @memberof Camera
     * @description Returns the default zoom, also uses DPR scaling if enabled
     * @since 1.0.0
     */
    get defaultZoom() {
        if (this.game.config.dprScale) {
            if (this.game.config.debug) {
                new debug.Log('Getter defaultZoom returned default zoom with dpr scaling. (info)');
            }
            return 1000 / window.devicePixelRatio;
        }
        else {
            return 1000;
        }
    }
    /**
     * @memberof Camera
     * @description Returns the default FOV
     * @since 1.0.0
     */
    get defaultFOV() {
        return Math.PI / 4;
    }
}

;// CONCATENATED MODULE: ./lib/helper/version.js
/* harmony default export */ const version = ('2.1.0');

;// CONCATENATED MODULE: ./lib/helper/startup.js

/* harmony default export */ const startup = (`
🦆 DuckEngine - ${version} 🦆

https://github.com/ksplatdev/DuckEngine

MIT License

Copyright (c) 2021 Bleart Emini
`);

;// CONCATENATED MODULE: ./lib/helper/dprScale.js
// https://gist.github.com/callumlocke/cc258a193839691f60dd
function dprScale(canvas, context, width, height) {
    // assume the device pixel ratio is 1 if the browser doesn't specify it
    const devicePixelRatio = window.devicePixelRatio || 1;
    // determine the 'backing store ratio' of the canvas context
    const backingStoreRatio = context.webkitBackingStorePixelRatio ||
        context.mozBackingStorePixelRatio ||
        context.msBackingStorePixelRatio ||
        context.oBackingStorePixelRatio ||
        context.backingStorePixelRatio ||
        1;
    // determine the actual ratio we want to draw at
    const ratio = devicePixelRatio / backingStoreRatio;
    if (devicePixelRatio !== backingStoreRatio) {
        // set the 'real' canvas size to the higher width/height
        canvas.width = width * ratio;
        canvas.height = height * ratio;
        // ...then scale it back down with CSS
        canvas.style.width = width + 'px';
        canvas.style.height = height + 'px';
    }
    else {
        // this is a normal 1:1 device; just scale it simply
        canvas.width = width;
        canvas.height = height;
        canvas.style.width = '';
        canvas.style.height = '';
    }
    // scale the drawing context so everything will work at the higher ratio
    context.scale(ratio, ratio);
}

;// CONCATENATED MODULE: ./lib/core/events/eventEmitter.js
/* eslint-disable @typescript-eslint/no-explicit-any */
class EventEmitter {
    constructor() {
        this.callbacks = {};
    }
    on(event, cb) {
        if (!this.callbacks[event])
            this.callbacks[event] = [];
        this.callbacks[event].push(cb);
    }
    off(event) {
        if (this.callbacks[event]) {
            this.callbacks[event] = [];
        }
    }
    emit(event, ...args) {
        const cbs = this.callbacks[event];
        if (cbs) {
            cbs.forEach((cb) => {
                if (args) {
                    cb(...args);
                }
                else {
                    cb();
                }
            });
        }
    }
}

;// CONCATENATED MODULE: ./lib/utils/detectBrowser.js
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
//@ts-nocheck
function detectBrowser() {
    // Opera 8.0+
    const isOpera = (!!window.opr && !!opr.addons) ||
        !!window.opera ||
        navigator.userAgent.indexOf(' OPR/') >= 0;
    // Firefox 1.0+
    const isFirefox = typeof InstallTrigger !== 'undefined';
    // Safari 3.0+ "[object HTMLElementConstructor]"
    const isSafari = /constructor/i.test(window.HTMLElement) ||
        (function (p) {
            return p.toString() === '[object SafariRemoteNotification]';
        })(!window['safari'] ||
            (typeof safari !== 'undefined' &&
                window['safari'].pushNotification));
    // Internet Explorer 6-11
    const isIE = /*@cc_on!@*/  false || !!document.documentMode;
    // Edge 20+
    const isEdge = !isIE && !!window.StyleMedia;
    // Chrome 1 - 79
    const isChrome = !!window.chrome &&
        (!!window.chrome.webstore || !!window.chrome.runtime);
    // Edge (based on chromium) detection
    const isEdgeChromium = isChrome && navigator.userAgent.indexOf('Edg') !== -1;
    // Blink engine detection
    const isBlink = (isChrome || isOpera) && !!window.CSS;
    const res = {
        isOpera: isOpera ? 'Opera' : false,
        isFirefox: isFirefox ? 'Firefox' : false,
        isIE: isIE ? 'IE' : false,
        isEdge: isEdge ? 'Edge' : false,
        isEdgeChromium: isEdgeChromium ? 'EdgeChromium' : false,
        isBlink: isBlink ? 'Blink' : false,
        isSafari: isSafari ? 'Safari' : false,
    };
    return (res.isOpera ||
        res.isFirefox ||
        res.isIE ||
        res.isEdge ||
        res.isEdgeChromium ||
        res.isBlink ||
        res.isSafari ||
        'Unknown');
}

;// CONCATENATED MODULE: ./lib/utils/averageArray.js
function averageArray(v) {
    return v.reduce((a, b) => a + b) / v.length;
}

;// CONCATENATED MODULE: ./lib/utils/smoothArray.js

function smoothOut(vector, variance) {
    const t_avg = averageArray(vector) * variance;
    const ret = Array(vector.length);
    for (let i = 0; i < vector.length; i++) {
        (function () {
            const prev = i > 0 ? ret[i - 1] : vector[i];
            const next = i < vector.length ? vector[i] : vector[i - 1];
            ret[i] = averageArray([
                t_avg,
                averageArray([prev, vector[i], next]),
            ]);
        })();
    }
    return averageArray(ret);
}

;// CONCATENATED MODULE: ./lib/core/misc/pluginManager.js

/**
 * @class PluginManager
 * @classdesc Creates a DuckEngine PluginManager
 * @description The PluginManager Class. Manages and handles plugins
 * @since 2.0.0
 */
class PluginManager {
    /**
     * @constructor PluginManager
     * @description Creates a PluginManager instance
     * @since 2.0.0
     */
    constructor() {
        this.plugins = {};
    }
    /**
     * @memberof PluginManager
     * @description Registers a plugin and adds a plugin by its name to the plugins object
     * @param {Duck.Types.Game.Plugin} plugin Plugin to add
     * @since 2.0.0
     */
    registerPlugin(plugin) {
        const foundPlugin = this.find(plugin.name);
        if (!foundPlugin) {
            this.plugins[plugin.name] = plugin;
        }
        else {
            new debug.Error(`Cannot register plugin with name "${plugin.name}". Plugin with that name already exists.`);
        }
    }
    /**
     * @memberof PluginManager
     * @description Unregisters a plugin by its name
     * @param {string} name Name of plugin to remove
     * @since 2.0.0
     */
    unregisterPlugin(name) {
        const plugin = this.find(name);
        if (plugin) {
            delete this.plugins[name];
        }
    }
    /**
     * @memberof PluginManager
     * @description Calls/Runs a plugin by its name
     * @param {string} name Name of plugin to call/run
     * @since 2.0.0
     */
    call(name) {
        const plugin = this.find(name);
        if (plugin) {
            plugin.func(...plugin.args);
        }
    }
    /**
     * @memberof PluginManager
     * @description Finds a plugin by its name
     * @param {string} name Name of plugin to find
     * @returns {Duck.Types.Game.Plugin}
     * @since 2.0.0
     */
    find(name) {
        return this.plugins[name];
    }
}

;// CONCATENATED MODULE: ./lib/core/storage/cacheManager.js
/* eslint-disable @typescript-eslint/no-explicit-any */

/**
 * @class CacheManager
 * @classdesc Creates a CacheManager
 * @description The CacheManager Class. Manages Cached items in memory and in storage
 * @since 2.0.0
 */
class CacheManager {
    /**
     * @constructor CacheManager
     * @description Creates a CacheManager instance
     * @since 2.0.0
     */
    constructor() {
        this.cache = {};
        this.storageIdentifier = 'DuckEngine_CacheManager_';
        this.sync();
    }
    sync() {
        this.cache = Object.assign({}, localStorage);
    }
    /**
     * @memberof CacheManager
     * @description Sets a Key-Value pair in the memory-cache and storage
     * @param {string} name Name/Key to save the value to
     * @param {string} value Value to save to the Name/Key
     * @since 2.0.0
     */
    set(name, value) {
        this.cache[name] = value;
        localStorage.setItem(`${this.storageIdentifier}${name}`, value);
    }
    /**
     * @memberof CacheManager
     * @description Gets a value from a Name/Key from the storage or memory-cache
     * @param {string} name Name/Key to get the value from
     * @returns {string}
     * @since 2.0.0
     */
    get(name) {
        return (localStorage.getItem(`${this.storageIdentifier}${name}`) ||
            this.cache[name]);
    }
    /**
     * @memberof CacheManager
     * @description Deletes a Key-Value pair from the memory-cache and storage
     * @param {string} name Name/Key to use to delete the pair
     * @since 2.0.0
     */
    delete(name) {
        if (this.has(name)) {
            delete this.cache[name];
            localStorage.removeItem(`${this.storageIdentifier}${name}`);
        }
        else {
            new debug.Error(`CacheManager: Cannot delete from values with name "${name}". "${name}" does not exist as an entry.`);
        }
    }
    /**
     * @memberof CacheManager
     * @description Gets a Key-Value pair from the storage or memory-cache and returns based on if it exists or not
     * @param {string} name Name/Key to save the value to
     * @since 2.0.0
     */
    has(name) {
        const e = localStorage.getItem(`${this.storageIdentifier}${name}`) ||
            this.cache[name];
        return e ? true : false;
    }
    /**
     * @memberof CacheManager
     * @description Returns the keys from the memory-cache
     * @returns {string[]}
     * @since 2.0.0
     */
    keys() {
        return Object.keys(this.cache);
    }
    /**
     * @memberof CacheManager
     * @description Returns the values from the memory-cache
     * @returns {string[]}
     * @since 2.0.0
     */
    values() {
        return Object.values(this.cache);
    }
    /**
     * @memberof CacheManager
     * @description Loops through each entry in the memory-cache
     * @param { (key:string,value:string) => any } cb Callback function
     * @since 2.0.0
     */
    each(cb) {
        for (const key in this.cache) {
            if (Object.prototype.hasOwnProperty.call(this.cache, key)) {
                cb(key, this.cache[key]);
            }
        }
    }
    /**
     * @memberof CacheManager
     * @description Returns the memory-cache entries as an object, calls CacheManager.sync
     * @returns {{[key: string]: string}}
     * @since 2.0.0
     */
    get entries() {
        this.sync();
        return this.cache;
    }
}

;// CONCATENATED MODULE: ./lib/core/texture/textureSheet.js

class TextureSheet extends TextureBase {
    constructor(texture, frameWidth, frameHeight, rows, cols) {
        super('image', 'sheet', texture, frameWidth, frameHeight);
        this.frameWidth = frameWidth;
        this.frameHeight = frameHeight;
        this.rows = rows;
        this.cols = cols;
    }
}

;// CONCATENATED MODULE: ./lib/core/renderer/baseRenderer.js
class BaseRenderer {
    constructor(game) {
        this.game = game;
    }
}

;// CONCATENATED MODULE: ./lib/core/renderer/canvas/pipeline/rendererPipeline.js

/**
 * @class RendererPipeline
 * @classdesc Creates a DuckEngine RendererPipeline
 * @description The RendererPipeline Class. Manages all visible scenes and renderables by pooling them on an interval for GC efficiency
 * @since 2.1.0
 */
class RendererPipeline {
    /**
     * @constructor RendererPipeline
     * @description Creates a RendererPipeline instance
     * @param {Game} game Game instance
     * @param {number} [poolingInterval= 1000 / game.fps] How often RendererPipeline.pool is called
     * @since 2.1.0
     */
    constructor(game, poolingInterval = 1000 / game.fps) {
        this.game = game;
        this.poolInterval = setInterval(() => {
            this.pool();
        }, poolingInterval);
        this.poolStack = [];
        this.updateTimeInterval = setInterval(() => {
            this.updateTime();
        }, 1000);
    }
    /**
     * @memberof RendererPipeline
     * @description Pools all visible scenes from the game stack and its renderables
     * @since 2.1.0
     */
    pool() {
        this.poolStack = [];
        // get visible scenes
        const visibleScenes = this.game.stack.scenes.filter((scene) => scene.visible === true);
        visibleScenes.forEach((scene) => {
            // displayList
            const depthSorted = scene.displayList.depthSort();
            const visibleObjects = depthSorted.filter((r) => r.visible && r.culled);
            this.poolStack.push({
                scene,
                renderables: visibleObjects,
            });
        });
        this.game.eventEmitter.emit(events.RENDERER.PIPELINE_POOL, this.poolStack);
    }
    /**
     * @memberof RendererPipeline
     * @description Clears the poolInterval and sets the interval again to a more recent time, time: 1000 / game.fps,
     * gets called automatically every 1000 ms
     * @since 2.1.0
     */
    updateTime() {
        const newInterval = 1000 / this.game.fps;
        clearInterval(this.poolInterval);
        this.poolInterval = setInterval(() => {
            this.pool();
        }, newInterval);
    }
}

;// CONCATENATED MODULE: ./lib/core/renderer/canvas/canvasRenderer.js
/* eslint-disable @typescript-eslint/no-non-null-assertion */




/**
 * @class CanvasRenderer
 * @classdesc Creates a DuckEngine CanvasRenderer
 * @description The CanvasRenderer Class. Renders everything using the RendererPipeline to pool what to draw
 * @since 2.1.0
 */
class CanvasRenderer extends BaseRenderer {
    /**
     * @constructor CanvasRenderer
     * @description Creates a CanvasRenderer instance
     * @param {Game} game Game instance
     * @param {number} [poolingInterval= 1000 / game.fps] How often the does the RendererPipeline pool visible scenes and renderables
     * @since 2.1.0
     */
    constructor(game, poolingInterval = 1000 / game.fps) {
        super(game);
        this.ctx = this.game.canvas.getContext('2d');
        this.pipeline = new RendererPipeline(this.game, poolingInterval);
    }
    /**
     * @memberof CanvasRenderer
     * @description Gets the poolStack from the pipeline and ticks, updates, and renders the scene and renderables
     * @param {number} deltaTime Time passed since last frame
     * @since 2.1.0
     */
    render(deltaTime) {
        this.pipeline.poolStack.forEach((pool) => {
            if (pool.scene.currentCamera) {
                pool.scene.currentCamera.begin();
            }
            pool.scene.__tick();
            pool.scene.update(deltaTime);
            pool.renderables.forEach((r) => {
                r._draw();
            });
            if (pool.scene.currentCamera) {
                pool.scene.currentCamera.end();
            }
        });
    }
    /**
     * @memberof CanvasRenderer
     * @description Calls this.ctx.save
     * @since 2.1.0
     */
    save() {
        this.ctx.save();
    }
    /**
     * @memberof CanvasRenderer
     * @description Calls this.ctx.restore
     * @since 2.1.0
     */
    restore() {
        this.ctx.restore();
    }
    /**
     * @memberof CanvasRenderer
     * @description Calls this.ctx.translate
     * @param {number} x X position
     * @param {number} y Y position
     * @since 2.1.0
     */
    translate(x, y) {
        this.ctx.translate(x, y);
    }
    /**
     * @memberof CanvasRenderer
     * @description Transforms the canvas using this.ctx.transform
     * @param {number} a
     * @param {number} b
     * @param {number} c
     * @param {number} d
     * @param {number} e
     * @param {number} f
     * @since 2.1.0
     */
    transform(a, b, c, d, e, f) {
        this.ctx.transform(a, b, c, d, e, f);
    }
    /**
     * @memberof CanvasRenderer
     * @description Scales the canvas using this.ctx.scale
     * @param {number} x X scale
     * @param {number} y Y scale
     * @since 2.1.0
     */
    scale(x, y) {
        this.ctx.scale(x, y);
    }
    /**
     * @memberof CanvasRenderer
     * @description Sets the font using this.ctx.font =
     * @param {string} font Font to set to
     * @since 2.1.0
     */
    setFont(font) {
        this.ctx.font = font;
    }
    /**
     * @memberof CanvasRenderer
     * @description Measures the text using the font
     * @param {string} font Font to set to
     * @param {string} text Text measure
     * @returns {TextMetrics}
     * @since 2.1.0
     */
    measureText(font, text) {
        this.setFont(font);
        return this.ctx.measureText(text);
    }
    /**
     * @memberof CanvasRenderer
     * @description Sets the fillStyle using this.ctx.fillStyle =
     * @param {string} color Color to set to
     * @since 2.1.0
     */
    setFillColor(color) {
        this.ctx.fillStyle = color;
    }
    /**
     * @memberof CanvasRenderer
     * @description Sets the strokeStyle using this.ctx.fillStyle =
     * @param {string} color Color to set to
     * @since 2.1.0
     */
    setStrokeColor(color) {
        this.ctx.strokeStyle = color;
    }
    /**
     * @memberof CanvasRenderer
     * @description Sets the lineWidth using this.ctx.lineWidth =
     * @param {number} width Width to set as
     * @since 2.1.0
     */
    setLineWidth(width) {
        this.ctx.lineWidth = width;
    }
    /**
     * @memberof CanvasRenderer
     * @description Draws text to the screen, must set font with this.setFont first and color with this.setFillColor
     * @param {string} text Text to draw
     * @param {number} x X Position
     * @param {number} y Y Position
     * @param {number} [maxWidth] Max width of text, optional
     * @since 2.1.0
     */
    drawText(text, x, y, maxWidth) {
        this.ctx.fillText(text, x, y, maxWidth);
    }
    /**
     * @memberof CanvasRenderer
     * @description Strokes text to the screen, must set font with this.setFont first and color with this.setStrokeColor
     * @param {string} text Text to draw
     * @param {number} x X Position
     * @param {number} y Y Position
     * @param {number} [maxWidth] Max width of text, optional
     * @since 2.1.0
     */
    strokeText(text, x, y, maxWidth) {
        this.ctx.strokeText(text, x, y, maxWidth);
    }
    /**
     * @memberof CanvasRenderer
     * @description Clears the screen
     * @since 2.1.0
     */
    clearFrame() {
        this.ctx.clearRect(0, 0, this.game.canvas.width, this.game.canvas.height);
        this.game.eventEmitter.emit(events.RENDERER.CLEAR_FRAME);
    }
    /**
     * @memberof CanvasRenderer
     * @description Clears an area
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of area to clear
     * @param {number} h Height of area to clear
     * @since 2.1.0
     */
    clearRect(x, y, w, h) {
        this.ctx.clearRect(x, y, w, h);
    }
    /**
     * @memberof CanvasRenderer
     * @description Fills a rectangle
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of rect
     * @param {number} h Height of rect
     * @param {string} color Color to fill rect with
     * @since 2.1.0
     */
    drawRect(x, y, w, h, color) {
        this.setFillColor(color);
        this.ctx.fillRect(x, y, w, h);
    }
    /**
     * @memberof CanvasRenderer
     * @description Fills a circle
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} r Radius of circle
     * @param {string} color Color to fill circle with
     * @since 2.1.0
     */
    drawCircle(x, y, r, color) {
        this.setFillColor(color);
        this.ctx.beginPath();
        this.ctx.arc(x, y, r, 0, 2 * Math.PI, false);
        this.ctx.fillStyle = color;
        this.ctx.fill();
    }
    /**
     * @memberof CanvasRenderer
     * @description Fills a roundRect
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of roundRect
     * @param {number} h Height of roundRect
     * @param {number} r Radius of roundRect
     * @param {string} color Color to fill roundRect with
     * @since 2.1.0
     */
    drawRoundRect(x, y, w, h, r, color) {
        if (w < 2 * r)
            r = w / 2;
        if (h < 2 * r)
            r = h / 2;
        this.setFillColor(color);
        this.ctx.beginPath();
        this.ctx.moveTo(x + r, y);
        this.ctx.arcTo(x + w, y, x + w, y + h, r);
        this.ctx.arcTo(x + w, y + h, x, y + h, r);
        this.ctx.arcTo(x, y + h, x, y, r);
        this.ctx.arcTo(x, y, x + w, y, r);
        this.ctx.closePath();
        this.ctx.fill();
    }
    /**
     * @memberof CanvasRenderer
     * @description Draws a sprite
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width of roundRect
     * @param {number} h Height of roundRect
     * @param {TextureBase<'image'>} texture Texture to use for the Sprite
     * @param {number} [currentRow] The default row to use for the texture, for spritesheets
     * @param {number} [currentCol] The default column to use for the texture, for spritesheets
     * @since 2.1.0
     */
    drawSprite(x, y, w, h, texture, currentRow, currentCol) {
        if (texture instanceof TextureSheet) {
            // spritesheet
            this.ctx.drawImage(texture.texture, // image
            (currentCol - 1) * texture.frameWidth, // source x
            (currentRow - 1) * texture.frameHeight, // source y
            texture.frameWidth, // source width
            texture.frameHeight, // source height
            x, // target x
            y, // target y
            texture.frameWidth, // target width
            texture.frameHeight // target height
            );
        }
        else {
            // normal sprite
            this.ctx.drawImage(texture.texture, x, y, w, h);
        }
    }
    /**
     * @memberof CanvasRenderer
     * @description Sets the blend mode / globalCompositionOperation
     * @param {keyof typeof BlendModes} blendMode Blend Mode
     * @since 2.1.0
     */
    setBlendMode(blendMode) {
        this.ctx.globalCompositeOperation = blendMode;
    }
}

;// CONCATENATED MODULE: ./lib/core/display/displayManager.js




class DisplayManager {
    constructor(game) {
        this.game = game;
        this.eventEmitter = new EventEmitter();
        /* Event Listeners */
        // blur and focus listener
        window.onfocus = () => {
            if (this.game.config.pauseRenderingOnBlur &&
                !this.game.config.debugRendering) {
                this.game.isRendering = true;
                this.eventEmitter.emit(events.DISPLAY.FOCUS);
                if (this.game.config.onResumeRendering) {
                    this.game.config.onResumeRendering('windowFocus');
                }
            }
        };
        window.onblur = () => {
            if (this.game.config.pauseRenderingOnBlur &&
                !this.game.config.debugRendering) {
                this.game.isRendering = false;
                this.eventEmitter.emit(events.DISPLAY.BLUR);
                if (this.game.config.onPauseRendering) {
                    this.game.config.onPauseRendering('windowBlur');
                }
            }
        };
        if (this.game.config.focus) {
            window.focus();
            if (this.game.config.onResumeRendering) {
                this.game.config.onResumeRendering('gameConfigFocus');
            }
        }
        if (this.game.config.blur) {
            window.blur();
            if (this.game.config.onPauseRendering) {
                this.game.config.onPauseRendering('gameConfigBlur');
            }
        }
    }
    scale(scale) {
        if (scale.x) {
            this.game.canvas.width = scale.x;
        }
        if (scale.y) {
            this.game.canvas.height = scale.y;
        }
    }
    dprScale() {
        dprScale(this.game.canvas, this.game.renderer.ctx, window.innerWidth, window.innerHeight);
    }
    smallCorrectionScale() {
        if (this.game.canvas.width > window.innerWidth) {
            this.game.canvas.width = window.innerWidth;
        }
        else {
            this.game.canvas.width = this.game.oldWidth;
        }
        if (this.game.canvas.height > window.innerHeight) {
            this.game.canvas.height = window.innerHeight;
        }
        else {
            this.game.canvas.height = this.game.oldHeight;
        }
    }
    scaleAspectRatio(maxScale) {
        const width = this.game.canvas.width;
        const height = this.game.canvas.height;
        let ratio = 0;
        if (width > maxScale.x && width > height) {
            ratio = width / height;
            this.game.canvas.width = maxScale.x;
            this.game.canvas.height = maxScale.x / ratio;
        }
        else if (height > maxScale.y && height > width) {
            ratio = height / width;
            this.game.canvas.width = maxScale.y / ratio;
            this.game.canvas.height = maxScale.y;
        }
        else {
            this.game.canvas.width = maxScale.x;
            this.game.canvas.height = maxScale.y;
        }
    }
    scaleToWindow() {
        if (window.devicePixelRatio === 1) {
            this.game.canvas.width = window.innerWidth;
            this.game.canvas.height = window.innerHeight;
            return;
        }
        if (this.game.config.dprScale && window.devicePixelRatio !== 1) {
            this.dprScale();
            if (this.game.config.debug) {
                new debug.Log(`Scaled with devicePixelRatio of ${window.devicePixelRatio} while fullscreen.`);
            }
        }
        this.eventEmitter.emit(events.DISPLAY.SET_SCALE, {
            w: window.innerWidth,
            h: window.innerHeight,
        });
    }
    resetScale() {
        if (window.devicePixelRatio === 1) {
            this.game.canvas.width = this.game.oldWidth;
            this.game.canvas.height = this.game.oldHeight;
            this.game.canvas.style.width = this.game.oldWidth + 'px';
            this.game.canvas.style.height = this.game.oldHeight + 'px';
        }
        if (this.game.config.dprScale && window.devicePixelRatio !== 1) {
            dprScale(this.game.canvas, this.game.renderer.ctx, this.game.canvas.width, this.game.canvas.height);
        }
    }
    fullscreen() {
        if (this.game.canvas && document.fullscreenEnabled) {
            this.game.canvas
                .requestFullscreen()
                .then(() => {
                this.game.isInFullscreen = true;
                if (this.game.canvas) {
                    this.scaleToWindow();
                }
                this.eventEmitter.emit(events.DISPLAY.FULLSCREEN);
            })
                .catch(() => new debug.Error('User must interact with the page before fullscreen API can be used.'));
            // on un fullscreen
            this.game.canvas.onfullscreenchange = () => {
                if (!document.fullscreenElement) {
                    this.resetScale();
                    this.game.isInFullscreen = false;
                    if (this.game.config.debug) {
                        new debug.Log('Unfullscreen, reset canvas scale.');
                    }
                }
            };
        }
        if (!document.fullscreenEnabled) {
            new debug.Warn('Fullscreen is not supported/enabled on this browser.');
        }
    }
    unfullscreen() {
        if (document.fullscreenElement) {
            document
                .exitFullscreen()
                .then(() => {
                if (this.game.canvas) {
                    this.resetScale();
                }
                this.eventEmitter.emit(events.DISPLAY.UNFULLSCREEN);
            })
                .catch((e) => new debug.Error(e));
        }
    }
}

;// CONCATENATED MODULE: ./lib/core/game.js
var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (undefined && undefined.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};












/**
 * @class Game
 * @classdesc Creates a DuckEngine Game
 * @description The Game Class. Stores many important methods and properties.
 * @since 1.0.0-beta
 */
class Game {
    /**
     * @constructor Game
     * @description Creates a Game instance
     * @param {Duck.Types.Game.Config} config Game Configuration
     * @since 1.0.0-beta
     */
    constructor(config) {
        var _a, _b, _c, _d;
        console.log(startup);
        this.config = config;
        if (this.config.canvas === null) {
            this.config.canvas = Duck.AutoCanvas();
        }
        if (!this.config.canvas) {
            new debug.Error('You must pass in an HTMLCanvasElement or pass in the return value of Duck.AutoCanvas()!');
        }
        if (this.config.canvas instanceof HTMLCanvasElement) {
            this.canvas = this.config.canvas;
        }
        else {
            this.canvas = this.config.canvas.canvas;
        }
        this.renderer = new CanvasRenderer(this, this.config.poolingInterval);
        this.deltaTimeArray = [];
        this.deltaTime = 0;
        this.smoothDeltaTime = 0;
        this.oldTime = 0;
        this.now = 0;
        this.fps = 0;
        this.eventEmitter = new EventEmitter();
        this.pluginManager = new PluginManager();
        this.displayManager = new DisplayManager(this);
        // set scale
        if (this.config.scale) {
            this.displayManager.scale(this.config.scale);
        }
        // set background
        if (this.config.background) {
            this.setBackground(this.config.background);
        }
        // mobile scaling / devicePixelRatio scaling
        if (this.config.dprScale) {
            dprScale(this.canvas, this.renderer.ctx, ((_a = this.config.scale) === null || _a === void 0 ? void 0 : _a.x) || this.canvas.width, ((_b = this.config.scale) === null || _b === void 0 ? void 0 : _b.y) || this.canvas.height);
            if (this.config.debug) {
                new debug.Log(`Scaled with devicePixelRatio of ${window.devicePixelRatio}`);
            }
        }
        // fullscreen scale
        this.isInFullscreen = false;
        this.oldWidth = this.canvas.width;
        this.oldHeight = this.canvas.height;
        // smart scale
        if (this.config.smartScale) {
            window.onresize = () => {
                this.displayManager.smallCorrectionScale();
            };
        }
        this.isRendering = false;
        this.isLoaded = false;
        this.splashScreen =
            ((_c = this.config.splashScreen) === null || _c === void 0 ? void 0 : _c.img) ||
                'https://i.ibb.co/bdN4CCN/Logo-Splash.png';
        if (((_d = this.config.splashScreen) === null || _d === void 0 ? void 0 : _d.img) === 'default') {
            this.splashScreen = 'https://i.ibb.co/bdN4CCN/Logo-Splash.png';
        }
        // stack
        this.stack = {
            scenes: [],
            defaultScene: this.config.defaultScene,
        };
        this.cacheManager = new CacheManager();
        // browser
        this.browser = detectBrowser();
        // animation frame
        this.animationFrame;
        // set up some events
        this.eventEmitter.on(events.GAME.CONTEXT_LOST, () => {
            if (this.config.debug) {
                new debug.Error('Context lost! Trying to restore...');
            }
            // restore context
            if (!this.canvas) {
                const res = Duck.AutoCanvas();
                this.canvas = res.canvas;
            }
            if (!this.renderer.ctx) {
                this.renderer.ctx = this.canvas.getContext('2d');
            }
            this.eventEmitter.emit(events.GAME.CONTEXT_RESTORED, this.renderer.ctx);
        });
        this.eventEmitter.on(events.GAME.CONTEXT_RESTORED, () => {
            if (this.config.debug) {
                new debug.Log('Restored context...');
            }
        });
        // methods
        this.scenes = {
            /**
             * @memberof Game#scenes
             * @description Add a scenes to the Game stack
             * @param {Scene[]} scenes Scenes to add to the Game stack
             * @since 1.0.0-beta
             */
            add: (scenes) => {
                scenes.forEach((scene) => {
                    this.stack.scenes.push(scene);
                    this.eventEmitter.emit(events.GAME.SCENE_ADD, scene);
                    this.renderer.pipeline.pool();
                });
            },
            /**
             * @memberof Game#scenes
             * @description Removes a scene from the Game stack
             * @param {Scene} scene Scene to remove from the Game stack
             * @since 1.0.0-beta
             */
            remove: (scene) => {
                const f = this.stack.scenes.find((_scene) => _scene.key === scene.key);
                if (f) {
                    this.stack.scenes.splice(this.stack.scenes.indexOf(scene), 1);
                    this.eventEmitter.emit(events.GAME.SCENE_REMOVE, scene);
                    this.renderer.pipeline.pool();
                }
            },
        };
    }
    /**
     * @memberof Game
     * @description Starts the game loop
     * @emits EVENTS.GAME.LOAD_BEGIN
     * @emits EVENTS.GAME.DRAW_SPLASH
     * @emits EVENTS.GAME.LOAD_SCENE
     * @emits EVENTS.GAME.LOAD_FINISH
     * @since 1.0.0-beta
     */
    start() {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.eventEmitter.emit(events.GAME.LOAD_BEGIN);
            // sync cache
            this.eventEmitter.emit(events.GAME.SYNC_CACHE);
            this.cacheManager.sync();
            // show loading splash screen
            this.eventEmitter.emit(events.GAME.DRAW_SPLASH);
            yield this.drawSplashScreen();
            try {
                // load scenes
                for (var _b = __asyncValues(this.stack.scenes), _c; _c = yield _b.next(), !_c.done;) {
                    const scene = _c.value;
                    // preload assets
                    yield scene.preload();
                    // create assets
                    scene.create();
                    this.eventEmitter.emit(events.GAME.LOAD_SCENE);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            // set states
            this.isRendering = true;
            this.isLoaded = true;
            yield this.hideSplashScreen();
            this.eventEmitter.emit(events.GAME.LOAD_FINISH);
            if (this.config.debug) {
                new debug.Log('Game loaded.');
            }
            if (this.config.onResumeRendering && !this.config.debugRendering) {
                this.config.onResumeRendering('gameStart');
            }
            // pool
            this.renderer.pipeline.pool();
            // start loop
            this.loop();
            if (this.config.debug) {
                new debug.Log('Started animation frame.');
            }
        });
    }
    /**
     * @memberof Game
     * @description Stops the game loop
     * @emits EVENTS.GAME.STOP
     * @since 1.0.0
     */
    stop() {
        if (this.animationFrame) {
            cancelAnimationFrame(this.animationFrame);
            // set states
            this.isRendering = false;
            this.isLoaded = false;
            if (this.config.onPauseRendering && !this.config.debugRendering) {
                this.config.onPauseRendering('gameStop');
            }
            if (this.config.debug) {
                new debug.Log('Stopped animation frame.');
            }
            this.eventEmitter.emit(events.GAME.STOP);
        }
        else {
            if (this.config.debug) {
                new debug.Error('Cannot stop animation frame. You must start the game loop first.');
            }
        }
    }
    /**
     * @memberof Game
     * @description Core loop
     * @since 1.0.0-beta
     */
    loop() {
        this.renderer.clearFrame();
        /* Delta Time Stuff */
        this.now = performance.now();
        this.deltaTime = (this.now - this.oldTime) / 1000;
        this.fps = 1 / this.deltaTime;
        if (this.deltaTimeArray.length > 100) {
            this.deltaTimeArray.shift();
        }
        this.deltaTimeArray.push(this.deltaTime);
        this.smoothDeltaTime = Number(smoothOut(this.deltaTimeArray, 1).toPrecision(1));
        /* Call Renderer.render */
        if (this.isRendering && this.deltaTimeArray.length > 99) {
            this.renderer.render(this.deltaTime);
        }
        this.oldTime = this.now;
        this.animationFrame = requestAnimationFrame(() => {
            this.loop();
        });
    }
    /**
     * @memberof Game
     * @description Draws the splash screen to the canvas by setting the background image
     * @since 2.0.0
     */
    drawSplashScreen() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            this.canvas.style.backgroundImage = `url('${this.splashScreen}')`;
            yield this.sleep((((_a = this.config.splashScreen) === null || _a === void 0 ? void 0 : _a.extraDuration) || 0) +
                (((_b = this.config) === null || _b === void 0 ? void 0 : _b.poolingInterval) || 1000) // add pooling interval so that scenes will be pooled on load
            );
        });
    }
    /**
     * @memberof Game
     * @description Hides the splash screen to the canvas
     * @since 2.0.0
     */
    hideSplashScreen() {
        return __awaiter(this, void 0, void 0, function* () {
            this.canvas.style.backgroundImage = 'none';
        });
    }
    sleep(ms) {
        return new Promise((resolve) => setTimeout(resolve, ms));
    }
    /**
     * @memberof Game
     * @description Sets the scale of the canvas
     * @param {Duck.Types.Misc.Scale} scale Scale to set the canvas to
     * @emits EVENTS.GAME.SET_SCALE
     * @since 1.0.0-beta
     */
    setScale(scale) {
        if (this.canvas) {
            if (scale.width) {
                this.canvas.width = scale.width;
            }
            if (scale.height) {
                this.canvas.height = scale.height;
            }
            this.eventEmitter.emit(events.GAME.SET_SCALE, scale);
        }
        else {
            new debug.Error('Cannot setScale to a canvas of undefined.');
        }
    }
    /**
     * @memberof Game
     * @description Sets the style background color of the canvas
     * @param {string} background Background color
     * @emits EVENTS.GAME.SET_BACKGROUND
     * @since 1.0.0-beta
     */
    setBackground(background) {
        if (this.canvas) {
            this.canvas.style.background = background;
            this.eventEmitter.emit(events.GAME.SET_BACKGROUND, background);
        }
        else {
            new debug.Error('Cannot set background of undefined. Canvas is undefined.');
        }
    }
    /**
     * @memberof Game
     * @description Switches the current scene by the key
     * @param {string} key Key of the scene to switch from
     * @param {string} key2 Key of the scene to switch to
     * @emits EVENTS.GAME.SWITCH_SCENE
     * @since 1.0.0-beta
     */
    switchScene(key, key2) {
        const f = this.stack.scenes.find((_scene) => _scene.key === key);
        const f2 = this.stack.scenes.find((_scene) => _scene.key === key2);
        if (f) {
            if (f2) {
                f.visible = false;
                f2.visible = true;
                // will be removed in v3.0.0
                f2.onChange();
                f.onSceneInactive();
                f2.onSceneActive();
                this.eventEmitter.emit(events.GAME.SWITCH_SCENE);
            }
            else {
                new debug.Error(`Cannot switch to scene with scene key "${key2}. Scene not found."`);
            }
        }
        else {
            new debug.Error(`Cannot switch from scene from scene key "${key}. Scene not found."`);
        }
    }
    /**
     * @memberof Game
     * @description Sets a scene to visible. Keeps the current scene visible
     * @param {string} key Key of the scene to show
     * @emits EVENTS.GAME.SHOW_SCENE
     * @since 1.0.0-beta
     */
    showScene(key) {
        const f = this.stack.scenes.find((_scene) => _scene.key === key);
        if (f) {
            f.visible = true;
            f.onSceneActive();
            this.eventEmitter.emit(events.GAME.SHOW_SCENE);
        }
        else {
            new debug.Error(`Cannot switch to scene with key "${key}. Scene not found."`);
        }
    }
    /**
     * @memberof Game
     * @description Locks the pointer on the canvas
     * @emits EVENTS.GAME.LOCK_POINTER
     * @since 1.0.0
     */
    lockPointer() {
        if (this.canvas) {
            this.canvas.requestPointerLock();
            this.eventEmitter.emit(events.GAME.LOCK_POINTER);
        }
    }
    /**
     * @memberof Game
     * @description Unlocks the pointer from the canvas
     * @emits EVENTS.GAME.UNLOCK_POINTER
     * @since 1.0.0
     */
    unlockPointer() {
        if (document.pointerLockElement) {
            document.exitPointerLock();
            this.eventEmitter.emit(events.GAME.UNLOCK_POINTER);
        }
    }
}

;// CONCATENATED MODULE: ./lib/base/render.js
/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable @typescript-eslint/no-empty-function */
var render_awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @class Render
 * @classdesc Creates a DuckEngine Render base class
 * @description The Render Class. Base class that DuckEngine Scene extends
 * @since 1.0.0-beta
 */
class Render {
    /**
     * @memberof Render
     * @description Preloads assets, gets called before DuckEngine loads
     * @since 2.0.0
     */
    preload() {
        return render_awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * @memberof Render
     * @description Creates objects, put all your Scene.add calls in here, gets after preload
     * @since 2.0.0
     */
    create() { }
    /**
     * @memberof Render
     * @description Updates everything in method, gets called **second** in the Game loop
     * @param {number} deltaTime Time since the last frame
     * @since 1.0.0-beta
     */
    update(deltaTime) { }
    /**
     * @memberof Render
     * @description Gets called on scene switch to this scene
     * @deprecated Set the Scene.onActive property instead
     *
     * **THIS WILL BE REMOVED IN 3.0.0**
     *
     * @since 1.0.0-beta
     */
    onChange() { }
}

;// CONCATENATED MODULE: ./lib/core/math/randomFloat.js
/**
 * @function
 * @description Returns a random float between two ranges
 * @param {number} min Minimum
 * @param {number} max Maximum
 * @param {number} [fixed=2] Number to fix the float to, optional -> defaults: 2
 */
function randomFloat(min, max, fixed) {
    const num = Math.random() * (max - min) + min;
    const res = num.toFixed(fixed || 2);
    return Number(res);
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/particles/particleEmitter.js



/**
 * @class ParticleEmitter
 * @classdesc Creates a DuckEngine ParticleEmitter
 * @description The ParticleEmitter Class. Emits, creates, and destroys particles
 * @since 1.0.0-beta
 */
class ParticleEmitter {
    /**
     * @constructor ParticleEmitter
     * @description Creates ParticleEmitter instance
     * @param {Particle} particle Base particle that is cloned and modified from
     * @param {Duck.Types.ParticleEmitter.Range} rangeX Where the new emitted particles x position is. A range of 2 values
     * @param {Duck.Types.ParticleEmitter.Range} rangeY Where the new emitted particles y position is. A range of 2 values
     * @param {number} amount Amount of starting particles
     * @param {boolean} [autoCreate=true] Determines if particles are created on init, populates the list, optional -> defaults: true
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(particle, rangeX, rangeY, amount, game, scene, autoCreate = true) {
        this.id = particle.id;
        this.particle = particle;
        this.rangeX = rangeX;
        this.rangeY = rangeY;
        this.amount = amount;
        this.list = [];
        this.game = game;
        this.scene = scene;
        this.emitting = false;
        this.enabled = false;
        this.floatRangeX = [0, 0];
        this.floatRangeY = [0, 0];
        // create particles
        if (autoCreate) {
            this.create();
        }
    }
    /**
     * @memberof ParticleEmitter
     * @description Creates an amount of particles
     * @since 2.1.0
     */
    create() {
        for (let i = 0; i < this.amount; i++) {
            this.createOne();
        }
    }
    /**
     * @memberof ParticleEmitter
     * @description Creates one particle
     * @since 2.1.0
     */
    createOne() {
        const obj = new Particle(this.particle.shape, this.particle.w, this.particle.h, this.particle.r, this.particle.originalFillColorOrIMGPath, this.game, this.scene);
        obj.visible = true;
        obj.enabled = true;
        obj.position.x = randomInt(this.rangeX[0], this.rangeX[1]);
        obj.position.y = randomInt(this.rangeY[0], this.rangeY[1]);
        obj.floatVelocity.x = randomFloat(this.floatRangeX[0], this.floatRangeX[1], 1);
        obj.floatVelocity.y = randomFloat(this.floatRangeY[0], this.floatRangeY[1], 1);
        this.list.push(obj);
        // add to display list
        this.scene.displayList.add(obj);
        // no need to pool the renderer pipeline as displayList.add pools the renderer pipeline
        // add to physics list
        this.scene.physicsList.add(obj);
    }
    /**
     * @memberof ParticleEmitter
     * @description Starts emitting particles
     * @since 1.0.0-beta
     */
    emit() {
        this.emitting = true;
        this.enabled = true;
    }
    /**
     * @memberof ParticleEmitter
     * @description Stops emitting particles
     * @since 1.0.0-beta
     */
    stopEmit() {
        this.emitting = false;
        this.enabled = false;
    }
    /**
     * @memberof ParticleEmitter
     * @description Starts emitting particles for a certain duration
     * @param {number} ms Duration in milliseconds
     * @since 1.0.0-beta
     */
    emitFor(ms) {
        this.emitting = true;
        setTimeout(() => {
            this.emitting = false;
        }, ms);
    }
    /**
     * @memberof ParticleEmitter
     * @description Sets the new or old particles' position range
     * @param {Duck.Types.ParticleEmitter.Range} rangeX Where the new emitted particles x position is. A range of 2 values
     * @param {Duck.Types.ParticleEmitter.Range} rangeY Where the new emitted particles y position is. A range of 2 values
     * @since 1.0.0-beta
     */
    setRange(rangeX, rangeY) {
        this.rangeX = rangeX;
        this.rangeY = rangeY;
        this.list.forEach((obj) => {
            obj.position.x = randomInt(this.rangeX[0], this.rangeX[1]);
            obj.position.y = randomInt(this.rangeY[0], this.rangeY[1]);
            obj.floatVelocity.x = randomFloat(this.floatRangeX[0], this.floatRangeX[1], 1);
            obj.floatVelocity.y = randomFloat(this.floatRangeY[0], this.floatRangeY[1], 1);
        });
    }
    /**
     * @memberof ParticleEmitter
     * @description Spawns new particles after the initial particles
     * @param {number} intervalMS How often a new particle gets spawned
     * @param {number} [limitTo] A limit to how many particles can be rendered at once, optional
     * @since 1.0.0-beta
     */
    keepEmitting(intervalMS, limitTo) {
        setInterval(() => {
            if (limitTo) {
                if (this.list.length < limitTo) {
                    this.createOne();
                }
            }
            else {
                this.createOne();
            }
        }, intervalMS);
    }
    /**
     * @memberof ParticleEmitter
     * @description Pops particles from the list if too many particles exist
     * @since 1.0.0
     */
    offloadMaxAmount(limit) {
        if (this.list.length >= limit) {
            this.list.pop();
        }
        // remove from display list
        this.scene.displayList.list.filter((renderableObject) => renderableObject instanceof Particle).pop();
        // remove from physics list
        this.scene.displayList.list.filter((obj) => obj instanceof Particle).pop();
    }
    /**
     * @memberof ParticleEmitter
     * @description Offloads particles based on position
     * @param {number} bounds The Bounds of the particles, a BoundsLike object that causes a particle to be offloaded
     * @since 2.0.0
     */
    offloadBounds(bounds) {
        this.list.forEach((particle, index) => {
            if (particle.position.x < bounds.x) {
                this.list.splice(index, 1);
            }
            if (particle.position.y < bounds.y) {
                this.list.splice(index, 1);
            }
            if (particle.position.x > bounds.w) {
                this.list.splice(index, 1);
            }
            if (particle.position.y > bounds.w) {
                this.list.splice(index, 1);
            }
        });
        // remove from displayList
        this.scene.displayList.each((renderableObject, index) => {
            if (renderableObject instanceof Particle) {
                if (renderableObject.position.x < bounds.x) {
                    this.list.splice(index, 1);
                }
                if (renderableObject.position.y < bounds.y) {
                    this.list.splice(index, 1);
                }
                if (renderableObject.position.x > bounds.w) {
                    this.list.splice(index, 1);
                }
                if (renderableObject.position.y > bounds.w) {
                    this.list.splice(index, 1);
                }
            }
        });
        // remove from physicsList
        this.scene.physicsList.each((obj, index) => {
            if (obj instanceof Particle) {
                if (obj instanceof Particle) {
                    if (obj.position.x < bounds.x) {
                        this.list.splice(index, 1);
                    }
                    if (obj.position.y < bounds.y) {
                        this.list.splice(index, 1);
                    }
                    if (obj.position.x > bounds.w) {
                        this.list.splice(index, 1);
                    }
                    if (obj.position.y > bounds.w) {
                        this.list.splice(index, 1);
                    }
                }
            }
        });
    }
    /**
     * @memberof ParticleEmitter
     * @description Offloads particles based on how many seconds they existed for
     * @param {number} ageInSeconds Max amount of seconds a particle existed for
     * @since 1.0.0
     */
    offloadMaxAge(ageInSeconds) {
        this.list.forEach((particle, index) => {
            if (particle.age >= ageInSeconds) {
                this.list.splice(index, 1);
            }
        });
        // remove from displayList
        this.scene.displayList.each((renderableObject, index) => {
            if (renderableObject instanceof Particle) {
                if (renderableObject.age >= ageInSeconds) {
                    this.scene.displayList.splice(index, 1);
                }
            }
        });
        // remove from physicsList
        this.scene.physicsList.each((obj, index) => {
            if (obj instanceof Particle) {
                if (obj.age >= ageInSeconds) {
                    this.scene.physicsList.splice(index, 1);
                }
            }
        });
    }
    /**
     * @memberof ParticleEmitter
     * @description Sets the velocity of all particles
     * @param {'x'|'y'} axis 'x' or 'y' axis to change the velocity of all particles
     * @param {number} v Value to set the velocity of all particles
     * @since 1.0.0-beta
     */
    setVelocity(axis, v) {
        this.list.forEach((particle) => {
            particle.setVelocity(axis, v);
        });
    }
    /**
     * @memberof ParticleEmitter
     * @description Wobbles the particle on a random direction and axis
     * @param {number} v Value to set the wobble velocity to
     * @since 1.0.0-beta
     */
    wobble(v) {
        const r = randomInt(1, 4);
        if (r === 1) {
            this.list.forEach((particle) => {
                particle.setVelocity('x', v);
            });
        }
        if (r === 2) {
            this.list.forEach((particle) => {
                particle.setVelocity('y', v);
            });
        }
        if (r === 3) {
            this.list.forEach((particle) => {
                particle.setVelocity('x', -v);
            });
        }
        if (r === 4) {
            this.list.forEach((particle) => {
                particle.setVelocity('y', -v);
            });
        }
    }
    /**
     * @memberof ParticleEmitter
     * @description Sets the floatVelocity of all particles in between a range
     * @param {Duck.Types.ParticleEmitter.Range} rangeVX A range of 2 values for the floatVelocity.x, randomly chosen between that range
     * @param {Duck.Types.ParticleEmitter.Range} rangeVY A range of 2 values for the floatVelocity.y, randomly chosen between that range
     * @since 1.0.0-beta
     */
    float(rangeVX, rangeVY) {
        this.floatRangeX = rangeVX;
        this.floatRangeY = rangeVY;
        this.list.forEach((particle) => {
            particle.floatVelocity.x = randomInt(rangeVX[0], rangeVX[1]);
            particle.floatVelocity.y = randomInt(rangeVY[0], rangeVY[1]);
        });
    }
    /**
     * @memberof ParticleEmitter
     * @description Sets the fillColor of all particles
     * @param {string} fillColor Color
     * @since 1.0.0-beta
     */
    setFillColor(fillColor) {
        this.list.forEach((particle) => {
            particle.setFillColor(fillColor);
        });
    }
    /**
     * @memberof ParticleEmitter
     * @description Sets the imagePath of all particles
     * @param {string} imagePath Image path
     * @since 1.0.0
     */
    setImagePath(imagePath) {
        this.list.forEach((particle) => {
            particle.setImagePath(imagePath);
        });
    }
}

;// CONCATENATED MODULE: ./lib/core/sound/soundPlayer.js

/**
 * @class SoundPlayer
 * @classdesc Creates a SoundPlayer class.
 * @description The SoundPlayer Class. Plays audio
 * @since 1.0.0-beta
 */
class SoundPlayer {
    /**
     * @constructor SoundPlayer
     * @description Creates a SoundPlayer instance
     * @param {string} path Path to sound file
     * @param {Duck.Types.Sound.Config} [options] Sound Configuration
     * @since 1.0.0-beta
     */
    constructor(path, options) {
        this.path = path;
        this.element = document.createElement('audio');
        this.element.className = 'duckengine-audio-sound';
        this.element.style.display = 'none';
        this.element.controls = false;
        this.element.src = this.path;
        this.element.volume = (options === null || options === void 0 ? void 0 : options.volume) || 1;
        this.element.autoplay = (options === null || options === void 0 ? void 0 : options.autoplay) || false;
        this.element.loop = (options === null || options === void 0 ? void 0 : options.loop) || false;
        if ((options === null || options === void 0 ? void 0 : options.volume) === 0) {
            this.mute();
        }
        this.sprites = (options === null || options === void 0 ? void 0 : options.sprites) || [];
        document.body.appendChild(this.element);
    }
    /**
     * @memberof Sound
     * @description Plays the audio
     * @since 1.0.0-beta
     */
    play() {
        this.element.play();
    }
    /**
     * @memberof Sound
     * @description Pauses the audio
     * @since 1.0.0-beta
     */
    pause() {
        this.element.pause();
    }
    /**
     * @memberof Sound
     * @description Mutes the audio
     * @since 1.1.0
     */
    mute() {
        this.element.muted = true;
    }
    /**
     * @memberof Sound
     * @description Unmutes the audio
     * @since 1.1.0
     */
    unmute() {
        this.element.muted = false;
    }
    /**
     * @memberof Sound
     * @description Pauses the audio, alias to SoundPlayer.pause
     * @since 2.1.0
     */
    stop() {
        this.pause();
    }
    /**
     * @memberof Sound
     * @description Sets the loop of the audio
     * @since 2.1.0
     */
    loop(loop = true) {
        this.element.loop = loop;
    }
    /**
     * @memberof Sound
     * @description Seeks the audio
     * @param {number} timeInSeconds Time in seconds to seek to
     * @since 1.0.0-beta
     */
    seek(timeInSeconds) {
        this.element.currentTime = timeInSeconds;
    }
    /**
     * @memberof Sound
     * @description Restarts the audio
     * @since 1.0.0-beta
     */
    restart() {
        this.seek(0);
    }
    /**
     * @memberof Sound
     * @description Sets the volume of the audio
     * @since 1.0.0-beta
     */
    setVolume(volume) {
        this.element.volume = volume;
    }
    /**
     * @memberof Sound
     * @description Gets the duration of the sound
     * @since 1.0.0-beta
     */
    get duration() {
        return this.element.duration;
    }
    /**
     * @memberof Sound
     * @description Gets the playing state of the sound
     * @since 1.0.0-beta
     */
    get isPlaying() {
        return !this.element.paused;
    }
    /**
     * @memberof Sound
     * @description Gets the volume of the sound
     * @since 1.0.0-beta
     */
    get volume() {
        return this.element.volume;
    }
    /**
     * @memberof Sound
     * @description Gets the mute state of the sound
     * @since 1.0.0-beta
     */
    get isMuted() {
        return this.element.muted;
    }
    /**
     * @memberof Sound
     * @description Plays a sound sprite based on the key
     * @param {string} key Key of the sound sprite
     * @since 1.0.0-beta
     */
    playSprite(key) {
        const foundSprite = this.sprites.find((_sprite) => _sprite.key === key);
        if (foundSprite) {
            this.seek(foundSprite.startSeconds);
            if (!this.isPlaying) {
                this.play();
            }
            const int = setInterval(() => {
                if (foundSprite) {
                    if (this.element.currentTime >= foundSprite.endSeconds) {
                        clearInterval(int);
                        this.pause();
                        this.restart();
                    }
                }
            }, 1000);
        }
        else {
            new debug.Error(`Cannot find sound sprite with key: "${key}".`);
        }
    }
    /**
     * @memberof Sound
     * @description Sets the audio src/path
     * @param {string} path Audio src/path
     * @since 1.0.0-beta
     */
    setPath(path) {
        this.path = path;
        this.element.src = this.path;
    }
}

;// CONCATENATED MODULE: ./lib/core/input/models/key.js
/**
 * @class Key
 * @classdesc Creates a DuckEngine Key
 * @description The Key Class. Stores info about a key's state
 * @since 2.0.0
 */
class Key {
    /**
     * @constructor Key
     * @description Creates a Key instance
     * @param {number} keyCode The Key keyCode
     * @param {string} descriptor The Key descriptor, used to set the value to KeyboardInput.keys
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param { (e:KeyboardEvent) => any } [keyDown] Key Down callback
     * @param { (e:KeyboardEvent) => any } [keyUp] Key Up callback
     * @param { (e:KeyboardEvent) => any } [keyJustPressed] Key Just Pressed callback
     * @param { (e:KeyboardEvent) => any } [keyState] Key State callback
     * @since 2.0.0
     */
    constructor(keyCode, descriptor, game, scene, keyboardInput, keyDown, keyUp, keyJustPressed, keyState) {
        this.keyCode = keyCode;
        this.descriptor = descriptor.toUpperCase();
        this.game = game;
        this.scene = scene;
        this.isDown = false;
        this.isUp = false;
        this.isJustPressed = false;
        this.state = false;
        this.keyboardInput = keyboardInput;
        if (keyDown) {
            this.keyboardInput.eventEmitter.on(`KEY_DOWN_${this.descriptor}`, keyDown);
        }
        if (keyUp) {
            this.keyboardInput.eventEmitter.on(`KEY_UP_${this.descriptor}`, keyUp);
        }
        if (keyJustPressed) {
            this.keyboardInput.eventEmitter.on(`KEY_JUST_PRESSED_${this.descriptor}`, keyJustPressed);
        }
        if (keyState) {
            this.keyboardInput.eventEmitter.on(`KEY_STATE_${this.descriptor}`, keyState);
        }
        this.registerListeners();
    }
    registerListeners() {
        document.addEventListener('keydown', (e) => {
            if (e.keyCode === this.keyCode) {
                this.state = true;
                this.keyboardInput.eventEmitter.emit(`KEY_STATE_${this.descriptor}`, e, this.state);
                if (!e.repeat) {
                    this.isDown = false;
                    this.isUp = false;
                    this.isJustPressed = true;
                    this.keyboardInput.eventEmitter.emit(`KEY_JUST_PRESSED_${this.descriptor}`, e);
                    this.keyboardInput.eventEmitter.emit(`KEY_INPUT_${this.descriptor}`, this, e);
                }
                else {
                    this.isDown = true;
                    this.isUp = false;
                    this.isJustPressed = false;
                    this.keyboardInput.eventEmitter.emit(`KEY_DOWN_${this.descriptor}`, e);
                    this.keyboardInput.eventEmitter.emit(`KEY_INPUT_${this.descriptor}`, this, e);
                }
            }
        });
        document.addEventListener('keyup', (e) => {
            if (e.keyCode === this.keyCode) {
                this.state = false;
                this.keyboardInput.eventEmitter.emit(`KEY_STATE_${this.descriptor}`, e, this.state);
                this.isDown = false;
                this.isUp = true;
                this.isJustPressed = false;
                this.keyboardInput.eventEmitter.emit(`KEY_UP_${this.descriptor}`, e);
                this.keyboardInput.eventEmitter.emit(`KEY_INPUT_${this.descriptor}`, this, e);
            }
        });
    }
    /**
     * @memberof Key
     * @description Creates on event on event emitter to listen for input
     * @param {(key:Key, e:KeyboardEvent) => any} cb Callback to call on input
     * @since 2.0.0
     */
    onInput(cb) {
        this.keyboardInput.eventEmitter.on(`KEY_INPUT_${this.descriptor}`, cb);
    }
}

;// CONCATENATED MODULE: ./lib/core/input/keyboardInput.js


/**
 * @class KeyboardInput
 * @classdesc Creates a DuckEngine KeyboardInput
 * @description The KeyboardInput Class. Use the Keyboard as input
 * @since 2.0.0
 */
class KeyboardInput {
    /**
     * @memberof KeyboardInput
     * @description Creates a KeyboardInput instance
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(game, scene) {
        this.game = game;
        this.scene = scene;
        this.eventEmitter = new EventEmitter();
        this.keys = {};
    }
    /**
     * @memberof KeyboardInput
     * @description Adds a Key to the KeyboardInput
     * @param {number} keyCode The Key keyCode
     * @param {string} descriptor The Key descriptor, used to set the value to KeyboardInput.keys
     * @param { (e:KeyboardEvent) => any } [keyDown] Key Down callback
     * @param { (e:KeyboardEvent) => any } [keyUp] Key Up callback
     * @param { (e:KeyboardEvent) => any } [keyJustPressed] Key Just Pressed callback
     * @param { (e:KeyboardEvent) => any } [keyState] Key State callback
     * @since 2.0.0
     */
    addKey(keyCode, descriptor, keyDown, keyUp, keyJustPressed, keyState) {
        const myKey = new Key(keyCode, descriptor, this.game, this.scene, this, keyDown, keyUp, keyJustPressed, keyState);
        this.keys[descriptor] = myKey;
        return myKey;
    }
    /**
     * @memberof KeyboardInput
     * @description Adds a Key to the KeyboardInput
     * @param {Duck.Types.KeyboardInput.KeyBase[]} keys An array of base key configurations to create Keys from
     * @since 2.0.0
     */
    addKeys(keys) {
        keys.forEach((keyBase) => {
            this.addKey(keyBase.keyCode, keyBase.descriptor, keyBase.keyDown, keyBase.keyUp, keyBase.keyJustPressed, keyBase.keyState);
        });
    }
    /**
     * @memberof KeyboardInput
     * @description Removes a Key from the KeyboardInput
     * @param {string} descriptor Key descriptor
     * @since 2.0.0
     */
    removeKey(descriptor) {
        const key = this.keys[descriptor];
        if (key) {
            delete this.keys[descriptor];
        }
    }
    /**
     * @memberof KeyboardInput
     * @description Adds an onInput event listener to all keys
     * @param {(key:Key,e:KeyboardEvent)=>any} cb
     * @since 2.0.0
     */
    onInput(cb) {
        for (const key in this.keys) {
            if (Object.prototype.hasOwnProperty.call(this.keys, key)) {
                const myKey = this.keys[key];
                myKey.onInput(cb);
            }
        }
    }
    /**
     * @memberof KeyboardInput
     * @description Returns KeyboardInput.keys
     * @returns {{ [key: string]: Key }}
     * @since 2.0.0
     */
    get inputs() {
        return this.keys;
    }
}

;// CONCATENATED MODULE: ./lib/core/input/models/mouse.js
class Mouse {
    /**
     * @constructor Mouse
     * @description Creates a Mouse instance
     * @param { 0 | 1 | 2 | 3 | 4 } button Mouse button
     * @param {string} descriptor Mouse descriptor
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param {MouseInput} mouseInput MouseInput that the Mouse is attached to
     * @param { (e:MouseEvent) => any } [mouseDown] Mouse down callback
     * @param { (e:MouseEvent) => any } [mouseUp] Mouse up callback
     * @param { (e:MouseEvent) => any } [mouseMove] Mouse move callback
     * @since 2.0.0
     */
    constructor(button, descriptor, game, scene, mouseInput, mouseDown, mouseUp, mouseMove) {
        this.button = button;
        this.descriptor = descriptor.toUpperCase();
        this.game = game;
        this.scene = scene;
        this.mouseInput = mouseInput;
        if (mouseDown) {
            this.mouseInput.eventEmitter.on(`MOUSE_DOWN_${this.descriptor}`, mouseDown);
        }
        if (mouseUp) {
            this.mouseInput.eventEmitter.on(`MOUSE_UP_${this.descriptor}`, mouseUp);
        }
        if (mouseMove) {
            this.mouseInput.eventEmitter.on(`MOUSE_MOVE_${this.descriptor}`, mouseMove);
        }
        this.registerListeners();
    }
    registerListeners() {
        document.addEventListener('mousedown', (e) => {
            if (e.button === this.button) {
                this.mouseInput.eventEmitter.emit(`MOUSE_DOWN_${this.descriptor}`, e);
            }
        });
        document.addEventListener('mouseup', (e) => {
            if (e.button === this.button) {
                this.mouseInput.eventEmitter.emit(`MOUSE_UP_${this.descriptor}`, e);
            }
        });
        document.addEventListener('mousemove', (e) => {
            if (e.button === this.button) {
                this.mouseInput.eventEmitter.emit(`MOUSE_MOVE_${this.descriptor}`, e);
            }
        });
    }
}

;// CONCATENATED MODULE: ./lib/core/input/mouseInput.js


/**
 * @class MouseInput
 * @classdesc Creates a DuckEngine MouseInput
 * @description The MouseInput Class. Use the mouse as input
 * @since 1.0.0-beta
 */
class MouseInput {
    /**
     * @memberof MouseInput
     * @description Creates a MouseInput instance
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(game, scene) {
        this.game = game;
        this.scene = scene;
        this.eventEmitter = new EventEmitter();
        this.mice = {};
    }
    /**
     * @memberof MouseInput
     * @description Adds a Mouse to the MouseInput
     * @param { 0 | 1 | 2 | 3 | 4 } button Mouse Button
     * @param {string} descriptor The Mouse descriptor, used to set the value to MouseInput.mouse
     * @param { (e: MouseEvent) => any } [mouseDown] Mouse down callback
     * @param { (e: MouseEvent) => any } [mouseUp] Mouse up callback
     * @param { (e: MouseEvent) => any } [mouseMove] Mouse move callback
     * @returns {Mouse}
     * @since 2.0.0
     */
    addMouse(button, descriptor, mouseDown, mouseUp, mouseMove) {
        const myMouse = new Mouse(button, descriptor, this.game, this.scene, this, mouseDown, mouseUp, mouseMove);
        this.mice[descriptor] = myMouse;
        return myMouse;
    }
    /**
     * @memberof MouseInput
     * @description Adds Mice to the MouseInput
     * @param {Duck.Types.MouseInput.MouseBase[]} mice An array of Mouse Base configuration to use to add mice to the MouseInput
     * @since 2.0.0
     */
    addMice(mice) {
        mice.forEach((mouseBase) => {
            this.addMouse(mouseBase.button, mouseBase.descriptor, mouseBase.mouseDown, mouseBase.mouseUp, mouseBase.mouseDown);
        });
    }
    /**
     * @memberof MouseInput
     * @description Returns MouseInput.mice
     * @returns {{ [key: string]: Mouse }}
     * @since 2.0.0
     */
    get inputs() {
        return this.mice;
    }
}

;// CONCATENATED MODULE: ./lib/core/input/input.js


/**
 * @class Input
 * @classdesc Creates a DuckEngine Input
 * @description The Input Class. Main inputs are captured here
 * @since 1.0.0-beta
 */
class input_Input {
    /**
     * @constructor Input
     * @description Creates an Input instance.
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(game, scene) {
        this.game = game;
        this.scene = scene;
    }
    /**
     * @memberof Input
     * @description Creates a KeyboardInput instance
     * @returns {KeyboardInput}
     * @since 2.0.0
     */
    createKeyboardInput() {
        return new KeyboardInput(this.game, this.scene);
    }
    /**
     * @memberof Input
     * @description Creates a MouseInput instance
     * @returns {MouseInput}
     * @since 2.0.0
     */
    createMouseInput() {
        return new MouseInput(this.game, this.scene);
    }
}

;// CONCATENATED MODULE: ./lib/core/group/group.js









/**
 * @class Group
 * @classdesc Creates a DuckEngine Group
 * @description The Group Class. Contains and manages a group of items
 * @template t Stack Item type generic
 * @since 1.0.0-beta
 */
class Group {
    /**
     * @constructor
     * @description Creates a Group instance.
     * @param {string} name Name of group
     * @param {Game} game Game instance
     * @param {t[]} [defaultItems=[]] Default items, optional -> defaults: []
     * @since 1.0.0-beta
     */
    constructor(name, game, defaultItems) {
        this.name = name;
        this.stack = defaultItems || [];
        this.game = game;
    }
    /**
     * @memberof Group
     * @description Adds an item to the Group
     * @param {t} item Item to add
     * @emits EVENTS.GROUP.ADD
     * @since 1.0.0-beta
     */
    add(item) {
        this.stack.push(item);
        if (this.game.config.debug) {
            new debug.Log('Added item to group.');
        }
        // listener
        this.game.eventEmitter.emit(events.GROUP.ADD, item);
    }
    /**
     * @memberof Group
     * @description Removes an item from the Group
     * @param {t} item Item to remove
     * @emits EVENTS.GROUP.REMOVE
     * @since 1.0.0-beta
     */
    remove(item) {
        if (this.find(item)) {
            this.stack.splice(this.indexOf(item), 1);
            if (this.game.config.debug) {
                new debug.Log('Removed item from group.');
            }
        }
        else {
            new debug.Error('Cannot remove item from Group. Item does not exist in Group.');
        }
        // listener
        this.game.eventEmitter.emit(events.GROUP.REMOVE, item);
    }
    /**
     * @memberof Group
     * @description Finds an item in the Group
     * @param {t} item Item to find
     * @since 1.0.0-beta
     */
    find(item) {
        const a = this.stack.find((_a) => _a === item);
        return a;
    }
    /**
     * @memberof Group
     * @description Gets the index of an item from the Group list array
     * @param {t} item Item to find the index of
     * @since 1.0.0-beta
     */
    indexOf(item) {
        return this.stack.indexOf(item);
    }
    /**
     * @memberof Group
     * @description Loops through each item in the Group array
     * @param {(item: t, index: number) => void} cb Callback function
     * @since 1.0.0-beta
     */
    each(cb) {
        return this.stack.forEach(cb.bind(this));
    }
    /**
     * @memberof Group
     * @description Pops an item from the Group array
     * @returns {t | undefined}
     * @since 1.0.0-beta
     */
    pop() {
        return this.stack.pop();
    }
    /**
     * @memberof Group
     * @description Shifts an item from the Group array
     * @returns {t | undefined}
     * @since 1.0.0-beta
     */
    shift() {
        return this.stack.shift();
    }
    /**
     * @memberof Group
     * @description Splices an item from the Group array
     * @param {number} index Index to splice at
     * @param {number} [deleteCount] How many items to remove, optional
     * @returns {t[]}
     * @since 1.0.0-beta
     */
    splice(index, deleteCount) {
        return this.stack.splice(index, deleteCount);
    }
    /**
     * @memberof Group
     * @description Filters items from the Group
     * @param {Duck.Types.Group.Filter} filter Filter string
     * @returns {t[]}
     * @since 1.0.0-beta
     */
    filter(filter) {
        switch (filter) {
            case 'cameras':
                return this.stack.filter((item) => item instanceof Camera);
                break;
            case 'gameobject':
                return this.stack.filter((item) => item instanceof GameObject);
                break;
            case 'ui':
                return this.stack.filter((item) => item instanceof UI);
                break;
            case 'lights':
                return this.stack.filter((item) => item instanceof StaticLight);
                break;
            case 'physics':
                return this.stack.filter((item) => item instanceof Collider ||
                    item instanceof Hitbox ||
                    item instanceof PhysicsBody);
                break;
            default:
                new debug.Error('Cannot filter Group. Filter must be "cameras", "gameobject", "ui", "lights", or "physics".');
                return this.stack;
                break;
        }
    }
    /**
     * @memberof Group
     * @description Adds an event listener to Group
     * @param {Duck.Types.Group.ListenerType} type Listener type
     * @param {() => unknown} cb Callback function
     * @since 1.0.0-beta
     */
    on(type, cb) {
        this.game.eventEmitter.on(`GROUP_${type}`, cb);
    }
    /**
     * @memberof Group
     * @description Removes an event listener from Group
     * @param {Duck.Types.Group.ListenerType} type Listener type
     * @since 1.0.0-beta
     */
    off(type) {
        this.game.eventEmitter.off(`GROUP_${type}`);
    }
    /**
     * @memberof Group
     * @description Returns an item or undefined at a specified index
     * @param {number} index The index to return a value from, 0-based like an array
     * @returns {t | undefined}
     * @since 2.1.0
     */
    at(index) {
        return this.stack[index];
    }
    /**
     * @memberof Group
     * @description Returns the array
     * @returns {t[]}
     * @since 1.0.0-beta
     */
    get group() {
        return this.stack;
    }
    /**
     * @memberof Group
     * @description Returns the length of the array
     * @returns {number}
     * @since 1.0.0-beta
     */
    get length() {
        return this.stack.length;
    }
}

;// CONCATENATED MODULE: ./lib/core/cutscene/cutscene.js
var cutscene_awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};

/**
 * @class Cutscene
 * @classdesc Creates a DuckEngine Cutscene
 * @description The Cutscene Class. Create a cutscene
 * @since 1.0.0-beta
 */
class Cutscene {
    /**
     * @constructor Cutscene
     * @description Creates an instance of a Cutscene.
     * @param {Duck.Types.Cutscene.Config} config Configuration
     * @param {Duck.Types.Cutscene.Instructions} instructions Cutscene instructions
     * @param {Game} game
     * @since 1.0.0-beta
     */
    constructor(config, instructions, game) {
        this.config = config;
        this.instructions = instructions;
        this.game = game;
        this.steps = this.instructions.steps;
        this.mainObject = this.config.mainObject;
        this.otherObjects = this.config.otherObjects || [];
        this.camera = this.config.mainCamera;
        this.otherCameras = this.config.otherCameras || [];
        this.index = 0;
        this.running = false;
        this.init();
    }
    init() {
        this.mainObject.position.x = this.instructions.init.mainObjectPos.x;
        this.mainObject.position.y = this.instructions.init.mainObjectPos.y;
        this.otherObjects.forEach((otherObject, index) => {
            otherObject.position.x =
                this.instructions.init.otherObjectPos[index].x;
            otherObject.position.y =
                this.instructions.init.otherObjectPos[index].y;
        });
    }
    /**
     * @memberof Cutscene
     * @description Starts the cutscene
     * @emits EVENTS.CUTSCENE.START
     * @since 1.0.0-beta
     */
    start() {
        this.running = true;
        this.game.eventEmitter.emit(events.CUTSCENE.START);
    }
    /**
     * @memberof Cutscene
     * @description Stops the cutscene
     * @emits EVENTS.CUTSCENE.END
     * @since 1.0.0-beta
     */
    stop() {
        this.running = false;
        this.game.eventEmitter.emit(events.CUTSCENE.END);
    }
    /**
     * @memberof Cutscene
     * @description Updates the cutscene, must be in a loop or interval and come before Cutscene.next()
     * @since 1.0.0-beta
     */
    update() {
        var _a;
        if (this.running) {
            // camera
            if (this.instructions.init.cameraSettings) {
                this.camera.setFOV(this.instructions.init.cameraSettings.FOV);
                this.camera.setZoom(this.instructions.init.cameraSettings.zoom);
                if (this.instructions.init.cameraSettings.pos) {
                    this.camera.moveTo(this.instructions.init.cameraSettings.pos.x, this.instructions.init.cameraSettings.pos.y);
                }
            }
            if (this.instructions.init.otherCameraSettings) {
                this.otherCameras.forEach((camera, index) => {
                    if (this.instructions.init.otherCameraSettings) {
                        camera.setFOV(this.instructions.init.otherCameraSettings[index]
                            .FOV);
                        camera.setZoom(this.instructions.init.otherCameraSettings[index]
                            .zoom);
                        if (this.instructions.init.otherCameraSettings[index]
                            .pos) {
                            camera.moveTo(
                            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                            this.instructions.init.otherCameraSettings[index].pos.x, 
                            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                            this.instructions.init.otherCameraSettings[index].pos.y);
                        }
                    }
                });
            }
            // follow
            if ((_a = this.instructions.init.cameraSettings) === null || _a === void 0 ? void 0 : _a.follow) {
                this.camera.startFollow(this.instructions.init.cameraSettings.follow, this.instructions.init.cameraSettings.followLerpX, this.instructions.init.cameraSettings.followLerpY);
            }
            // steps
            const step = this.steps[this.index];
            if (step.type === 'DRAW' && step.affect) {
                return step.affect._draw();
            }
            if (step.type === 'MOVE' && step.moveTo) {
                if (step.moveTo.x) {
                    step.affect.position.x = step.moveTo.x;
                }
                if (step.moveTo.y) {
                    step.affect.position.y = step.moveTo.y;
                }
            }
            if (step.type === 'FUNC' && step.func) {
                step.func();
            }
            if (step.type === 'CAMERA_FOV' && step.cameraValue) {
                step.affect.setFOV(step.cameraValue);
            }
            if (step.type === 'CAMERA_ZOOM' && step.cameraValue) {
                step.affect.setZoom(step.cameraValue);
            }
            if (step.moveTo && step.affect) {
                if (step.type === 'CAMERA_MOVE' &&
                    step.moveTo.x &&
                    step.moveTo.y) {
                    step.affect.moveTo(step.moveTo.x, step.moveTo.y);
                }
            }
            if (step.type === 'CAMERA_SHAKE' &&
                step.cameraIntervalMS &&
                step.cameraTimeMS &&
                step.cameraValue &&
                step.affect) {
                step.affect.shake(step.cameraIntervalMS, step.cameraTimeMS, step.cameraValue);
            }
            if (step.type === 'CAMERA_START_FOLLOW' &&
                step.cameraFollow &&
                step.affect) {
                // no lerp
                if (!step.cameraFollowLerpX && !step.cameraFollowLerpY) {
                    step.affect.startFollow(step.cameraFollow);
                }
                // only lerpX
                if (step.cameraFollowLerpX && !step.cameraFollowLerpY) {
                    step.affect.startFollow(step.cameraFollow, step.cameraFollowLerpX);
                }
                // only lerpY
                if (!step.cameraFollowLerpX && step.cameraFollowLerpY) {
                    step.affect.startFollow(step.cameraFollow, 1, step.cameraFollowLerpY);
                }
                // lerp both
                if (step.cameraFollowLerpX && step.cameraFollowLerpY) {
                    step.affect.startFollow(step.cameraFollow, step.cameraFollowLerpX, step.cameraFollowLerpY);
                }
            }
            if (step.type === 'CAMERA_STOP_FOLLOW' && step.affect) {
                step.affect.stopFollow();
            }
        }
    }
    /**
     * @memberof Cutscene
     * @description Starts the next step
     * @emits EVENTS.CUTSCENE.NEXT
     * @since 1.0.0-beta
     */
    next() {
        this.index++;
        if (this.index > this.steps.length - 1) {
            this.stop();
        }
        this.game.eventEmitter.emit(events.CUTSCENE.NEXT);
    }
    /**
     * @memberof Cutscene
     * @description Sleeps/Waits for a duration
     * @param {number} ms Duration in millisecond
     * @since 1.0.0-beta
     */
    sleep(ms) {
        return cutscene_awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => setTimeout(resolve, ms));
        });
    }
    /**
     * @memberof Cutscene
     * @description Adds an event listener to Cutscene
     * @param {Duck.Types.Cutscene.OnListenerType} type Listener type
     * @param {(...args: any) => void} cb Callback function
     * @since 1.0.0-beta
     */
    on(type, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    cb) {
        this.game.eventEmitter.on(`CUTSCENE_${type}`, cb);
    }
    /**
     * @memberof Cutscene
     * @description Removes an event listener from Cutscene
     * @param {Duck.Types.Cutscene.OnListenerType} type Listener type
     * @since 1.0.0-beta
     */
    off(type) {
        this.game.eventEmitter.off(`CUTSCENE_${type}`);
    }
}

;// CONCATENATED MODULE: ./lib/utils/getImageData.js
function getImageData(image, w, h, onComplete, onFail) {
    const canvas = document.createElement('canvas');
    const ctx = canvas.getContext('2d');
    let data = '__EMPTY__';
    canvas.width = w;
    canvas.height = h;
    image.onload = () => {
        if (ctx) {
            ctx.drawImage(image, 0, 0);
            data = canvas.toDataURL();
            onComplete(data);
        }
        else {
            data = '__FAILED__';
            onFail();
        }
    };
}

;// CONCATENATED MODULE: ./lib/utils/clipImage.js
function clipImage(image, clipX, clipY, clipW, clipH, onComplete, onFail) {
    const canvas = document.createElement('canvas');
    const ctx = canvas.getContext('2d');
    let data = '__EMPTY__';
    canvas.width = clipW;
    canvas.height = clipH;
    image.onload = () => {
        if (ctx) {
            ctx.drawImage(image, clipX, clipY, clipW, clipH, 0, 0, clipW, clipH);
            data = canvas.toDataURL();
            const img = new Image();
            img.setAttribute('src', data);
            onComplete(img);
        }
        else {
            data = '__FAILED__';
            onFail();
        }
    };
}

;// CONCATENATED MODULE: ./lib/core/texture/textureAtlas.js




class TextureAtlas extends TextureBase {
    constructor(textureKey, jsonKey, w, h, game, scene) {
        super('image', 'atlas', scene.loader.textureStack.find((image) => image.key === textureKey)
            .value.texture, w, h);
        this.game = game;
        this.scene = scene;
        this.json = scene.loader.jsonStack.find((json) => json.key === jsonKey)
            .value;
        this.validateJSON();
    }
    validateJSON() {
        if (this.json) {
            if ('name' in this.json && 'data' in this.json) {
                if (typeof this.json.name !== 'string') {
                    new debug.Error('Invalid TextureAtlas JSON. "name" must be a string');
                    return;
                }
                if (!Array.isArray(this.json.data)) {
                    new debug.Error('Invalid TextureAtlas JSON. "data" must be an array');
                    return;
                }
            }
            else {
                new debug.Error('Invalid TextureAtlas JSON. "name" or "data" may be missing');
                return;
            }
            this.parseJSON();
        }
        else {
            new debug.Error('Cannot find JSON using the passed key. Make sure you preloaded the json before using the key');
            return;
        }
    }
    parseJSON() {
        this.json.data.forEach((frameData) => {
            clipImage(this.texture, frameData.x, frameData.y, frameData.w, frameData.h, (image) => {
                const texture = new Texture('image', image, frameData.w, frameData.h);
                this.scene.loader.textureStack.push({
                    type: 'texture',
                    value: texture,
                    key: frameData.key,
                    dataType: 'base',
                });
            }, () => {
                new debug.Error('Failed to clip and create image frame from texture atlas whilst parsing json. Unknown error occurred, report this bug');
            });
        });
    }
    get(key) {
        var _a;
        return (_a = this.scene.loader.textureStack.find((texture) => texture.key === key)) === null || _a === void 0 ? void 0 : _a.value;
    }
}

;// CONCATENATED MODULE: ./lib/core/loader/loader.js
var loader_awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};





// loads images by URL or file path
// static class
/**
 * @class Loader
 * @classdesc A class that loads images and other assets
 * @description The Loader Class. Preloading and loading is handled here
 * @since 1.0.0-beta
 */
class Loader {
    /**
     * @constructor Loader
     * @description Creates a Loader instance
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0-beta
     */
    constructor(game, scene) {
        this.game = game;
        this.scene = scene;
        this.textureStack = [];
        this.jsonStack = [];
        this.htmlStack = [];
        this.xmlStack = [];
        this.fontStack = [];
        this.audioStack = [];
    }
    tryCache(prefix, key) {
        return loader_awaiter(this, void 0, void 0, function* () {
            return this.game.cacheManager.get(`${prefix}_${key}`);
        });
    }
    saveCache(prefix, key, value) {
        return loader_awaiter(this, void 0, void 0, function* () {
            return this.game.cacheManager.set(`${prefix}_${key}`, value);
        });
    }
    /**
     * @memberof Loader
     * @description Loads an image and creates a texture, caches it if it does not already exist in the cache (clear cache if texture does not update
     * if you edit the image src)
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the texture, used to load the texture in sprites
     * @param {number} w Width of image
     * @param {number} h Height of image
     * @since 2.0.0
     */
    loadTexture(pathOrURL, key, w, h) {
        return loader_awaiter(this, void 0, void 0, function* () {
            const image = new Image();
            image.width = w;
            image.height = h;
            // try cache
            const cacheData = yield this.tryCache('texture', key);
            if (cacheData) {
                image.setAttribute('src', cacheData); // DATA URL object
                if (this.game.config.debug) {
                    new debug.Log('Loaded texture from cache.');
                }
                const texture = new Texture('image', image, w, h);
                this.textureStack.push({
                    type: 'texture',
                    value: texture,
                    key,
                    dataType: 'base',
                });
                return image;
            }
            else {
                const res = yield fetch(pathOrURL);
                const blob = yield res.blob();
                const url = URL.createObjectURL(blob.slice(0, 4000));
                image.src = url;
                const texture = new Texture('image', image, w, h);
                this.textureStack.push({
                    type: 'texture',
                    value: texture,
                    key,
                    dataType: 'base',
                });
                // save image in cache
                getImageData(image, w, h, (data) => {
                    this.saveCache('texture', key, data);
                }, () => {
                    new debug.Error('Failed to save texture in cache, report this bug.');
                });
                return image;
            }
        });
    }
    /**
     * @memberof Loader
     * @description Loads an image and creates a texture sheet, caches it if it does not already exist in the cache (clear cache if texture does not update
     * if you edit the image src)
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the texture, used to load the texture in sprites and spritesheet
     * @param {number} w Width of image
     * @param {number} h Height of image
     * @since 2.1.0
     */
    loadTextureSheet(pathOrURL, key, frameWidth, frameHeight, rows, cols) {
        return loader_awaiter(this, void 0, void 0, function* () {
            const image = new Image();
            image.width = frameWidth * cols;
            image.height = frameHeight * rows;
            // try cache
            const cacheData = yield this.tryCache('texture_sheet', key);
            if (cacheData) {
                image.setAttribute('src', cacheData); // DATA URL object
                if (this.game.config.debug) {
                    new debug.Log('Loaded texture sheet from cache.');
                }
                const texture = new TextureSheet(image, frameWidth, frameHeight, rows, cols);
                this.textureStack.push({
                    type: 'texture',
                    value: texture,
                    key,
                    dataType: 'sheet',
                });
                return image;
            }
            else {
                const res = yield fetch(pathOrURL);
                const blob = yield res.blob();
                const url = URL.createObjectURL(blob.slice(0, 4000));
                image.src = url;
                const texture = new TextureSheet(image, frameWidth, frameHeight, rows, cols);
                this.textureStack.push({
                    type: 'texture',
                    value: texture,
                    key,
                    dataType: 'sheet',
                });
                // save image in cache
                getImageData(image, frameWidth * cols, frameHeight * rows, (data) => {
                    this.saveCache('texture_sheet', key, data);
                }, () => {
                    new debug.Error('Failed to save texture sheet in cache, report this bug.');
                });
                return image;
            }
        });
    }
    loadTextureAtlas(atlasKey, texturePathOrURL, jsonPathOrURL, textureKey, jsonKey, imageW, imageH) {
        return loader_awaiter(this, void 0, void 0, function* () {
            // load texture and json
            // both try for cache
            yield this.loadTexture(texturePathOrURL, textureKey, imageW, imageH);
            yield this.loadJSON(jsonPathOrURL, jsonKey);
            // create TextureAtlas
            const textureAtlas = new TextureAtlas(textureKey, jsonKey, imageW, imageH, this.game, this.scene);
            this.textureStack.push({
                type: 'texture',
                value: textureAtlas,
                key: atlasKey,
                dataType: 'atlas',
            });
        });
    }
    /**
     * @memberof Loader
     * @description Loads a JSON file and adds it to the jsonStack, caches it if it does not already exist
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the file to use to save it as
     * @since 2.0.0
     */
    loadJSON(pathOrURL, key) {
        return loader_awaiter(this, void 0, void 0, function* () {
            // try cache
            const cacheData = yield this.tryCache('json', key);
            if (cacheData) {
                const json = JSON.parse(cacheData);
                if (this.game.config.debug) {
                    new debug.Log('Loaded JSON from cache');
                }
                this.jsonStack.push({
                    type: 'json',
                    value: json,
                    key,
                });
                return json;
            }
            else {
                const res = yield fetch(pathOrURL);
                const json = yield res.json();
                this.jsonStack.push({
                    type: 'json',
                    value: json,
                    key,
                });
                // save in cache
                this.saveCache('json', key, JSON.stringify(json));
                return json;
            }
        });
    }
    /**
     * @memberof Loader
     * @description Loads a HTML file and adds it to the htmlStack
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the file to use to save it as
     * @since 2.0.0
     */
    loadHTML(pathOrURL, key) {
        return loader_awaiter(this, void 0, void 0, function* () {
            // try cache
            const cacheData = yield this.tryCache('html', key);
            if (cacheData) {
                const text = cacheData;
                if (this.game.config.debug) {
                    new debug.Log('Loaded HTML from cache');
                }
                const html = new window.DOMParser().parseFromString(text, 'text/html');
                this.htmlStack.push({
                    type: 'html',
                    value: html,
                    key,
                });
                return html;
            }
            else {
                const res = yield fetch(pathOrURL);
                const text = yield res.text();
                const html = new window.DOMParser().parseFromString(text, 'text/html');
                this.htmlStack.push({
                    type: 'html',
                    value: html,
                    key,
                });
                // save in cache
                this.saveCache('html', key, text);
                return html;
            }
        });
    }
    /**
     * @memberof Loader
     * @description Loads a XML file and adds it to the xmlStack
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the file to use to save it as
     * @since 2.0.0
     */
    loadXML(pathOrURL, key) {
        return loader_awaiter(this, void 0, void 0, function* () {
            // try cache
            const cacheData = yield this.tryCache('xml', key);
            if (cacheData) {
                const text = cacheData;
                if (this.game.config.debug) {
                    new debug.Log('Loaded XML from cache');
                }
                const xml = new window.DOMParser().parseFromString(text, 'text/xml');
                this.xmlStack.push({
                    type: 'xml',
                    value: xml,
                    key,
                });
                return xml;
            }
            else {
                const res = yield fetch(pathOrURL);
                const text = yield res.text();
                const xml = new window.DOMParser().parseFromString(text, 'text/xml');
                this.xmlStack.push({
                    type: 'xml',
                    value: xml,
                    key,
                });
                // save in cache
                this.saveCache('xml', key, text);
                return xml;
            }
        });
    }
    /**
     * @memberof Loader
     * @description Loads a font and adds it to the fontStack
     * @param {string} fontFamily Font Family
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the file to use to save it as
     * @param {FontFaceDescriptors} [descriptors] Font Face Descriptors
     * @since 2.0.0
     */
    loadFont(fontFamily, pathOrURL, key, descriptors) {
        return loader_awaiter(this, void 0, void 0, function* () {
            const font = new FontFace(fontFamily, `url(${pathOrURL})`, descriptors);
            const res = yield font.load();
            document.fonts.add(font);
            this.fontStack.push({
                type: 'font',
                value: res,
                key,
            });
            return res;
        });
    }
    /**
     * @memberof Loader
     * @description Loads an Audio file and adds it to the audioStack
     * @param {string} pathOrURL Path to the file or the URL
     * @param {string} key Key of the file to use to save it as
     * @param {string} [mimeType] The mime type of the file, optional -> default: 'audio/mp3'
     * @since 2.0.0
     */
    loadAudio(pathOrURL, key, mimeType = 'audio/mp3') {
        var _a;
        return loader_awaiter(this, void 0, void 0, function* () {
            const res = yield fetch(pathOrURL);
            const reader = (_a = res.body) === null || _a === void 0 ? void 0 : _a.getReader();
            const result = yield (reader === null || reader === void 0 ? void 0 : reader.read());
            const blob = new Blob([result === null || result === void 0 ? void 0 : result.value], {
                type: mimeType,
            });
            const url = window.URL.createObjectURL(blob);
            const audio = new Audio();
            audio.src = url;
            this.audioStack.push({
                type: 'audio',
                value: audio,
                key,
            });
            return audio;
        });
    }
}

;// CONCATENATED MODULE: ./lib/helper/color/rgbToHSL.js
function rgbToHsl(r, g, b) {
    (r /= 255), (g /= 255), (b /= 255);
    const max = Math.max(r, g, b), min = Math.min(r, g, b);
    let h, s, l = (max + min) / 2;
    if (max === min) {
        h = s = 0; // achromatic
    }
    else {
        const d = max - min;
        s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
        switch (max) {
            case r:
                h = (g - b) / d + (g < b ? 6 : 0);
                break;
            case g:
                h = (b - r) / d + 2;
                break;
            case b:
                h = (r - g) / d + 4;
                break;
        }
        //@ts-ignore
        h /= 6;
    }
    return `hsl(${h},${s},${l})`;
}

;// CONCATENATED MODULE: ./lib/helper/color/rgbaToHSLA.js
function rgbaToHSLA(r, g, b, a) {
    r = parseInt(r);
    g = parseInt(g);
    b = parseInt(b);
    (r /= 255), (g /= 255), (b /= 255);
    const max = Math.max(r, g, b), min = Math.min(r, g, b);
    let h, s, l = (max + min) / 2;
    if (max === min) {
        h = s = 0; // achromatic
    }
    else {
        const d = max - min;
        s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
        switch (max) {
            case r:
                h = (g - b) / d + (g < b ? 6 : 0);
                break;
            case g:
                h = (b - r) / d + 2;
                break;
            case b:
                h = (r - g) / d + 4;
                break;
        }
        //@ts-ignore
        h /= 6;
    }
    return `hsla(${h},${s},${l},${a})`;
}

;// CONCATENATED MODULE: ./lib/helper/color/rgbaToRGB.js
function rgbaToRGB(rgba) {
    let res = rgba
        .replace('rgba', 'rgb')
        .replace(/[+-]?([.])[0-9]+/, '')
        .split(',');
    res.pop();
    res.join(',');
    res += ')';
    return res;
}

;// CONCATENATED MODULE: ./lib/helper/color/hexToRGB.js
function hexToRGB(hex) {
    var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
    hex = hex.replace(shorthandRegex, function (m, r, g, b) {
        return r + r + g + g + b + b;
    });
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result
        ? `rgb(${parseInt(result[1], 16)},${parseInt(result[2], 16)},${parseInt(result[3], 16)})`
        : null;
}

;// CONCATENATED MODULE: ./lib/helper/color/hexToHSL.js
function hexToHSL(hex) {
    // Convert hex to RGB first
    let r = 0, g = 0, b = 0;
    if (hex.length === 4) {
        r = '0x' + hex[1] + hex[1];
        g = '0x' + hex[2] + hex[2];
        b = '0x' + hex[3] + hex[3];
    }
    else if (hex.length === 7) {
        r = '0x' + hex[1] + hex[2];
        g = '0x' + hex[3] + hex[4];
        b = '0x' + hex[5] + hex[6];
    }
    // Then to HSL
    r /= 255;
    g /= 255;
    b /= 255;
    let cmin = Math.min(r, g, b), cmax = Math.max(r, g, b), delta = cmax - cmin, h = 0, s = 0, l = 0;
    if (delta === 0)
        h = 0;
    else if (cmax === r)
        h = ((g - b) / delta) % 6;
    else if (cmax === g)
        h = (b - r) / delta + 2;
    else
        h = (r - g) / delta + 4;
    h = Math.round(h * 60);
    if (h < 0)
        h += 360;
    l = (cmax + cmin) / 2;
    s = delta === 0 ? 0 : delta / (1 - Math.abs(2 * l - 1));
    s = +(s * 100).toFixed(1);
    l = +(l * 100).toFixed(1);
    return 'hsl(' + h + ',' + s + '%,' + l + '%)';
}

;// CONCATENATED MODULE: ./lib/helper/color/randomColor.js
function randomColor() {
    let a = '#';
    return (a += Math.floor(Math.random() * 16777215).toString(16));
}

;// CONCATENATED MODULE: ./lib/helper/color/randomAlphaColor.js



function randomColorWithAlpha(alpha) {
    return convertColorToRGBA(randomColor(), alpha || randomFloat(0.1, 0.9));
}

;// CONCATENATED MODULE: ./lib/core/map/tilemap.js

/**
 * @class TileMap
 * @classdesc Creates a DuckEngine TileMap
 * @description The TileMap Class. Creates a tilemap of images
 * @extends Map
 * @since 1.0.0
 */
class TileMap extends map_Map {
    /**
     * @constructor TileMap
     * @description Creates a TileMap instance.
     * @param {TileLayer[]} tileLayers An array of TileLayers
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 1.0.0
     */
    constructor(origin, tileLayers, game, scene) {
        super(origin, tileLayers, game, scene);
    }
    /**
     * @description Draws the tilemap.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        this.sortTileLayers().forEach((layer) => {
            for (let row = 0; row < layer.map.length; row++) {
                for (let col = 0; col < layer.map[row].length; col++) {
                    const tile = layer.tileset.getTile(layer.map[row][col]);
                    if (tile) {
                        if (this.game.renderer.ctx &&
                            layer.tileset.texture.texture) {
                            this.game.renderer.ctx.drawImage(layer.tileset.texture.texture, tile.position.x, tile.position.y, layer.tileset.tileW, layer.tileset.tileH, layer.tileset.tileW * col + this.origin.x, layer.tileset.tileH * row + this.origin.y, layer.tileset.tileW, layer.tileset.tileH);
                        }
                    }
                }
            }
        });
    }
    /**
     * @memberof Tilemap
     * @description Gets visible layers based on filter, then depth sorts the layers
     * @param {boolean} [filter=true]
     * @returns {TileLayer[]}
     * @since 2.0.0
     */
    sortTileLayers(filter = true) {
        const visibleLayers = this.tileLayers.filter((layer) => layer.visible === filter);
        const depthSortedLayers = [...visibleLayers].sort((layer, layer2) => layer.zIndex - layer2.zIndex);
        return depthSortedLayers;
    }
}

;// CONCATENATED MODULE: ./lib/base/once.js
/**
 * @class Once
 * @classdesc Creates a DuckEngine Once
 * @description The Once Class. A tool that can run a function once even if it is in a loop
 * @since 1.0.0
 */
class Once {
    /**
     * @constructor
     * @description Creates a Once instance.
     * @param {(...args: unknown[]) => unknown} func Function to run on Once.run
     * @param {boolean} [run] Determines if function is run as soon as Once class is constructed
     * @since 1.0.0
     */
    constructor(func, run) {
        this.func = func;
        this.ran = false;
        if (run) {
            this.run();
        }
    }
    /**
     * @memberof Once
     * @description Runs the function once
     * @since 1.0.0
     */
    run() {
        if (!this.ran) {
            this.func();
            this.ran = true;
        }
    }
    /**
     * @memberof Once
     * @description Resets the state and allows the function to be ran again if called
     * @since 1.0.0
     */
    reset() {
        this.ran = false;
    }
    /**
     * @memberof Once
     * @description Sets the ran state
     * @param {boolean} ran Ran state
     * @since 1.0.0
     */
    set state(ran) {
        this.ran = ran;
    }
}

;// CONCATENATED MODULE: ./lib/base/amount.js
/**
 * @class Amount
 * @classdesc Creates a DuckEngine Amount
 * @description The Amount Class. A tool that can run a function a max amount of times even if it is in a loop
 * @since 1.1.0
 */
class Amount {
    /**
     * @constructor
     * @description Creates an Amount instance.
     * @param {(currentCount: number) => void} func Callback function
     * @param {number} maxAmount Max amount of times the function can be called
     * @param {Game} game Game instance
     * @param {boolean} [run] Determines if function is run as soon as Amount class is constructed
     * @since 1.1.0
     */
    constructor(func, maxAmount, game, run) {
        this.func = func;
        this.reachedMaxAmount = false;
        this.game = game;
        this.currentCount = 0;
        this.maxAmount = maxAmount;
        if (run) {
            this.run();
        }
    }
    /**
     * @memberof Amount
     * @description Runs the function once if amount of times ran is not over the maxAmount
     * @since 1.1.0
     */
    run() {
        if (this.currentCount < this.maxAmount) {
            this.currentCount += 1;
            this.func(this.currentCount);
        }
        else {
            this.reachedMaxAmount = true;
        }
    }
    /**
     * @memberof Amount
     * @description Resets the amount of times ran
     * @since 1.1.0
     */
    reset() {
        this.currentCount = 0;
        this.reachedMaxAmount = false;
    }
    /**
     * @memberof Amount
     * @description Sets the amount of times ran
     * @param {number} currentCount Number of times ran
     * @since 1.1.0
     */
    set state(currentCount) {
        this.currentCount = currentCount;
    }
}

;// CONCATENATED MODULE: ./lib/core/effect/effect.js

/**
 * @class Effect
 * @classdesc Creates a DuckEngine Effect
 * @description The Effect Class. Store a particleEmitter instance and particles that can be run easily
 * @since 1.2.0
 */
class Effect {
    /**
     * @constructor Effect
     * @description Creates an Effect instance.
     * @param {Duck.Types.ParticleEmitter.Range} rangeX A range of 2 numbers to determine the new created particles x position
     * @param {Duck.Types.ParticleEmitter.Range} rangeY A range of 2 numbers to determine the new created particles y position
     * @param {ParticleEmitter} particleEmitter ParticleEmitter instance
     * @param {Game} game Game instance
     * @since 1.2.0
     */
    constructor(rangeX, rangeY, particleEmitter, game) {
        this.id = uniqueID();
        this.shape = 'effect';
        this.rangeX = rangeX;
        this.rangeY = rangeY;
        this.particleEmitter = particleEmitter;
        this.game = game;
        this.visible = true;
        this.zIndex = 2;
        this.culled = true;
        this.particleEmitter.rangeX = rangeX;
        this.particleEmitter.rangeY = rangeY;
        this.following;
        this.randomMaxOffset = 5;
    }
    /**
     * @memberof Effect
     * @description Makes the particleEmitter emit
     * @since 1.2.0
     */
    emit() {
        this.particleEmitter.emit();
    }
    /**
     * @memberof Effect
     * @description Stops the particleEmitter emitting
     * @since 1.2.0
     */
    stopEmit() {
        this.particleEmitter.stopEmit();
    }
    /**
     * @memberof Effect
     * @description Makes the particleEmitter emit for a duration
     * @param {number} ms Duration in milliseconds
     * @since 1.2.0
     */
    emitFor(ms) {
        this.particleEmitter.emitFor(ms);
    }
    /**
     * @description Draws the effect.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.following) {
            const rangeX = [
                this.following.position.x,
                this.following.position.x + this.randomMaxOffset,
            ];
            const rangeY = [
                this.following.position.y,
                this.following.position.y + this.randomMaxOffset,
            ];
            this.particleEmitter.setRange(rangeX, rangeY);
        }
    }
    /**
     * @memberof Effect
     * @description Makes the effect follow the GameObject
     * @param {Duck.Types.GameObject<Duck.Types.Texture.Type>} object GameObject to follow
     * @param {number} [randomMaxOffset] A random offset of the effect relative to the GameObject
     * @since 1.2.0
     */
    follow(object, randomMaxOffset = 5) {
        this.following = object;
        this.randomMaxOffset = randomMaxOffset;
    }
    /**
     * @memberof Effect
     * @description Makes the effect stop following the GameObject
     * @since 1.2.0
     */
    stopFollow() {
        this.following = undefined;
    }
}

;// CONCATENATED MODULE: ./lib/core/effect/preset/explosion.js




class ExplosionEffect extends Effect {
    constructor(rangeX, rangeY, game, particleAmount = 50, speedRange = [450, 450], maxAge = 3, color = '#FFA500', scene) {
        const particle = new Particle('circle', 0, 0, 5, color, game, scene);
        particle.visible = false; // for now, ParticleEmitter sets it to true when emits
        const particleEmitter = new ParticleEmitter(particle, rangeX, rangeY, particleAmount, game, scene, false);
        super(rangeX, rangeY, particleEmitter, game);
        this.maxAge = maxAge;
        this.speedRange = speedRange;
    }
    /**
     * @memberof Effect
     * @description Makes the particleEmitter emit
     * @since 1.2.0
     */
    emit() {
        const min = randomFloat(-this.speedRange[0], -this.speedRange[1]);
        const max = randomFloat(this.speedRange[0], this.speedRange[1]);
        this.particleEmitter.create();
        this.particleEmitter.float([min, max], [min, max]);
        this.particleEmitter.emit();
    }
    /**
     * @memberof Effect
     * @description Stops the particleEmitter emitting
     * @since 1.2.0
     */
    stopEmit() {
        this.particleEmitter.stopEmit();
    }
    /**
     * @memberof Effect
     * @description Makes the particleEmitter emit for a duration
     * @param {number} ms Duration in milliseconds
     * @since 1.2.0
     */
    emitFor(ms) {
        this.particleEmitter.emitFor(ms);
    }
    /**
     * @description Draws the effect.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        this.particleEmitter.offloadMaxAge(this.maxAge);
    }
}

;// CONCATENATED MODULE: ./lib/core/effect/preset/smoke.js



class SmokeEffect extends Effect {
    constructor(rangeX, rangeY, game, particleAmount = 50, speedRangeX = [-10, -40], speedRangeY = [-10, -40], maxAge = 20, color = '#2e2e2e', interval = 50, scene) {
        const particle = new Particle('circle', 0, 0, 5, color, game, scene);
        particle.visible = false;
        const particleEmitter = new ParticleEmitter(particle, rangeX, rangeY, particleAmount, game, scene, false);
        super(rangeX, rangeY, particleEmitter, game);
        this.maxAge = maxAge;
        this.speedRangeX = speedRangeX;
        this.speedRangeY = speedRangeY;
        this.interval = interval;
    }
    /**
     * @memberof Effect
     * @description Makes the particleEmitter emit
     * @since 1.2.0
     */
    emit() {
        this.particleEmitter.create();
        this.particleEmitter.float([this.speedRangeX[0], this.speedRangeX[0]], [this.speedRangeY[1], this.speedRangeY[1]]);
        this.particleEmitter.keepEmitting(this.interval);
        this.particleEmitter.emit();
    }
    /**
     * @memberof Effect
     * @description Stops the particleEmitter emitting
     * @since 1.2.0
     */
    stopEmit() {
        this.particleEmitter.stopEmit();
    }
    /**
     * @memberof Effect
     * @description Makes the particleEmitter emit for a duration
     * @param {number} ms Duration in milliseconds
     * @since 1.2.0
     */
    emitFor(ms) {
        this.particleEmitter.emitFor(ms);
    }
    /**
     * @description Draws the effect.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        this.particleEmitter.offloadMaxAge(this.maxAge);
    }
}

;// CONCATENATED MODULE: ./lib/core/models/displayList.js

/**
 * @class DisplayList
 * @classdesc Creates a DisplayList that manages Renderable objects
 * @description The DisplayList Class. Keeps track of all Renderable objects
 * @since 2.0.0
 */
class DisplayList {
    constructor(game) {
        this.list = [];
        this.game = game;
    }
    /**
     * @memberof DisplayList
     * @description Adds a renderableObject to the list
     * @param {Duck.Types.Renderable} renderableObject A renderableObject
     *
     * Note: this pools the game's renderer pipeline for instant updates
     *
     * @since 2.0.0
     */
    add(renderableObject) {
        this.list.push(renderableObject);
        // pool renderer pipeline for instant update
        this.game.renderer.pipeline.pool();
    }
    /**
     * @memberof DisplayList
     * @description Removes a renderableObject from the list
     * @param {Duck.Types.Renderable} renderableObject A renderableObject
     * @since 2.0.0
     */
    remove(renderableObject) {
        const f = this.list.find((r) => r.id === renderableObject.id);
        if (f) {
            this.list.splice(this.list.findIndex((r) => r.id === renderableObject.id), 1);
        }
        else {
            new debug.Error(`INTERNAL -> Cannot remove renderableObject from displayList with id "${renderableObject.id}". If you suspect this as a bug, please report it on github.`);
        }
    }
    /**
     * @memberof DisplayList
     * @description Sorts all renderableObject by zIndex
     * @returns {Duck.Types.Renderable[]}
     * @since 2.0.0
     */
    depthSort() {
        const sorted = [...this.list].sort((obj, obj2) => obj.zIndex - obj2.zIndex);
        return sorted;
    }
    /**
     * @memberof DisplayList
     * @description Filters all renderableObjects by its visible boolean property
     * @param {boolean} [filter=true] By what to filter. EX: true, filters all visible objects, optional -> default: true
     * @returns {Duck.Types.Renderable[]}
     * @since 2.0.0
     */
    visibilityFilter(filter = true) {
        const filtered = this.list.filter((r) => r.visible === filter);
        return filtered;
    }
    /**
     * @memberof DisplayList
     * @template t
     * @description Loops through each renderableObject and runs a callback with it passed
     * @param {(renderableObject: Duck.Types.Renderable, index: number) => t} cb Callback
     * @since 2.0.0
     */
    each(cb) {
        this.list.forEach(cb.bind(this));
    }
    /**
     * @memberof DisplayList
     * @description Removes an item from the list based on passed index
     * @param {number} index
     * @param {number} [amount]
     * @since 2.0.0
     */
    splice(index, amount) {
        return this.list.splice(index, amount);
    }
    /**
     * @memberof DisplayList
     * @description Gets the length of the list
     * @since 2.0.0
     */
    get length() {
        return this.list.length;
    }
    /**
     * @memberof DisplayList
     * @description Overwrites the list items with passed parameter
     * @param {Duck.Types.Renderable[]} listItems Renderable objects to overwrite with
     * @since 2.0.0
     */
    set listItems(listItems) {
        this.list = listItems;
    }
}

;// CONCATENATED MODULE: ./lib/core/gameobjects/misc/canvasModulate.js




/**
 * @class CanvasModulate
 * @classdesc Creates a DuckEngine CanvasModulate GameObject
 * @description The CanvasModulate Class. A large rectangle with a special zIndex to use as a background, works with lights
 * @extends GameObject
 * @since 2.0.0
 */
class CanvasModulate extends GameObject {
    /**
     * @constructor CanvasModulate
     * @description Creates a CanvasModulate instance.
     * @param {number} x X position
     * @param {number} y Y position
     * @param {number} w Width
     * @param {number} h Height
     * @param {string} fillColor Color to fill the CanvasModulate with
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     */
    constructor(x, y, w, h, fillColor, game, scene) {
        super('rect', x, y, w, h, 0, Texture.fromColor(fillColor, w, h), game, scene);
        this.zIndex = Duck.Layers.Rendering.zIndex.canvasModulate;
    }
    /**
     * @description Draws the CanvasModulate gameobject.
     *
     * DO NOT CALL MANUALLY, CALLED IN GAME LOOP USING SCENE.displayList
     *
     */
    _draw() {
        if (this.game.renderer.ctx) {
            this.game.renderer.drawRect(this.position.x, this.position.y, this.w, this.h, this.texture.texture);
        }
        else {
            new debug.Error('CanvasRenderingContext2D is undefined. HTMLCanvasElement is undefined.');
        }
    }
}

;// CONCATENATED MODULE: ./lib/core/physics/server/physicsServer.js
/**
 * @class PhysicsServer
 * @classdesc Creates a DuckEngine PhysicsServer
 * @description The PhysicsServer Class. Handles all Physics updates
 * @since 2.0.0
 */
class PhysicsServer {
    /**
     * @constructor PhysicsServer
     * @description Creates a PhysicsServer instance
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(game, scene) {
        this.game = game;
        this.scene = scene;
    }
    /**
     * @memberof PhysicsServer
     * @description Uses PhysicsServer.Scene.physicsList and filters all enabled PhysicsBodies and calls PhysicsBody._update,
     * PhysicsBody.hitbox._update, and PhysicsBody.collider._update
     *
     * *DO NOT CALL MANUALLY! CALLED IN SCENE.__TICK*
     *
     * @since 2.0.0
     */
    __tick() {
        this.scene.physicsList.enabledFilter(true).forEach((r) => {
            r._update();
            if (r.collider && r.collidesWith && r.hitbox) {
                r.hitbox._update(r);
                r.collider._update(r.hitbox, r.collidesWith);
            }
        });
    }
}

;// CONCATENATED MODULE: ./lib/core/models/physicsList.js

/**
 * @class PhysicsList
 * @classdesc Creates a PhysicsList that manages Physics objects
 * @description The PhysicsList Class. Keeps track of all Physics objects
 * @since 2.0.0
 */
class PhysicsList {
    constructor() {
        this.list = [];
    }
    /**
     * @memberof PhysicsList
     * @description Adds a physicsObject to the list
     * @param {Duck.Types.PhysicsProcessMember} physicsObject A physicsObject
     * @since 2.0.0
     */
    add(physicsObject) {
        this.list.push(physicsObject);
    }
    /**
     * @memberof PhysicsList
     * @description Removes a physicsObject from the list
     * @param {Duck.Types.PhysicsProcessMember} physicsObject A physicsObject
     * @since 2.0.0
     */
    remove(physicsObject) {
        const f = this.list.find((r) => r.id === physicsObject.id);
        if (f) {
            this.list.splice(this.list.findIndex((r) => r.id === physicsObject.id), 1);
        }
        else {
            new debug.Error(`INTERNAL -> Cannot remove physicsObject from physicsList with id "${physicsObject.id}". If you suspect this as a bug, please report it on github.`);
        }
    }
    /**
     * @memberof PhysicsList
     * @template t
     * @description Loops through each physicsObject and runs a callback with it passed
     * @param {(physicsObject: Duck.Types.PhysicsProcessMember, index: number) => t} cb Callback
     * @since 2.0.0
     */
    each(cb) {
        this.list.forEach(cb.bind(this));
    }
    enabledFilter(filter = true) {
        const filtered = this.list.filter((r) => r.enabled === filter);
        return filtered;
    }
    /**
     * @memberof PhysicsList
     * @description Removes an item from the list based on passed index
     * @param {number} index
     * @param {number} [amount]
     * @since 2.0.0
     */
    splice(index, amount) {
        return this.list.splice(index, amount);
    }
    /**
     * @memberof PhysicsList
     * @description Gets the length of the list
     * @since 2.0.0
     */
    get length() {
        return this.list.length;
    }
    /**
     * @memberof PhysicsList
     * @description Overwrites the list items with passed parameter
     * @param {Duck.Types.PhysicsProcessMember[]} listItems Renderable objects to overwrite with
     * @since 2.0.0
     */
    set listItems(listItems) {
        this.list = listItems;
    }
}

;// CONCATENATED MODULE: ./lib/core/physics/models/area.js


/**
 * @class Area
 * @classdesc Creates a DuckEngine Area
 * @description The Area Class. Detects Hitboxes entering and leaving an area
 * @extends PhysicsBody<'color'>
 * @since 2.0.0
 */
class Area extends PhysicsBody {
    /**
     * @constructor Area
     * @description Creates an Area instance
     * @param {number} x X position of the area
     * @param {number} y Y position of the area
     * @param {number} w Width of the area
     * @param {number} h Height of the area
     * @param {PhysicsBody<Duck.Types.Texture.Type>[]} collisionFilter An array of PhysicsBodies, with Hitboxes attached to them,
     * to check for when it leaves/enters the area
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(x, y, w, h, collisionFilter, game, scene) {
        const id = uniqueID();
        super('rect', id, x, y, w, h, 0, game, scene);
        this.collisionFilter = collisionFilter;
        this.bodies = [];
        this.physics.addHitbox(w, h);
    }
    /**
     * @memberof Area
     * @description Updates the Area, check for Hitbox intersections with the passed collisionFilter array of PhysicsBodies
     *
     * *DO NOT CALL MANUALLY! CALLED IN SCENE.PHYSICS SERVER.__tick*
     *
     * @since 2.0.0
     */
    _update() {
        this.collisionFilter.forEach((object) => {
            if (object.hitbox && this.hitbox) {
                // add body
                if (this.hitbox.intersectsWith(object.hitbox)) {
                    this.addBody(object);
                }
                // remove body
                if (!this.hitbox.intersectsWith(object.hitbox)) {
                    this.removeBody(object);
                }
            }
        });
    }
    addBody(physicsBody) {
        const exists = this.bodyIsInArea(physicsBody);
        if (!exists) {
            this.bodies.push(physicsBody);
            if (this.onBodyEnter) {
                this.onBodyEnter(physicsBody);
            }
        }
    }
    removeBody(physicsBody) {
        const exists = this.bodyIsInArea(physicsBody);
        if (exists) {
            this.bodies.splice(this.bodies.findIndex((object) => object.id === physicsBody.id), 1);
            if (this.onBodyLeave) {
                this.onBodyLeave(physicsBody);
            }
        }
    }
    /**
     * @memberof Area
     * @description Checks if the body is currently in the area
     * @param {PhysicsBody<Duck.Types.Texture.Type>} physicsBody PhysicsBody to check
     * @returns {boolean}
     */
    bodyIsInArea(physicsBody) {
        const f = this.bodies.find((object) => object.id === physicsBody.id);
        return f ? true : false;
    }
}

;// CONCATENATED MODULE: ./lib/core/map/tile.js


/**
 * @class Tile
 * @classdesc Creates a DuckEngine Tile
 * @description The Tile Class. Stores information about a Tile on a Tileset
 * @since 2.0.0
 */
class Tile {
    /**
     * @constructor Tile
     * @description Creates a Tile instance
     * @param {number} x X position of the Tile
     * @param {number} y Y position of the Tile
     * @param {number} w Width of the Tile
     * @param {number} h Height of the Tile
     */
    constructor(x, y, w, h) {
        this.id = uniqueID();
        this.position = new Vector2(x, y);
        this.w = w;
        this.h = h;
    }
}

;// CONCATENATED MODULE: ./lib/core/map/tileset.js

/**
 * @class Tileset
 * @classdesc Creates a DuckEngine Tileset
 * @description The Tileset Class. Stores information about a tileset image and splits it into tiles
 * @since 2.0.0
 */
class Tileset {
    /**
     * @constructor Tileset
     * @description Creates a Tileset instance
     * @param {string} textureKey Key to the tileset image texture
     * @param {number} tileW Width of each Tile
     * @param {number} tileH Height of each Tile
     * @param {number} rows Number of rows in the tileset image texture
     * @param {number} cols Number of columns in the tileset image texture
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @since 2.0.0
     */
    constructor(textureKey, tileW, tileH, rows, cols, game, scene) {
        this.texture = scene.loader.textureStack.find((v) => v.key === textureKey).value;
        this.tileW = tileW;
        this.tileH = tileH;
        this.rows = rows;
        this.cols = cols;
        this.game = game;
        this.scene = scene;
        this.tiles = [];
        this.generateTiles();
    }
    generateTiles() {
        for (let rowIndex = 0; rowIndex < this.rows; rowIndex++) {
            for (let colIndex = 0; colIndex < this.cols; colIndex++) {
                const tileX = colIndex * this.tileW;
                const tileY = rowIndex * this.tileH;
                this.tiles.push(this.createTile(tileX, tileY, this.tileW, this.tileH));
            }
        }
    }
    createTile(x, y, w, h) {
        return new Tile(x, y, w, h);
    }
    /**
     * @memberof Tileset
     * @description Gets the Tile based on index
     * @param {number} num Index
     * @since 2.0.0
     */
    getTile(num) {
        return this.tiles[num];
    }
}

;// CONCATENATED MODULE: ./lib/core/map/tilelayer.js
/**
 * @class TileLayer
 * @classdesc Creates a DuckEngine TileLayer
 * @description The TileLayer Class. Stores a 2D array that relates to a Tileset that is then used in a TileMap
 * @since 1.0.0-beta
 */
class TileLayer {
    /**
     * @param {Tileset} tileset Tileset that the 2D array relates to
     * @param {number[][]} map A 2D array of numbers that relates to the Tileset
     * @param {Game} game Game instance
     * @param {Scene} scene Scene instance
     * @param {number} [zIndex=2] zIndex, optional -> defaults: 2
     * @param {boolean} [visible=true] visible, optional -> defaults: true
     */
    constructor(tileset, map, game, scene, zIndex = 2, visible = true) {
        this.tileset = tileset;
        this.map = map;
        this.game = game;
        this.scene = scene;
        this.zIndex = zIndex;
        this.visible = visible;
    }
}

;// CONCATENATED MODULE: ./lib/core/scene.js


















































/**
 * @class Scene
 * @classdesc Creates a DuckEngine Scene
 * @description The Scene Class. Main rendering happens here
 * @since 1.0.0-beta
 */
class Scene extends Render {
    /**
     * @constructor Scene
     * @description Creates a Scene instance
     * @param {string} key Key/Identifier or name of scene
     * @param {Game} game Game instance
     * @param {boolean} [visible=false] Is the scene visible, defaults to false
     * @since 1.0.0-beta
     */
    constructor(key, game, visible) {
        var _a;
        super();
        this.key = key;
        this.game = game;
        this.visible = visible || false;
        this.default = false;
        // set visible if key is same as defaultScene key
        if (this.game.stack.defaultScene === this.key) {
            this.default = true;
            this.visible = true;
        }
        // main camera
        this.currentCamera;
        this.mainCamera;
        this.cameras = [];
        this.displayList = new DisplayList(this.game);
        this.physicsList = new PhysicsList();
        this.loader = new Loader(this.game, this);
        if ((_a = this.game.config.physics) === null || _a === void 0 ? void 0 : _a.enabled) {
            this.physicsServer = new PhysicsServer(this.game, this);
        }
        this.onSceneActive = () => {
            // On scene change to visible
        };
        this.onSceneInactive = () => {
            // On scene change to not visible
        };
        // methods
        this.add = {
            gameobject: {
                misc: {
                    canvasModulate: (x, y, w, h, fillColor) => {
                        const myCanvasModulate = new CanvasModulate(x, y, w, h, fillColor, this.game, this);
                        this.displayList.add(myCanvasModulate);
                        this.physicsList.add(myCanvasModulate);
                        return myCanvasModulate;
                    },
                },
                existing: (gameobject) => {
                    this.displayList.add(gameobject);
                    this.physicsList.add(gameobject);
                    return gameobject;
                },
                sprite: (x, y, w, h, textureKey, currentRow, currentCol) => {
                    const sprite = new Sprite(x, y, w, h, textureKey, this.game, this, currentRow, currentCol);
                    this.displayList.add(sprite);
                    this.physicsList.add(sprite);
                    return sprite;
                },
                rect: (x, y, w, h, fillColor) => {
                    const rect = new Rect(x, y, w, h, fillColor, this.game, this);
                    this.displayList.add(rect);
                    this.physicsList.add(rect);
                    return rect;
                },
                circle: (x, y, r, fillColor) => {
                    const circle = new Circle(x, y, r, fillColor, this.game, this);
                    this.displayList.add(circle);
                    this.physicsList.add(circle);
                    return circle;
                },
                roundRect: (x, y, w, h, r, fillColor) => {
                    const roundRect = new RoundRect(x, y, w, h, r, fillColor, this.game, this);
                    this.displayList.add(roundRect);
                    this.physicsList.add(roundRect);
                    return roundRect;
                },
            },
            misc: {
                area: (x, y, w, h, collisionFilter) => {
                    const myArea = new Area(x, y, w, h, collisionFilter, this.game, this);
                    this.physicsList.add(myArea);
                    return myArea;
                },
            },
            ui: {
                text: (text, config) => {
                    const myText = new Text(text, config, this.game, this);
                    this.displayList.add(myText);
                    this.physicsList.add(myText);
                    return myText;
                },
                button: (shape, x, y, w, h, r, fillColor, text) => {
                    const myButton = new Button(shape, x, y, w, h, r, fillColor, text, this.game, this);
                    this.displayList.add(myButton);
                    this.physicsList.add(myButton);
                    return myButton;
                },
            },
            soundPlayer: (path, options) => {
                return new SoundPlayer(path, options);
            },
            input: () => {
                return new input_Input(this.game, this);
            },
            camera: () => {
                const c = new Camera(this.game, this);
                this.cameras.push(c);
                return c;
            },
            mainCamera: () => {
                const c = new Camera(this.game, this);
                this.cameras.push(c);
                this.mainCamera = c;
                this.currentCamera = c;
                return c;
            },
            light: {
                staticLight: (x, y, r, fillColor, alpha) => {
                    const myStaticLight = new StaticLight(x, y, r, fillColor, alpha, this.game, this);
                    this.displayList.add(myStaticLight);
                    this.physicsList.add(myStaticLight);
                    return myStaticLight;
                },
            },
            group: (name, defaultValues) => {
                return new Group(name, this.game, defaultValues);
            },
            particle: (shape, w, h, r, fillColor) => {
                const myParticle = new Particle(shape, w, h, r, fillColor, this.game, this);
                return myParticle;
            },
            particleEmitter: (particle, rangeX, rangeY, amount, autoCreate = true) => {
                return new ParticleEmitter(particle, rangeX, rangeY, amount, this.game, this, autoCreate);
            },
            cutscene: (config, instructions) => {
                return new Cutscene(config, instructions, this.game);
            },
            map: {
                tilemap: (origin, tilelayers) => {
                    const myTileMap = new TileMap(origin, tilelayers, this.game, this);
                    this.displayList.add(myTileMap);
                    return myTileMap;
                },
                tileset: (textureKey, tileW, tileH, rows, cols) => {
                    return new Tileset(textureKey, tileW, tileH, rows, cols, this.game, this);
                },
                tileLayer: (tileset, map, zIndex = 2, visible = true) => {
                    return new TileLayer(tileset, map, this.game, this, zIndex, visible);
                },
            },
            effect: (rangeX, rangeY, particleEmitter) => {
                const myEffect = new Effect(rangeX, rangeY, particleEmitter, this.game);
                this.displayList.add(myEffect);
                return myEffect;
            },
            presetEffect: {
                explosionEffect: (rangeX, rangeY, particleAmount = 50, speedRange = [1, 1], maxAge = 3, color = '#FFA500') => {
                    const myExplosionEffect = new ExplosionEffect(rangeX, rangeY, this.game, particleAmount, speedRange, maxAge, color, this);
                    this.displayList.add(myExplosionEffect);
                    return myExplosionEffect;
                },
                smokeEffect: (rangeX, rangeY, particleAmount = 50, speedRangeX = [-0.1, 0.4], speedRangeY = [-0.1, 0.4], maxAge = 20, color = '#2e2e2e', interval = 50) => {
                    const mySmokeEffect = new SmokeEffect(rangeX, rangeY, this.game, particleAmount, speedRangeX, speedRangeY, maxAge, color, interval, this);
                    this.displayList.add(mySmokeEffect);
                    return mySmokeEffect;
                },
            },
        };
        this.tools = {
            loader: Loader,
            color: {
                random: randomColor,
                randomWithAlpha: (alpha) => randomColorWithAlpha(alpha),
                is: {
                    hex: isHex,
                    hsl: isHSL,
                    rgb: isRGB,
                },
                convert: {
                    rgb: {
                        toHsl: rgbToHsl,
                        toRgba: rgbToRGBA,
                    },
                    rgba: {
                        toHsla: rgbaToHSLA,
                        toRgb: rgbaToRGB,
                    },
                    hex: {
                        toRgba: hexToRGBA,
                        toRgb: hexToRGB,
                        toHsl: hexToHSL,
                    },
                },
            },
            physics: {
                rectToRectIntersect: (rect, rect2) => {
                    return rectToRectIntersect(rect, rect2);
                },
                circleToRectIntersect: (circle, rect) => {
                    return circleRectCollision(circle, rect);
                },
            },
            math: {
                createVector: (x, y) => {
                    return Vector2.CREATE(x, y);
                },
                vector: Vector2,
                clamp: clamp,
                lerp: lerp,
                randomInt: randomInt,
                randomFloat: randomFloat,
            },
        };
    }
    /**
     * @memberof Scene
     * @description Calls physics server __tick method if game.config.physics.enabled is true
     * and game.config.physics.customTick is false
     * *Do not call manually, this is called in GAME LOOP*
     *
     * @since 2.0.0
     */
    __tick() {
        var _a, _b;
        if (!((_a = this.game.config.physics) === null || _a === void 0 ? void 0 : _a.customTick)) {
            (_b = this.physicsServer) === null || _b === void 0 ? void 0 : _b.__tick();
        }
    }
    /**
     * @memberof Scene
     * @description Sets the visible property and calls the game.renderer.pipeline.pool method to immediately update the visibility
     *
     * **Note: this calls Game.renderer.pipeline.pool to immediately update the visibility**
     *
     * @param {boolean} visible What to set the visible property to
     * @since 2.1.0
     */
    setVisible(visible) {
        this.visible = visible;
        this.game.renderer.pipeline.pool();
    }
    /**
     * @memberof Scene
     * @description Switches the active camera to a passed camera
     * @param {Camera} camera Camera to switch to
     * @memberof 1.0.0-beta
     */
    switchCamera(camera) {
        const foundCamera = this.cameras.find((_camera) => _camera === camera);
        if (foundCamera) {
            this.currentCamera = foundCamera;
            if (this.game.config.debug) {
                new debug.Log('Switched cameras.');
            }
        }
        else {
            new debug.Error('Cannot switch camera. Camera not found in the current scene.');
        }
    }
    /**
     * @memberof Scene
     * @description Switches the active camera to the main camera
     * @since 1.0.0-beta
     */
    switchToMainCamera() {
        this.currentCamera = this.mainCamera;
        if (this.game.config.debug) {
            new debug.Log('Switched to main camera.');
        }
    }
    /**
     * @memberof Scene
     * @description Sets a camera as the main camera
     * @param {Camera} camera Camera to set the main camera as
     * @since 1.0.0-beta
     */
    setMainCamera(camera) {
        this.mainCamera = camera;
        if (this.game.config.debug) {
            new debug.Log(`Set main camera to ${camera}.`);
        }
    }
    /**
     * @memberof Scene
     * @description Runs a function once no matter if it is in a loop or not
     * @param {(...args: unknown[]) => unknown} func Function to run
     * @param {boolean} [run] Determines if function is ran right when it is initialized
     * @returns {Once}
     * @since 1.0.0
     */
    once(func, run) {
        const one = new Once(func, run);
        return one;
    }
    /**
     * @memberof Scene
     * @description Allows a function to only be ran a max amount of times
     * @param {(currentCount:number) => void} func Function to call
     * @param {number} maxAmount Max amount of times to allow the function to be called
     * @param {boolean} [run] Determines if function is ran right when it is initialized
     * @returns {Amount}
     * @since 1.1.0
     */
    runAmount(func, maxAmount, run) {
        const amount = new Amount(func, maxAmount, this.game, run);
        return amount;
    }
    /**
     * @memberof Scene
     * @description Creates and returns a Timer instance
     * @param {number} ms Milliseconds, converted to seconds which is used to check if the target time is reached in Timer.count
     * @param {(...args:unknown[])=>unknown} cb Callback to call every time the timer reaches its target
     * @param {unknown[]} args Arguments to pass to the callback
     * @param {number} repeat Amount of times to repeat, set to infinity to repeat forever
     * @returns {Timer}
     * @since 2.1.0
     */
    createTimer(ms, cb, args, repeat) {
        const timer = new Timer(ms, cb, args, repeat);
        return timer;
    }
}

;// CONCATENATED MODULE: ./lib/index.js
/* eslint-disable @typescript-eslint/ban-types */
// import















































// main
// spec
/**
 * @namespace Duck
 * @description All Types, Type Classes, Classes, Layers, and Configurations are stored here.
 * @since 1.0.0-beta
 */
var Duck;
(function (Duck) {
    /**
     * @memberof Duck
     * @description Returns a HTMLCanvasElement and CanvasRenderingContext2D, finds a canvas, if none exist,
     * it creates and appends one
     * @returns {{canvas: HTMLCanvasElement, ctx: CanvasRenderingContext2D}}
     */
    Duck.AutoCanvas = () => {
        let canvas = document.querySelector('canvas');
        let ctx = canvas.getContext('2d');
        // check if canvas exists on document
        if (document.querySelector('canvas')) {
            canvas = document.querySelector('canvas');
            ctx = canvas.getContext('2d');
        }
        else {
            canvas = document.createElement('canvas');
            document.body.appendChild(canvas);
            ctx = canvas.getContext('2d');
        }
        return {
            canvas,
            ctx,
        };
    };
    /**
     * @namespace Duck.Classes
     * @memberof Duck
     * @description All classes are stored here so that it can be extended.
     * @since 2.0.0
     */
    let Classes;
    (function (Classes) {
        Classes.Game = Game;
        Classes.Scene = Scene;
        let GameObjects;
        (function (GameObjects) {
            GameObjects.GameObject = GameObject;
            GameObjects.Circle = Circle;
            GameObjects.Rect = Rect;
            GameObjects.RoundRect = RoundRect;
            GameObjects.Sprite = Sprite;
            let Particles;
            (function (Particles) {
                Particles.Particle = Particle;
                Particles.ParticleEmitter = ParticleEmitter;
            })(Particles = GameObjects.Particles || (GameObjects.Particles = {}));
            let UI;
            (function (UI) {
                UI.Button = Button;
                UI.Text = Text;
            })(UI = GameObjects.UI || (GameObjects.UI = {}));
            let Misc;
            (function (Misc) {
                Misc.CanvasModulate = CanvasModulate;
            })(Misc = GameObjects.Misc || (GameObjects.Misc = {}));
        })(GameObjects = Classes.GameObjects || (Classes.GameObjects = {}));
        let Effects;
        (function (Effects) {
            Effects.Effect = Effect;
            let Presets;
            (function (Presets) {
                Presets.ExplosionEffect = ExplosionEffect;
                Presets.SmokeEffect = SmokeEffect;
            })(Presets = Effects.Presets || (Effects.Presets = {}));
        })(Effects = Classes.Effects || (Classes.Effects = {}));
        let Misc;
        (function (Misc) {
            Misc.Loader = Loader;
            Misc.Group = Group;
            Misc.Cutscene = Cutscene;
            Misc.CacheManager = CacheManager;
            Misc.PluginManager = PluginManager;
        })(Misc = Classes.Misc || (Classes.Misc = {}));
        let Base;
        (function (Base) {
            Base.Amount = Amount;
            Base.Once = Once;
            Base.Render = Render;
        })(Base = Classes.Base || (Classes.Base = {}));
        let Sound;
        (function (Sound) {
            Sound.SoundPlayer = SoundPlayer;
        })(Sound = Classes.Sound || (Classes.Sound = {}));
        let Cameras;
        (function (Cameras) {
            Cameras.Camera = Camera;
        })(Cameras = Classes.Cameras || (Classes.Cameras = {}));
        let Physics;
        (function (Physics) {
            Physics.Collider = Collider;
            Physics.PhysicsServer = PhysicsServer;
            Physics.PhysicsBody = PhysicsBody;
            let Models;
            (function (Models) {
                Models.Hitbox = Hitbox;
                Models.Area = Area;
            })(Models = Physics.Models || (Physics.Models = {}));
        })(Physics = Classes.Physics || (Classes.Physics = {}));
        let Models;
        (function (Models) {
            Models.DisplayList = DisplayList;
            Models.PhysicsList = PhysicsList;
            Models.Texture = Texture;
        })(Models = Classes.Models || (Classes.Models = {}));
        let Map;
        (function (Map_1) {
            Map_1.Map = map_Map;
            Map_1.TileMap = TileMap;
        })(Map = Classes.Map || (Classes.Map = {}));
        let Math;
        (function (Math) {
            Math.Vector2 = Vector2;
        })(Math = Classes.Math || (Classes.Math = {}));
        let Lights;
        (function (Lights) {
            Lights.StaticLight = StaticLight;
        })(Lights = Classes.Lights || (Classes.Lights = {}));
        let Input;
        (function (Input_1) {
            Input_1.Input = input_Input;
            Input_1.KeyboardInput = KeyboardInput;
            Input_1.MouseInput = MouseInput;
            let Models;
            (function (Models) {
                Models.Key = Key;
                Models.Mouse = Mouse;
            })(Models = Input_1.Models || (Input_1.Models = {}));
        })(Input = Classes.Input || (Classes.Input = {}));
        let Animation;
        (function (Animation_1) {
            Animation_1.Animation = animation_Animation;
            Animation_1.AnimationFrame = AnimationFrame;
            Animation_1.AnimationManager = AnimationManager;
            Animation_1.AnimationState = AnimationState;
            Animation_1.StateMachine = StateMachine;
        })(Animation = Classes.Animation || (Classes.Animation = {}));
    })(Classes = Duck.Classes || (Duck.Classes = {}));
    /**
     * @namespace Duck.Layers
     * @memberof Duck
     * @description All rendering zIndexes are stored here.
     * @since 2.0.0
     */
    let Layers;
    (function (Layers) {
        let Rendering;
        (function (Rendering) {
            Rendering.zIndex = {
                canvasModulate: 1,
                gameobject: 2,
                particle: 3,
                button: 4,
                text: 5,
                graphicDebug: 6,
                fades: 7,
            };
        })(Rendering = Layers.Rendering || (Layers.Rendering = {}));
    })(Layers = Duck.Layers || (Duck.Layers = {}));
    /**
     * @namespace Duck.Types
     * @memberof Duck
     * @description All Class configs and types for that class are stored here. All types are here.
     * @since 2.0.0
     */
    let Types;
    (function (Types) {
        let GamepadInput;
        (function (GamepadInput) {
            let XboxMapping;
            (function (XboxMapping) {
                XboxMapping[XboxMapping["A"] = 0] = "A";
                XboxMapping[XboxMapping["B"] = 1] = "B";
                XboxMapping[XboxMapping["X"] = 2] = "X";
                XboxMapping[XboxMapping["Y"] = 3] = "Y";
                XboxMapping[XboxMapping["LB"] = 4] = "LB";
                XboxMapping[XboxMapping["RB"] = 5] = "RB";
                XboxMapping[XboxMapping["LT"] = 6] = "LT";
                XboxMapping[XboxMapping["RT"] = 7] = "RT";
                XboxMapping[XboxMapping["SHOW_ADDRESS_BAR"] = 8] = "SHOW_ADDRESS_BAR";
                XboxMapping[XboxMapping["SHOW_MENU"] = 9] = "SHOW_MENU";
                XboxMapping[XboxMapping["LEFT_STICK_PRESS"] = 10] = "LEFT_STICK_PRESS";
                XboxMapping[XboxMapping["RIGHT_STICK_PRESS"] = 11] = "RIGHT_STICK_PRESS";
                XboxMapping[XboxMapping["D_PAD_UP"] = 12] = "D_PAD_UP";
                XboxMapping[XboxMapping["D_PAD_DOWN"] = 13] = "D_PAD_DOWN";
                XboxMapping[XboxMapping["D_PAD_LEFT"] = 14] = "D_PAD_LEFT";
                XboxMapping[XboxMapping["D_PAD_RIGHT"] = 15] = "D_PAD_RIGHT";
                XboxMapping[XboxMapping["LOGO"] = 16] = "LOGO";
            })(XboxMapping = GamepadInput.XboxMapping || (GamepadInput.XboxMapping = {}));
            let PlaystationMapping;
            (function (PlaystationMapping) {
                PlaystationMapping[PlaystationMapping["X"] = 0] = "X";
                PlaystationMapping[PlaystationMapping["CIRCLE"] = 1] = "CIRCLE";
                PlaystationMapping[PlaystationMapping["SQUARE"] = 2] = "SQUARE";
                PlaystationMapping[PlaystationMapping["TRIANGLE"] = 3] = "TRIANGLE";
                PlaystationMapping[PlaystationMapping["L1"] = 4] = "L1";
                PlaystationMapping[PlaystationMapping["R1"] = 5] = "R1";
                PlaystationMapping[PlaystationMapping["L2"] = 6] = "L2";
                PlaystationMapping[PlaystationMapping["R2"] = 7] = "R2";
                PlaystationMapping[PlaystationMapping["SHARE"] = 8] = "SHARE";
                PlaystationMapping[PlaystationMapping["OPTIONS"] = 9] = "OPTIONS";
                PlaystationMapping[PlaystationMapping["LEFT_STICK_PRESS"] = 10] = "LEFT_STICK_PRESS";
                PlaystationMapping[PlaystationMapping["RIGHT_STICK_PRESS"] = 11] = "RIGHT_STICK_PRESS";
                PlaystationMapping[PlaystationMapping["D_PAD_UP"] = 12] = "D_PAD_UP";
                PlaystationMapping[PlaystationMapping["D_PAD_DOWN"] = 13] = "D_PAD_DOWN";
                PlaystationMapping[PlaystationMapping["D_PAD_LEFT"] = 14] = "D_PAD_LEFT";
                PlaystationMapping[PlaystationMapping["D_PAD_RIGHT"] = 15] = "D_PAD_RIGHT";
                PlaystationMapping[PlaystationMapping["LOGO"] = 16] = "LOGO";
            })(PlaystationMapping = GamepadInput.PlaystationMapping || (GamepadInput.PlaystationMapping = {}));
        })(GamepadInput = Types.GamepadInput || (Types.GamepadInput = {}));
    })(Types = Duck.Types || (Duck.Types = {}));
})(Duck || (Duck = {}));
// export
/**
 * @namespace
 * @property {Game} game Game Class
 * @property {Scene} scene Scene Class
 * @description Main Export of DuckEngine
 * @since 1.0.0-beta
 */
const DuckEngine = {
    Game: Game,
    Scene: Scene,
};
/* harmony default export */ const lib = (DuckEngine);

var __webpack_exports__Duck = __webpack_exports__._;
var __webpack_exports__default = __webpack_exports__.Z;
export { __webpack_exports__Duck as Duck, __webpack_exports__default as default };
